/*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License as
* published by the Free Software Foundation; either version 2 of
* the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* General Public License for more details at
* http://www.gnu.org/copyleft/gpl.html
*
* Author: Marius Stene
* Email: marius.stene at gmail dot com
*
*/

/*! @class CalculateFlowThread
 *  @brief Class for doing the actual calculations, inherits QThread
 */

#include "calculateflowthread.h"

/** Constructor that creates the thread and readies for simulation
*   @param parent Pointer to parent (owner)
*   @param data Flow data
*   @param frictionFunctions Pointer to class with pointer to functions
*   @param calculateFunctions Pointer to class with pointers to calculate functions
*   @param useMinimumHoldupCriterion Should the calculation use the minimum holdup criterion or simply calculate for the selected regime
*   @param id Thread identification, used to keep track of it in CalculateManager
*   @param tolerance Numerical tolerance in calculation answer
*/
CalculateFlowThread::CalculateFlowThread(QWidget *parent, Data *data, FrictionFunctions *frictionFunctions, CalculateFunctions *calculateFunctions,
                                bool useMinimumHoldupCriterion, unsigned int id, double tolerance)
        :  _parent(parent), _data(data), _frictionFunctions(frictionFunctions), _regimeTransitionPlugin(NULL), _calculateFunctions(calculateFunctions),
        _useMinimumHoldupCriterion(useMinimumHoldupCriterion), _id(id), _tolerance(tolerance)
{
    qDebug() << "CalculateFlowThread::CalculateFlowThread";
    _dpdx = _alpha = _holdup = _diameterGas = 0;
}

CalculateFlowThread::CalculateFlowThread(QWidget *parent, Data *data, FrictionFunctions *frictionFunctions, CalculateFunctions *calculateFunctions,
                                         RegimeTransitionInterface *regimeTransitionPlugin, unsigned int id, double tolerance)
        :  _parent(parent), _data(data), _frictionFunctions(frictionFunctions), _regimeTransitionPlugin(regimeTransitionPlugin),
        _calculateFunctions(calculateFunctions), _id(id), _tolerance(tolerance), _useMinimumHoldupCriterion(false)
{
    qDebug() << "CalculateFlowThread::CalculateFlowThread";
    _dpdx = _alpha = _holdup = _diameterGas = 0;
}

/** Destructor, delete data and so on */
CalculateFlowThread::~CalculateFlowThread()
{
    qDebug() << "CalculateFlowThread::~CalculateFlowThread() ";
    delete _data;
}

/** Run thread, gets called from ::start() */
void CalculateFlowThread::run()
{
    qDebug() << "CalculateFlowThread::run()";

    //Regime transition calculation
    if (_regimeTransitionPlugin != NULL) {
        try
          {
            _y = _regimeTransitionPlugin->calculateRegimeTransition(_data, _frictionFunctions, _calculateFunctions, _tolerance, _data->Usg());
          }
          catch (PluginException e)
          {
            emit simulationError(e.what());
            _y = -1.0;
          }
        return;
    }
    //Normal flow calculation with minimum holdup criterion
    //Add regimes to calculate with to vector
    if (_useMinimumHoldupCriterion) {
        std::vector<CalculateRegimeInterface *> plugins;
        if (_calculateFunctions->isEnabledAnnular())
            plugins.push_back(_calculateFunctions->pluginAnnular());
        if (_calculateFunctions->isEnabledDispersed())
            plugins.push_back(_calculateFunctions->pluginDispersed());
        if (_calculateFunctions->isEnabledStratified())
            plugins.push_back(_calculateFunctions->pluginStratified());
        if (_calculateFunctions->isEnabledSlug())
            plugins.push_back(_calculateFunctions->pluginSlug());

        if (plugins.size() == 0)
            return;

        _alpha = -10.0;
        double tempAlpha;
        CalculateRegimeInterface *tempPlugin;
        CalculateRegimeInterface *plugin;
        for (int i = 0; i < plugins.size(); i++) {
            tempPlugin = plugins.at(i);
            tempAlpha = tempPlugin->calculateAlpha(_data, _frictionFunctions, _tolerance);
            if (tempAlpha > _alpha) {
                _alpha = tempAlpha;
                plugin = tempPlugin;
            }
        }
        _holdup = 1-_alpha;
        _data->setAlpha(_alpha);
        _dpdx = plugin->calculatePressureDrop(_data, _frictionFunctions, _tolerance);
    } else {
        //Normal flow calculation without minimum holdup
        switch (_data->regimeType()) {
        case REGIME_ANNULAR: {
            qDebug() << "case REGIME_ANNULAR:";
            //Annular flow
            if (!_calculateFunctions->isDefined())
                return;
            _alpha = _calculateFunctions->pluginAnnular()->calculateAlpha(_data, _frictionFunctions, _tolerance);
            _holdup = 1-_alpha;
            _data->setAlpha(_alpha);
            _dpdx = _calculateFunctions->pluginAnnular()->calculatePressureDrop(_data, _frictionFunctions, _tolerance);
        }
        break;
        case REGIME_DISPERSED: {
            qDebug() << "case REGIME_DISPERSED:";
            //Dispersed flow
            if (!_calculateFunctions->isDefined())
                return;
            _alpha = _calculateFunctions->pluginDispersed()->calculateAlpha(_data, _frictionFunctions, _tolerance);
            _holdup = 1-_alpha;
            _data->setAlpha(_alpha);
            _dpdx = _calculateFunctions->pluginDispersed()->calculatePressureDrop(_data, _frictionFunctions, _tolerance);
        }
        break;
        case REGIME_STRATIFIED: {
            qDebug() << "case REGIME_STRATIFIED:";
            //Statified flow
            if (!_calculateFunctions->isDefined())
                return;
            _alpha = _calculateFunctions->pluginStratified()->calculateAlpha(_data, _frictionFunctions, _tolerance);
            _holdup = 1-_alpha;
            _data->setAlpha(_alpha);
            _dpdx = _calculateFunctions->pluginStratified()->calculatePressureDrop(_data, _frictionFunctions, _tolerance);
        }
        break;
        case REGIME_SLUG: {
            qDebug() << "case REGIME_SLUG:";
            //Slug flow
            if (!_calculateFunctions->isDefined())
                return;
            _alpha = _calculateFunctions->pluginSlug()->calculateAlpha(_data, _frictionFunctions, _tolerance);
            _holdup = 1-_alpha;
            _data->setAlpha(_alpha);
            _dpdx = _calculateFunctions->pluginSlug()->calculatePressureDrop(_data, _frictionFunctions, _tolerance);
        }
        break;
        default:
            qDebug() << "ERROR, INCORRECT REGIME TYPE";
            break;
        }
    }

    qDebug() << "CalculateFlowThread::run() finished";
}


/** Returns hydraulic diameters for all variables */
double CalculateFlowThread::diameterGas()
{
    qDebug() << "CalculateFlowThread::diameterGas()";
    return _diameterGas;
}

/** Returns x value */
double CalculateFlowThread::x()
{
    qDebug() << "CalculateFlowThread::x()";
    return _data->variable();
}

/** Returns y value */
double CalculateFlowThread::y()
{
    qDebug() << "CalculateFlowThread::y()";
    return _y;
}

/** Returns holdup */
double CalculateFlowThread::holdup()
{
    qDebug() << "CalculateFlowThread::holdup()";
    return _holdup;
}

/** Returns alpha */
double CalculateFlowThread::alpha()
{
    qDebug() << "CalculateFlowThread::alpha()";
    return _alpha;
}

/** Returns  pressure loss */
double CalculateFlowThread::dpdx()
{
    qDebug() << "CalculateFlowThread::dpdx()";
    return _dpdx;
}
