/*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License as
* published by the Free Software Foundation; either version 2 of
* the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* General Public License for more details at
* http://www.gnu.org/copyleft/gpl.html
*
* Author: Marius Stene
* Email: marius.stene at gmail dot com
*
*/

/*! @class CalculateManager
 *  @brief Class which splits calculation tasks over several threads for
 *  processing and handles the gathering of results
 */

#include "calculatemanager.h"

/** Constructor that creates the all calculation thread and readies for simulation
*   @param parent Pointer to parent (owner)
*   @param simulationId Identification number of simulation
*   @param data Flow data
*   @param frictionFunctions Pointer to class with pointer to friction functions
*   @param calculateFunctions Pointer to class with pointers to calculate functions
*   @param useMinimumHoldupCriterion Should the calculation use the minimum holdup criterion or simply calculate for the selected regime
*   @param resolution How many points to calculate
*   @param threads Number of threads to run simultaneously
*   @param scaleType What type of scale_t for x value intervals to use
*   @param tolerance Numeric tolerance
*   @param calculationName Name of calculation, used for plotting
*/
CalculateManager::CalculateManager(QWidget *parent, int simulationId, Data *data, FrictionFunctions *frictionFunctions, CalculateFunctions *calculateFunctions,
                           bool useMinimumHoldupCriterion, unsigned int resolution, unsigned int threads, scale_t scaleType, double tolerance, QString calculationName) :
                          _parent(parent), _simulationId(simulationId), _data(data), _frictionFunctions(frictionFunctions), _calculateFunctions(calculateFunctions),
                         _regimeTransitionPlugin(NULL), _useMinimumHoldupCriterion(useMinimumHoldupCriterion), _resolution(resolution), _threads(threads), _scaleType(scaleType),
                         _tolerance(tolerance), _calculationName(calculationName)
{
    qDebug() << "CalculateManager::CalculateManager( )";
    _isFinished = false;
    _isRunning = false;
    _isTerminating = _isTerminated = false;
    _progress = 0;
    _pluginName = "";
    _xList.clear();

    //For angle as variable we force linear scaling
    if (_data->variableName().compare("Angle", Qt::CaseInsensitive) == 0 && _scaleType != SCALE_LIN) {
        QMessageBox::warning(NULL, tr("Unsupported x scaling"),
                             tr("With 'angle' as x-variable the only scaling method supported is 'linear', simulation will be run using linear x value scaling"),
                             QMessageBox::Ok);
        _scaleType = SCALE_LIN;
    }
    generateXList(_xList);

    //Iterate through x variable list and instansiate a new threads for each variable.
    //These threads will be started continously so that _threads threads are running when
    //run() has been called.
    for(unsigned int i = 0; i < _xList.size(); ++i) {
        Data *sendData = new Data(data);
        sendData->setVariable(_xList.at(i));
        _ctVector.push_back(new CalculateFlowThread(_parent, sendData, frictionFunctions, calculateFunctions, _useMinimumHoldupCriterion, i, _tolerance));
        connect(_ctVector.at(i), SIGNAL(finished()), this, SLOT(on_calculateFlowThreadFinished()));
        connect(_ctVector.at(i), SIGNAL(simulationError(QString)), this, SLOT(on_simulationError(QString)));
    }
}

CalculateManager::CalculateManager(QWidget *parent, int simulationId, Data *data, FrictionFunctions *frictionFunctions,
                                   RegimeTransitionInterface *regimeTransitionPlugin, CalculateFunctions *calculateFunctions, unsigned int resolution,
                 unsigned int threads, scale_t scaleType, double tolerance, QString calculationName)
                   :    _parent(parent), _simulationId(simulationId), _data(data), _frictionFunctions(frictionFunctions),
                        _calculateFunctions(calculateFunctions), _regimeTransitionPlugin(regimeTransitionPlugin),_useMinimumHoldupCriterion(false), _resolution(resolution),
                        _threads(threads),_scaleType(scaleType), _tolerance(tolerance), _calculationName(calculationName)
{
    qDebug() << "CalculateManager::CalculateManager()";
    _isFinished = false;
    _isRunning = false;
    _isTerminating = _isTerminated = false;
    _progress = 0;
    _pluginName = QString(regimeTransitionPlugin->pluginName()).replace("&", "");
    _xList.clear(); 

    generateXList(_xList);

    //Iterate through x variable list and instansiate a new threads for each variable.
    //These threads will be started continously so that _threads threads are running when
    //run() has been called.
    for(unsigned int i = 0; i < _xList.size(); ++i) {
        Data *sendData = new Data(data);
        sendData->setVariable(_xList.at(i));
        _ctVector.push_back(new CalculateFlowThread(_parent, sendData, frictionFunctions, calculateFunctions, _regimeTransitionPlugin, i, _tolerance));
        connect(_ctVector.at(i), SIGNAL(finished()), this, SLOT(on_calculateRegimeTransitionThreadFinished()));
        connect(_ctVector.at(i), SIGNAL(simulationError(QString)), this, SLOT(on_simulationError(QString)));
    }
}

void CalculateManager::generateXList(std::vector<double> &xList)
{
    //Get X values to calculate y values for, the interval between them are decided by the _scaleType enum variable
    if (_scaleType == SCALE_LIN) {
        //Linear x value interval
        double dist = _data->variableMax() - _data->variableMin();
        double diff = dist/_resolution;
        double i = _data->variableMin();

        while (i <= _data->variableMax()) {
            xList.push_back(i);
            i += diff;
        }
    } else {
        //Log or Exp
        //Calculate what to multiply x with every time
        double power = 1.0/_resolution;
        double di = pow(_data->variableMax()/_data->variableMin(), power);

        if (_scaleType == SCALE_EXP) {
            //Exponential x value interval
            //Create list of x values to use
            for (double i = _data->variableMin(); i < _data->variableMax(); ) {
                xList.push_back(i);
                i = i*di;
            }
        } else {
            //Logarithmic
            std::vector<double> xListBackwards;
            di = 1/di;
            //Fill up backwards
            for (double i = _data->variableMax(); i > _data->variableMin(); ) {
                xListBackwards.push_back(i);
                i = i*di;
            }
            //reverse
            for (int j = xListBackwards.size()-1; j >= 0; j--) {
                xList.push_back(xListBackwards.at(j));
            }
        }
    }
    if (xList.at(xList.size()-1) != _data->variableMax()) {
        xList.push_back(_data->variableMax());
    }
}

/** Destructor, delete CalculateFlowThread content */
CalculateManager::~CalculateManager()
{
    qDebug() << "CalculateManager::~CalculateManager()";
    for(unsigned int i = 0; i < _ctVector.size(); ++i) {
        delete _ctVector.at(i);
    }
}

/** Run calculation threads, gets called by ::start() */
void CalculateManager::run()
{
    qDebug() << "CalculateManager::run()";
    _isRunning = true;
    _finishedSimulations = 0;
    _simulationsStarted = _threads;

    //Start threads, run _threads threads at a time
    for (unsigned int i = 0; i < _threads; i++) {
        qDebug() << "Starting thread " << i << " of " << _threads;
        _ct = _ctVector.at(i);
        _ct->start(QThread::NormalPriority);
    }
    qDebug() << "Finished starting";
}

/** Quit */
void CalculateManager::quit()
{
    qDebug() << "CalculateManager::quit()";
    _isTerminating = true;
    for(unsigned int i = 0; i < _ctVector.size(); ++i) {
        _ctVector.at(i)->terminate();
    }
    _isTerminated = true;
    emit terminated();
    return;
}

/** A simulation thread has finished, check if it was the last thread,
*   if so, gather results and emit signal finished()
*/
void CalculateManager::on_calculateFlowThreadFinished()
{
    qDebug() << "CalculateManager::on_calculateFlowThreadFinished()";

    if (_isTerminating)
        return;

    _finishedSimulations++;
    _progress = 100*_finishedSimulations/_ctVector.size();
    emit simulationProgressed(_progress);

    //check if all threads are finished
    if (_finishedSimulations == _xList.size()) {
        _isRunning = false;
        _isFinished = true;
        _holdupList.clear();
        _alphaList.clear();
        _diameterGasList.clear();
        _dpdxList.clear();
        for(unsigned int i = 0; i < _ctVector.size(); i++) {
            _holdupList.push_back(_ctVector.at(i)->holdup());
            _alphaList.push_back(_ctVector.at(i)->alpha());
            _diameterGasList.push_back(_ctVector.at(i)->diameterGas());
            _dpdxList.push_back(_ctVector.at(i)->dpdx());
        }
        //Write to file
        /*
        QFile file("vector.txt");
        if (!file.open(QIODevice::WriteOnly | QIODevice::Text))
            return;

        QTextStream out(&file);
        std::vector<double>::const_iterator iter;
        for(iter = _xList.begin(); iter != _xList.end(); ++iter) {
            out << *iter << "\n";
        }
        for(iter = _holdupList.begin(); iter != _holdupList.end(); ++iter) {
            out << *iter << "\n";
        }
        for(iter = _dpdxList.begin(); iter != _dpdxList.end(); ++iter) {
            out << *iter << "\n";
        }
        for(iter = _alphaList.begin(); iter != _alphaList.end(); ++iter) {
            out << *iter << "\n";
        }
        for(iter = _diameterGasList.begin(); iter != _diameterGasList.end(); ++iter) {
            out << *iter << "\n";
        }
        file.close();
        */
        //Emit signal to parent
        emit finished();
    } else {
        //Not all simulations finished
        if (_simulationsStarted != _xList.size()) {
            //Not all simulations have been started, start a new one
            _ctVector.at(_simulationsStarted)->start(QThread::NormalPriority);
            _simulationsStarted++;
        }
    }
}

/** A simulation thread has finished, check if it was the last thread,
*   if so, gather results and emit signal finished()
*/
void CalculateManager::on_calculateRegimeTransitionThreadFinished()
{
    qDebug() << "CalculateManager::on_calculateRegimeTransitionThreadFinished()";

    if (_isTerminating)
        return;

    _finishedSimulations++;
    _progress = 100*_finishedSimulations/_ctVector.size();
    emit simulationProgressed(_progress);

    //check if all threads are finished
    if (_finishedSimulations == _xList.size()) {
        _isRunning = false;
        _isFinished = true;
        _yList.clear();
        for(unsigned int i = 0; i < _ctVector.size(); i++) {
            _yList.push_back(_ctVector.at(i)->y());
        }
        //Remove zeros from result
        std::vector<double> tempYList;
        std::vector<double> tempXList;
        bool msg = true;
        for (int i = 0; i < _yList.size(); i++) {
            //Check for incompatible regime calculation methods
            /*
            if (_yList.at(i) == -1 && msg) {
                QMessageBox::critical(NULL, tr("MultiPhasePlot"),
                                 tr("The regime calculation plugins used by '%1' does not support all needed calculation methods required").arg(_regimeTransitionPlugin->pluginName()),
                                 QMessageBox::Ok, QMessageBox::Ok);
                msg = false;
            }
            */
            if (_yList.at(i) > _tolerance) {
                tempXList.push_back(_xList.at(i));
                tempYList.push_back(_yList.at(i));
            }
        }
        _xList.swap(tempXList);
        _yList.swap(tempYList);
        //Emit signal to parent
        emit finished();
    } else {
        //Not all simulations finished
        if (_simulationsStarted != _xList.size()) {
            //Not all simulations have been started, start a new one
            _ctVector.at(_simulationsStarted)->start(QThread::NormalPriority);
            _simulationsStarted++;
        }
    }
}

/** Forward error message to mainWindow */
void CalculateManager::on_simulationError(QString errorMsg)
{
    if (!errorMsgs.contains(errorMsg)) {
        errorMsgs.append(errorMsg);
        emit simulationError(errorMsg);
    }
}

/** Returns vector of doubles containing hydraulic diameters for all variables */
std::vector<double> CalculateManager::diameterGasList()
{
    qDebug() << "CalculateManager::diameterGasList()";
    if (isFinished())
        return _diameterGasList;
    else {
        std::vector<double> asd;
        asd.push_back(0);
        return asd;
    }
}

/** Returns vector of doubles containing superficial velocities */
std::vector<double> CalculateManager::xList()
{
    qDebug() << "CalculateManager::xList()";
    if (isFinished())
        return _xList;
    else {
        std::vector<double> asd;
        asd.push_back(0);
        return asd;
    }
}

/** Returns vector of doubles containing y values*/
std::vector<double> CalculateManager::yList()
{
    qDebug() << "CalculateManager::yList()";
    if (isFinished())
        return _yList;
    else {
        std::vector<double> asd;
        asd.push_back(0);
        return asd;
    }
}

/** Returns vector of doubles containing holdup for all variables */
std::vector<double> CalculateManager::holdupList()
{
    qDebug() << "CalculateManager::holdupList()";
    if (isFinished())
        return _holdupList;
    else {
        std::vector<double> asd;
        asd.push_back(0);
        return asd;
    }
}

/** Returns vector of doubles containing alpha values for all variables */
std::vector<double> CalculateManager::alphaList()
{
    qDebug() << "CalculateManager::alphaList()";
    if (isFinished())
        return _alphaList;
    else {
        std::vector<double> asd;
        asd.push_back(0);
        return asd;
    }
}

/** Returns vector of doubles containing pressure loss values for all variables */
std::vector<double> CalculateManager::dpdxList()
{
    qDebug() << "CalculateManager::dpdxList()";

    if (isFinished()) {
        //The return data should always have positive values
        if (_dpdxList.size() > 0) {
            bool invert = false;
            int i = -1;
            //Check if first value that isnt 0 is positive or negative
            do {
                i++;
                if (_dpdxList.at(i) < 0.0) {
                    invert = true;
                }
            } while (_dpdxList.at(i) == 0.0 && _dpdxList.size() > i+1);

            //if first value was negative, return a vector *-1
            if (invert) {
                std::vector<double> invdpdx;
                for (int i = 0; i < _dpdxList.size(); i++) {
                    invdpdx.push_back(-_dpdxList.at(i));
                }
                return invdpdx;
            }
        }
        return _dpdxList;
    } else {
        std::vector<double> asd;
        asd.push_back(0);
        return asd;
    }
}

int CalculateManager::progress()
{
    return _progress;
}

QString CalculateManager::calculationName()
{
    return _calculationName;
}

QString CalculateManager::pluginName()
{
    return _pluginName;
}

/** Returns QString with name of the x variable, first letter capitalized */
QString CalculateManager::xVariableName()
{
    return _data->variableName();
}

/** Returns the minimum x variable value */
double CalculateManager::xVariableMin()
{
    if (_xList.size() > 0)
        return _xList.at(0);
    else
        return 0.0;
}

/** Returns the maximum x variable value */
double CalculateManager::xVariableMax()
{
    if(_xList.size() > 0)
        return _xList.at(_xList.size()-1);
    else
        return 0.0;
}
