/**
 * @file
 * @author  Marius Stene <marius.stene at gmail.com>
 * @version 1.0
 *
 * @section LICENSE
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details at
 * http://www.gnu.org/copyleft/gpl.html
 *
 * @section DESCRIPTION
 * Class which splits calculation tasks over several threads for
 * processing and handles the gathering of results
 */

#ifndef CALCULATEMANAGER_H
#define CALCULATEMANAGER_H

#include <QObject>
#include <QFile>
#include <QTextStream>
#include <QString>
#include <QVariant>
#include <QMessageBox>

#include <vector>

#include "calculateflowthread.h"
#include "../../Support/calculatefunctions.h"
#include "../../Support/data.h"
#include "../../Support/frictionfunctions.h"
#include "../../Support/calculateregimeinterface.h"
#include "defines.h"

class CalculateManager : public QObject
{
    Q_OBJECT
public:
    CalculateManager(QWidget *parent, int simulationId, Data *data, FrictionFunctions *frictionFunctions, CalculateFunctions *calculateFunctions, bool useMinimumHoldupCriterion, unsigned int resolution,
                     unsigned int threads = 1, scale_t scaleType = SCALE_LIN, double tolerance = 0.0000001, QString calculationName = "");
    CalculateManager(QWidget *parent, int simulationId, Data *data, FrictionFunctions *frictionFunctions, RegimeTransitionInterface *regimeTransitionPlugin,
                     CalculateFunctions *calculateFunctions, unsigned int resolution,
                     unsigned int threads = 1, scale_t scaleType = SCALE_LIN, double tolerance = 0.0000001, QString calculationName = "");
    ~CalculateManager();
    void run();
    void quit();
    /** Returns true if simulation is currently running */
    inline bool isRunning() { return _isRunning; };
    /** Returns true if simulation is finished */
    inline bool isFinished() { return _isFinished; };
    /** Returns true if simulation is/was terminated*/
    inline bool isTerminated() { return _isTerminated; };
    /** Returns simulation ID */
    inline int simulationID() { return _simulationId; };
    /** Returns const pointer to data used in simulation */
    inline const Data * data() { return _data; };

    std::vector<double> diameterGasList();
    std::vector<double> xList();
    std::vector<double> yList();
    std::vector<double> holdupList();
    std::vector<double> alphaList();
    std::vector<double> dpdxList();

    int progress();

    QString calculationName();
    QString pluginName();

    QString xVariableName();
    double xVariableMin();
    double xVariableMax();

private:
    void generateXList(std::vector<double> &xList);

    QWidget *_parent;
    int _simulationId;
    Data *_data;
    FrictionFunctions *_frictionFunctions;
    CalculateFunctions *_calculateFunctions;
    RegimeTransitionInterface *_regimeTransitionPlugin;
    bool _isFinished;
    bool _isTerminated;
    bool _isTerminating;
    bool _isRunning;
    bool _useMinimumHoldupCriterion;
    int _progress;
    std::vector<CalculateFlowThread *> _ctVector;
    CalculateFlowThread *_ct;
    int _nThreadsLeft;
    variable_t _variableWhat;
    unsigned int _resolution;
    unsigned int _threads;
    scale_t _scaleType;
    double _tolerance;
    QString _calculationName;
    QString _pluginName;
    QStringList errorMsgs;

    unsigned int _finishedSimulations;
    unsigned int _simulationsStarted;

    std::vector<double> _diameterGasList;
    std::vector<double> _xList;
    std::vector<double> _yList;
    std::vector<double> _holdupList;
    std::vector<double> _alphaList;
    std::vector<double> _dpdxList;

private slots:
    void on_calculateFlowThreadFinished();
    void on_calculateRegimeTransitionThreadFinished();
    void on_simulationError(QString errorMsg);

signals:
    void simulationProgressed(int percent);
    void simulationError(QString errorMsg);
    void finished();
    void terminated();
};

#endif // CALCULATEMANAGER_H
