/*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License as
* published by the Free Software Foundation; either version 2 of
* the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* General Public License for more details at
* http://www.gnu.org/copyleft/gpl.html
*
* Author: Marius Stene
* Email: marius.stene at gmail dot com
*
*/

/*! @class MainWindow
 *  @brief Window class for main window
 */

#include "mainwindow.h"
#include "ui_mainwindow.h"
#include "ui_listframe.h"
#include "pluginloader.h"

/** Constructor for MainWindow
    @param parent Parent Widget
*/
MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    qDebug() << "MainWindow::MainWindow()";
    ui->setupUi(this);

#if defined(Q_OS_LINUX)
    QApplication::setStyle(new QPlastiqueStyle);
#endif

    //Do some setup if this is the first run
    QSettings settings(COMPANYNAME, PRODUCTNAME);
    if (settings.value("FirstRun", true).toBool()) {
        settings.setValue("FirstRun", false);
        firstRunSetup();
    }

    //Add list frame and xyplot and splitter to flow tab
    flowSplitter = new QSplitter(this);
    flowSplitter->setOrientation(Qt::Vertical);
    flowSplitter->setChildrenCollapsible(false);
    ui->flowVerticalLayout->addWidget(flowSplitter);
    QTabWidget *tabWidgetFlow = new QTabWidget(this);
    flowSplitter->addWidget(tabWidgetFlow);
    QFrame *widget2 = new QFrame(this);
    uiFlowListFrame = new Ui::ListFrame;
    uiFlowListFrame->setupUi(widget2);
    flowSplitter->addWidget(widget2);
    flowSplitter->setStretchFactor(0, 6);
    flowSplitter->setStretchFactor(1, 2);
    flowSplitter->setStatusTip("Resize vertically plot and list");
    connect(uiFlowListFrame->checkBoxIncludeOldSimulations, SIGNAL(clicked()),
            this, SLOT(on_checkBoxIncludeOldSimulationsFlow_clicked()));
    connect(uiFlowListFrame->pushButtonMoreInfo, SIGNAL(clicked()),
            this, SLOT(on_pushButtonFlowMoreInfo_clicked()));
    connect(uiFlowListFrame->pushButtonPlotSimulations, SIGNAL(clicked()),
            this, SLOT(on_pushButtonPlotSimulations_clicked()));
    connect(uiFlowListFrame->pushButtonLoadSimulationParameters, SIGNAL(clicked()),
            this, SLOT(on_pushButtonLoadSimulationParametersFlow_clicked()));
    connect(uiFlowListFrame->pushButtonRemoveSelected, SIGNAL(clicked()),
            this, SLOT(on_pushButtonRemoveSelectedFlow_clicked()));
    connect(uiFlowListFrame->pushButtonClearSelection, SIGNAL(clicked()),
            this, SLOT(on_pushButtonClearSelectionFlow_clicked()));
    //Trick to get better looking QSplitters
    QStyle *style = flowSplitter->style();
    flowSplitter->setStyle(new QPlastiqueStyle);
    tabWidgetFlow->setStyle(style);
    widget2->setStyle(style);

    //Add list frame and xyplot and splitter to regime tab
    regimeSplitter = new QSplitter(this);
    regimeSplitter->setOrientation(Qt::Vertical);
    regimeSplitter->setChildrenCollapsible(false);
    ui->regimeHorizontalLayout->addWidget(regimeSplitter);
    regimePlot = new XYPlotArea(NULL, "Regime Transition", "Usg", "Usl");
    regimeSplitter->insertWidget(0, regimePlot);
    regimeSplitter->setStatusTip("Resize vertically plot and list");
    QFrame *widget = new QFrame(this);
    uiRegimeListFrame = new Ui::ListFrame;
    uiRegimeListFrame->setupUi(widget);
    regimeSplitter->addWidget(regimePlot);
    regimeSplitter->addWidget(widget);
    regimeSplitter->setStretchFactor(0, 6);
    regimeSplitter->setStretchFactor(1, 2);
    connect(uiRegimeListFrame->checkBoxIncludeOldSimulations, SIGNAL(clicked()),
            this, SLOT(on_checkBoxIncludeOldSimulationsRegime_clicked()));
    connect(uiRegimeListFrame->pushButtonMoreInfo, SIGNAL(clicked()),
            this, SLOT(on_pushButtonRegimeMoreInfo_clicked()));
    connect(uiRegimeListFrame->pushButtonPlotSimulations, SIGNAL(clicked()),
            this, SLOT(on_pushButtonPlotSimulationsRegime_clicked()));
    connect(uiRegimeListFrame->pushButtonRemoveSelected, SIGNAL(clicked()),
            this, SLOT(on_pushButtonRemoveSelectedRegime_clicked()));
    connect(uiRegimeListFrame->pushButtonLoadSimulationParameters, SIGNAL(clicked()),
            this, SLOT(on_pushButtonLoadSimulationParametersRegime_clicked()));
    connect(uiRegimeListFrame->pushButtonClearSelection, SIGNAL(clicked()),
            this, SLOT(on_pushButtonClearSelectionRegime_clicked()));

    //Trick to get better looking QSplitters
    style = regimeSplitter->style();
    regimeSplitter->setStyle(new QPlastiqueStyle);
    regimePlot->setStyle(style);
    widget->setStyle(style);

    errorMessage = new QErrorMessage(this);

    //Create models and add them to the simulation tree views
    QStringList qsl;
    qsl << "Id" << "Regime" << "Variable" << "Min" << "Max" << "Usg" << "Usl" << "Angle" << "Roughness" << "Diameter";
    QStringList qslToolTip;
    qslToolTip << "Simulation identification number" << "Regime type" << "Simulation variable" << "Variable minimum value" << "Variable maximum value";
    qslToolTip << "Superficial gas velocity" << "Superficial liquid velocity" << "Pipeline inclination (degrees)" << "Roughness" << "Diameter";
    simulationsFlowModel = new QStandardItemModel(0, qsl.size(), this);
    simulationsRegimeModel = new QStandardItemModel(0, qsl.size(), this);
    for (int i = 0; i < qsl.size(); i++) {
        simulationsFlowModel->setHeaderData(i, Qt::Horizontal, QObject::tr(QString(qsl.at(i)).toLatin1()));
        simulationsFlowModel->horizontalHeaderItem(i)->setToolTip(QObject::tr(QString(qslToolTip.at(i)).toLatin1()));
        simulationsFlowModel->horizontalHeaderItem(i)->setWhatsThis(QObject::tr(QString(qslToolTip.at(i)).toLatin1()));
    }
    //Regime simulation list needs some other columns
    qsl.replaceInStrings("Regime", "Transition Criterion");
    qslToolTip.replaceInStrings("Regime type", "Name of plugin used to calculate regime transition");
    for (int i = 0; i < qsl.size(); i++) {
        simulationsRegimeModel->setHeaderData(i, Qt::Horizontal, QObject::tr(QString(qsl.at(i)).toLatin1()));
        simulationsRegimeModel->horizontalHeaderItem(i)->setToolTip(QObject::tr(QString(qslToolTip.at(i)).toLatin1()));
        simulationsRegimeModel->horizontalHeaderItem(i)->setWhatsThis(QObject::tr(QString(qslToolTip.at(i)).toLatin1()));
    }

    //Craetor list models
    uiFlowListFrame->simulationsTreeView->setModel(simulationsFlowModel);
    simulationFlowItemSelectionModel = new QItemSelectionModel(simulationsFlowModel);
    connect(simulationFlowItemSelectionModel, SIGNAL(selectionChanged(QItemSelection,QItemSelection)),
            this, SLOT(on_simulationFlowItemSelectionModel_selectionChanged(QItemSelection,QItemSelection)));
    uiFlowListFrame->simulationsTreeView->setSelectionModel(simulationFlowItemSelectionModel);
    uiFlowListFrame->simulationsTreeView->setSortingEnabled(true);
    uiFlowListFrame->simulationsTreeView->header()->setResizeMode(QHeaderView::ResizeToContents);

    uiRegimeListFrame->simulationsTreeView->setModel(simulationsRegimeModel);
    simulationRegimeItemSelectionModel = new QItemSelectionModel(simulationsRegimeModel);
    connect(simulationRegimeItemSelectionModel, SIGNAL(selectionChanged(QItemSelection,QItemSelection)),
            this, SLOT(on_simulationRegimeItemSelectionModel_selectionChanged(QItemSelection,QItemSelection)));
    uiRegimeListFrame->simulationsTreeView->setSelectionModel(simulationRegimeItemSelectionModel);
    uiRegimeListFrame->simulationsTreeView->setSortingEnabled(true);
    uiRegimeListFrame->simulationsTreeView->header()->setResizeMode(QHeaderView::ResizeToContents);
    /*
    uiFlowListFrame->simulationsTreeView->setColumnWidth(0, 40);
    uiFlowListFrame->simulationsTreeView->setColumnWidth(1, 45);
    uiFlowListFrame->simulationsTreeView->setColumnWidth(2, 90);
    uiFlowListFrame->simulationsTreeView->setColumnWidth(7, 40);
    uiFlowListFrame->simulationsTreeView->setColumnWidth(8, 62);
    uiFlowListFrame->simulationsTreeView->setColumnWidth(9, 50);
    */

    //Make vectors of lineedits and radiobuttons
    _radioButtonsVariable.push_back(ui->radioButtonVariableUsg);
    _variableScaler.push_back(1);
    _radioButtonsVariable.push_back(ui->radioButtonVariableUsl);
    _variableScaler.push_back(1);
    _radioButtonsVariable.push_back(ui->radioButtonVariableLiquidDensity);
    _variableScaler.push_back(1);
    _radioButtonsVariable.push_back(ui->radioButtonVariableLiquidViscosity);
    _variableScaler.push_back(1);
    _radioButtonsVariable.push_back(ui->radioButtonVariableGasDensity);
    _variableScaler.push_back(1);
    _radioButtonsVariable.push_back(ui->radioButtonVariableGasViscosity);
    _variableScaler.push_back(1);
    _radioButtonsVariable.push_back(ui->radioButtonVariableGasTemperature);
    _variableScaler.push_back(1);
    _radioButtonsVariable.push_back(ui->radioButtonVariableGasPressure);
    _variableScaler.push_back(1);
    _radioButtonsVariable.push_back(ui->radioButtonVariableSurfaceTension);
    _variableScaler.push_back(1);
    _radioButtonsVariable.push_back(ui->radioButtonVariableAngle);
    _variableScaler.push_back(1);
    _radioButtonsVariable.push_back(ui->radioButtonVariableDiameter);
    _variableScaler.push_back(pow(10, -3));
    _radioButtonsVariable.push_back(ui->radioButtonVariableRoughness);
    _variableScaler.push_back(pow(10, -6));

    _lineEdits.push_back(ui->lineEditUsg);
    _lineEdits.push_back(ui->lineEditUsl);
    _lineEdits.push_back(ui->lineEditLiquidDensity);
    _lineEdits.push_back(ui->lineEditLiquidViscosity);
    _lineEdits.push_back(ui->lineEditGasDensity);
    _lineEdits.push_back(ui->lineEditGasViscosity);
    _lineEdits.push_back(ui->lineEditGasTemperature);
    _lineEdits.push_back(ui->lineEditGasPressure);
    _lineEdits.push_back(ui->lineEditSurfaceTension);
    _lineEdits.push_back(ui->lineEditAngle);
    _lineEdits.push_back(ui->lineEditDiameter);
    _lineEdits.push_back(ui->lineEditRoughness);

    _lineEditsMin.push_back(ui->lineEditUsgMin);
    _lineEditsMin.push_back(ui->lineEditUslMin);
    _lineEditsMin.push_back(ui->lineEditLiquidDensityMin);
    _lineEditsMin.push_back(ui->lineEditLiquidViscosityMin);
    _lineEditsMin.push_back(ui->lineEditGasDensityMin);
    _lineEditsMin.push_back(ui->lineEditGasViscosityMin);
    _lineEditsMin.push_back(ui->lineEditGasTemperatureMin);
    _lineEditsMin.push_back(ui->lineEditGasPressureMin);
    _lineEditsMin.push_back(ui->lineEditSurfaceTensionMin);
    _lineEditsMin.push_back(ui->lineEditAngleMin);
    _lineEditsMin.push_back(ui->lineEditDiameterMin);
    _lineEditsMin.push_back(ui->lineEditRoughnessMin);

    _lineEditsMax.push_back(ui->lineEditUsgMax);
    _lineEditsMax.push_back(ui->lineEditUslMax);
    _lineEditsMax.push_back(ui->lineEditLiquidDensityMax);
    _lineEditsMax.push_back(ui->lineEditLiquidViscosityMax);
    _lineEditsMax.push_back(ui->lineEditGasDensityMax);
    _lineEditsMax.push_back(ui->lineEditGasViscosityMax);
    _lineEditsMax.push_back(ui->lineEditGasTemperatureMax);
    _lineEditsMax.push_back(ui->lineEditGasPressureMax);
    _lineEditsMax.push_back(ui->lineEditSurfaceTensionMax);
    _lineEditsMax.push_back(ui->lineEditAngleMax);
    _lineEditsMax.push_back(ui->lineEditDiameterMax);
    _lineEditsMax.push_back(ui->lineEditRoughnessMax);

    _lineEditsAll.insert(_lineEditsAll.end(), _lineEdits.begin(), _lineEdits.end());
    _lineEditsAll.insert(_lineEditsAll.end(), _lineEditsMin.begin(), _lineEditsMin.end());
    _lineEditsAll.insert(_lineEditsAll.end(), _lineEditsMax.begin(), _lineEditsMax.end());

    _settingNamesLineEdits << "Usg" << "Usl" << "LiquidDensity" << "LiquidViscosity";
    _settingNamesLineEdits << "GasDensity" << "GasViscosity" << "GasTemperature" << "GasPressure" ;
    _settingNamesLineEdits << "SurfaceTension" << "Angle" << "Diameter" << "Roughness";

    _settingDefaultsLineEdits << "0.3" << "0.1" << "890" << "0.01";
    _settingDefaultsLineEdits << "1.1708" << "0.000018" << "298" << "1" ;
    _settingDefaultsLineEdits << "0.07" << "0.0" << "22" << "25";

    _settingDefaultsLineEditsMin << "0.03" << "0.02" << "800" << "0.001";
    _settingDefaultsLineEditsMin << "1.1708" << "0.00001" << "268" << "1" ;
    _settingDefaultsLineEditsMin << "0.005" << "-5.0" << "20" << "20";

    _settingDefaultsLineEditsMax << "1.6" << "1.6" << "1200" << "0.1";
    _settingDefaultsLineEditsMax << "10" << "0.5" << "500" << "6" ;
    _settingDefaultsLineEditsMax << "0.7" << "5.0" << "25" << "30";

    QString temp;
    for (int i = 0; i < _settingNamesLineEdits.size(); i++) {
        temp = _settingNamesLineEdits.at(i);
        temp.append("Min");
        _settingNamesLineEditsMin.append(temp);

        temp = _settingNamesLineEdits.at(i);
        temp.append("Max");
        _settingNamesLineEditsMax.append(temp);

        temp = _settingNamesLineEdits.at(i);
        temp.prepend("Variable");
        _settingNamesRadioBoxes.append(temp);
    }
    _settingNamesLineEditsAll.append(_settingNamesLineEdits);
    _settingNamesLineEditsAll.append(_settingNamesLineEditsMin);
    _settingNamesLineEditsAll.append(_settingNamesLineEditsMax);

    _settingDefaultsLineEditsAll.append(_settingDefaultsLineEdits);
    _settingDefaultsLineEditsAll.append(_settingDefaultsLineEditsMin);
    _settingDefaultsLineEditsAll.append(_settingDefaultsLineEditsMax);

    if (_lineEditsAll.size() != _settingNamesLineEditsAll.size() || _lineEditsAll.size() != _settingDefaultsLineEditsAll.size()) {
        qDebug() << "Line edits and settingnameslinesedits are not the same size!!";
        return;
    }

    _radioButtonsRegime.push_back(ui->radioButtonRegimeAnnular);
    _radioButtonsRegime.push_back(ui->radioButtonRegimeDispersed);
    _radioButtonsRegime.push_back(ui->radioButtonRegimeSlug);
    _radioButtonsRegime.push_back(ui->radioButtonRegimeStratified);

    //connect radio buttons to on_radioButtonVariable_clicked()
    for (unsigned int i = 0; i < _radioButtonsVariable.size(); i++) {
        connect(_radioButtonsVariable.at(i), SIGNAL(clicked()), this, SLOT(on_radioButtonVariable_clicked()));
    }

    //Make validators which make sure that the data inputed into lineedits are of the correct format
    ui->lineEditUsg->setValidator(new QDoubleValidator(0.0, 999.0, 8, this));
    ui->lineEditUsl->setValidator(new QDoubleValidator(0.0, 999.0, 8, this));
    ui->lineEditGasDensity->setValidator(new QDoubleValidator(0.0, 1000000.0, 8, this));
    ui->lineEditGasPressure->setValidator(new QDoubleValidator(0.0, 1000.0, 8, this));
    ui->lineEditGasTemperature->setValidator(new QDoubleValidator(0.0, 6000.0, 6, this));
    ui->lineEditGasViscosity->setValidator(new QDoubleValidator(0.0, 5000.0, 12, this));
    ui->lineEditLiquidDensity->setValidator(new QDoubleValidator(0.0, 20000.0, 8, this));
    ui->lineEditLiquidViscosity->setValidator(new QDoubleValidator(0.0, 20000.0, 12, this));
    ui->lineEditSurfaceTension->setValidator(new QDoubleValidator(0.0, 999.0, 12, this));
    ui->lineEditAngle->setValidator(new QDoubleValidator(-90.0, 999.0, 12, this));
    ui->lineEditDiameter->setValidator(new QDoubleValidator(0.0, 999.0, 12, this));
    ui->lineEditRoughness->setValidator(new QDoubleValidator(0.0, 999.0, 12, this));

    ui->lineEditUsgMin->setValidator(new QDoubleValidator(0.0, 999.0, 8, this));
    ui->lineEditUslMin->setValidator(new QDoubleValidator(0.0, 999.0, 8, this));
    ui->lineEditGasDensityMin->setValidator(new QDoubleValidator(0.0, 1000000.0, 8, this));
    ui->lineEditGasPressureMin->setValidator(new QDoubleValidator(0.0, 1000.0, 8, this));
    ui->lineEditGasTemperatureMin->setValidator(new QDoubleValidator(0.0, 6000.0, 6, this));
    ui->lineEditGasViscosityMin->setValidator(new QDoubleValidator(0.0, 5000.0, 12, this));
    ui->lineEditLiquidDensityMin->setValidator(new QDoubleValidator(0.0, 20000.0, 8, this));
    ui->lineEditLiquidViscosityMin->setValidator(new QDoubleValidator(0.0, 20000.0, 12, this));
    ui->lineEditSurfaceTensionMin->setValidator(new QDoubleValidator(0.0, 999.0, 12, this));
    ui->lineEditAngleMin->setValidator(new QDoubleValidator(-90.0, 90.0, 8, this));
    ui->lineEditDiameterMin->setValidator(new QDoubleValidator(0.0, 2000, 6, this));
    ui->lineEditRoughnessMin->setValidator(new QDoubleValidator(0.0, 5000, 6, this));

    ui->lineEditUsgMax->setValidator(new QDoubleValidator(0.0, 999.0, 8, this));
    ui->lineEditUslMax->setValidator(new QDoubleValidator(0.0, 999.0, 8, this));
    ui->lineEditGasDensityMax->setValidator(new QDoubleValidator(0.0, 1000000.0, 8, this));
    ui->lineEditGasPressureMax->setValidator(new QDoubleValidator(0.0, 1000.0, 8, this));
    ui->lineEditGasTemperatureMax->setValidator(new QDoubleValidator(0.0, 6000.0, 6, this));
    ui->lineEditGasViscosityMax->setValidator(new QDoubleValidator(0.0, 5000.0, 12, this));
    ui->lineEditLiquidDensityMax->setValidator(new QDoubleValidator(0.0, 20000.0, 8, this));
    ui->lineEditLiquidViscosityMax->setValidator(new QDoubleValidator(0.0, 20000.0, 12, this));
    ui->lineEditSurfaceTensionMax->setValidator(new QDoubleValidator(0.0, 999.0, 12, this));
    ui->lineEditAngleMax->setValidator(new QDoubleValidator(-90.0, 90.0, 8, this));
    ui->lineEditDiameterMax->setValidator(new QDoubleValidator(0.0, 2000, 6, this));
    ui->lineEditRoughnessMax->setValidator(new QDoubleValidator(0.0, 5000, 6, this));

    //Add XYPlots to flow tab
    QString plotTitle = "Holdup";
    holdupPlot =  new XYPlotArea(NULL, "Holdup", "xVariableName", "Holdup");
    tabWidgetFlow->addTab(holdupPlot, "Holdup");

    plotTitle = "Pressure Loss";
    pressurePlot = new XYPlotArea(NULL, "Pressure Loss", "xVariableName", "Pressure Loss (Pa/m)");
    tabWidgetFlow->addTab(pressurePlot, "Pressure Loss");

    plotTitle = "Alpha";
    alphaPlot = new XYPlotArea(NULL, "Alpha", "xVariableName", "Alpha");
    tabWidgetFlow->addTab(alphaPlot, "Alpha");

    //Make vector of XYPlots as well
    _XYPlotAreas.push_back(holdupPlot);
    _XYPlotAreas.push_back(pressurePlot);
    _XYPlotAreas.push_back(alphaPlot);

    //Load wall friction plugins
    QDir pluginDir(QDir::currentPath());
    pluginDir.cd("lib");
    pluginDir.cd("friction");
    PluginLoader<FrictionInterface *>::loadPlugins(pluginDir, frictionInterfacePlugins);
    pluginDir.cd("../interfacefriction");
    PluginLoader<InterfaceFrictionInterface *>::loadPlugins(pluginDir, interfaceFrictionInterfacePlugins);
    pluginDir.cd("../sliprelation");
    PluginLoader<SlipRelationInterface *>::loadPlugins(pluginDir, slipRelationInterfacePlugins);
    pluginDir.cd("../regimetransition");
    PluginLoader<RegimeTransitionInterface *>::loadPlugins(pluginDir, regimeInterfacePlugins);
    pluginDir.cd("../calculate");
    PluginLoader<CalculateRegimeInterface *>::loadPlugins(pluginDir, calculateInterfacePlugins);

    addPluginComboBoxItems();
    addPredefinesComboBoxItems(M_BOTH);
    addRegimeCheckBoxes();
    loadWidgetValues();
}

/** Destructor, calles function to save widget values before closing window */
MainWindow::~MainWindow()
{
    qDebug() << "MainWindow::~MainWindow()";
    saveWidgetValues();
    delete ui;
}

void MainWindow::changeEvent(QEvent *e)
{
    qDebug() << "MainWindow::changeEvent(QEvent *e)";
    QMainWindow::changeEvent(e);
    switch (e->type()) {
    case QEvent::LanguageChange:
        ui->retranslateUi(this);
        break;
    default:
        break;
    }
}

/** Returns pointer to FrictionFunctions class of friction functions for currently selected friction plugins
    @param frictionPlugins Vector of pointers pointing to all the loaded normal friction plugins
    @param interfaceFrictionPlugins Vector of pointers pointing to all the loaded interface friction plugins
    @param slipRelationFrictionPlugins Vector of pointers pointing to all the loaded slip relation plugins
*/
FrictionFunctions * MainWindow::getSelectedFrictionPlugins(std::vector<FrictionInterface *> frictionPlugins,
                                                           std::vector<InterfaceFrictionInterface *> interfaceFrictionPlugins,
                                                           std::vector<SlipRelationInterface *> slipRelationFrictionPlugins)
{
    qDebug() << "MainWindow::getSelectedFrictionPlugins()";
    FrictionFunctions *frictionFunctions = new FrictionFunctions();

    //Gas & Liquid wall friction
    QString pluginNameGasWall = ui->comboBoxGasWallFriction->currentText();
    QString pluginNameLiquidWall = ui->comboBoxLiquidWallFriction->currentText();
    QString pluginNameDispersed = ui->comboBoxDispersedFriction->currentText();

    std::vector<FrictionInterface *>::const_iterator iter;
    for(iter = frictionPlugins.begin(); iter != frictionPlugins.end(); ++iter) {
        qDebug() << (*iter)->pluginName();
        //Gas
        if (pluginNameGasWall.compare((*iter)->pluginName()) == 0) {
            frictionFunctions->setPluginWallGas(*iter);
        }
        //Liquid
        if (pluginNameLiquidWall.compare((*iter)->pluginName()) == 0) {
            frictionFunctions->setPluginWallLiquid(*iter);
        }
        //Dispersed
        if (pluginNameDispersed.compare((*iter)->pluginName()) == 0) {
            frictionFunctions->setPluginDispersed(*iter);
        }
    }

    //Interface friction
    QString pluginNameInterface = ui->comboBoxInterfaceFriction->currentText();
    std::vector<InterfaceFrictionInterface *>::const_iterator iter2;
    for(iter2 = interfaceFrictionPlugins.begin(); iter2 != interfaceFrictionPlugins.end(); ++iter2) {
        qDebug() << (*iter2)->pluginName();
        //Gas
        if (pluginNameInterface.compare((*iter2)->pluginName()) == 0) {
            frictionFunctions->setPluginInterface(*iter2);
        }
    }

    //Slip relation
    QString pluginNameSlipRelation = ui->comboBoxSlipRelation->currentText();
    std::vector<SlipRelationInterface *>::const_iterator iter3;
    for(iter3 = slipRelationFrictionPlugins.begin(); iter3 != slipRelationFrictionPlugins.end(); ++iter3) {
        qDebug() << (*iter3)->pluginName();
        //Gas
        if (pluginNameSlipRelation.compare((*iter3)->pluginName()) == 0) {
            frictionFunctions->setPluginSlipRelation(*iter3);
        }
    }

    return frictionFunctions;
}

/** Populates the friction model selection comboboxes with frition calculation models
    @param calculatePlugins Vector of pointers pointing to all the loaded calculate plugins
*/
CalculateFunctions * MainWindow::getSelectedCalculatePlugins(std::vector<CalculateRegimeInterface *> calculatePlugins)
{
    qDebug() << "MainWindow::getSelectedCalculatePlugins()";
    CalculateFunctions *calculateFunctions = new CalculateFunctions();

    //Annular
    QString pluginAnnularText = ui->comboBoxRegimeAnnular->currentText();
    QString pluginDispersedText = ui->comboBoxRegimeDispersed->currentText();
    QString pluginStratifiedText = ui->comboBoxRegimeStratified->currentText();
    QString pluginSlugText = ui->comboBoxRegimeSlug->currentText();
    std::vector<CalculateRegimeInterface *>::const_iterator iter;
    for(iter = calculatePlugins.begin(); iter != calculatePlugins.end(); ++iter) {
        qDebug() << (*iter)->pluginName();
        if ((*iter)->isAnnular() && pluginAnnularText.compare((*iter)->pluginName()) == 0) {
            calculateFunctions->setPluginAnnular(*iter);
        }
        if ((*iter)->isDispersed() && pluginDispersedText.compare((*iter)->pluginName()) == 0) {
            calculateFunctions->setPluginDispersed(*iter);
        }
        if ((*iter)->isStratified() && pluginStratifiedText.compare((*iter)->pluginName()) == 0) {
            calculateFunctions->setPluginStratified(*iter);
        }
        if ((*iter)->isSlug() && pluginSlugText.compare((*iter)->pluginName()) == 0) {
            calculateFunctions->setPluginSlug(*iter);
        }
    }
    calculateFunctions->setEnabledAnnular((ui->checkBoxRegimeAnnular->isChecked() || !ui->checkBoxRegimeUseMinimumHoldup->isChecked()));
    calculateFunctions->setEnabledDispersed((ui->checkBoxRegimeDispersed->isChecked() || !ui->checkBoxRegimeUseMinimumHoldup->isChecked()));
    calculateFunctions->setEnabledStratified((ui->checkBoxRegimeStratified->isChecked() || !ui->checkBoxRegimeUseMinimumHoldup->isChecked()));
    calculateFunctions->setEnabledSlug((ui->checkBoxRegimeSlug->isChecked() || !ui->checkBoxRegimeUseMinimumHoldup->isChecked()));
    return calculateFunctions;
}

/** Populates the friction model selection comboboxes with frition calculation models and the calculation method comboboxes with calculation methods
*/
void MainWindow::addPluginComboBoxItems()
{
    qDebug() << "MainWindow::addPluginComboBoxItems()";
    //remove old combobox items
    ui->comboBoxGasWallFriction->clear();
    ui->comboBoxLiquidWallFriction->clear();
    ui->comboBoxInterfaceFriction->clear();
    ui->comboBoxDispersedFriction->clear();
    ui->comboBoxSlipRelation->clear();
    ui->comboBoxRegimeAnnular->clear();

    //Wall friction Gas and Liquid
    FrictionInterface *wfi = NULL;
    qDebug() << "frictionInterfacePlugins.size() = " << frictionInterfacePlugins.size();
    for (unsigned int i = 0; i < frictionInterfacePlugins.size(); i++) {
        wfi = frictionInterfacePlugins.at(i);
        if (wfi->isGasWallFriction())
            ui->comboBoxGasWallFriction->addItem(wfi->pluginName());
        if (wfi->isLiquidWallFriction())
            ui->comboBoxLiquidWallFriction->addItem(wfi->pluginName());
        if (wfi->isDispersedFriction())
            ui->comboBoxDispersedFriction->addItem(wfi->pluginName());
    }

    //Interface friction
    InterfaceFrictionInterface *ifi = NULL;
    qDebug() << "interfaceFrictionInterfacePlugins.size() = " << interfaceFrictionInterfacePlugins.size();
    for (unsigned int i = 0; i < interfaceFrictionInterfacePlugins.size(); i++) {
        ifi = interfaceFrictionInterfacePlugins.at(i);
        if (ifi->isInterfaceFriction())
            ui->comboBoxInterfaceFriction->addItem(ifi->pluginName());
    }

    //Slip relation
    SlipRelationInterface *sri = NULL;
    qDebug() << "slipRelationInterfacePlugins.size() = " << slipRelationInterfacePlugins.size();
    for (unsigned int i = 0; i < slipRelationInterfacePlugins.size(); i++) {
        sri = slipRelationInterfacePlugins.at(i);
        if (sri->isSlipRelationFriction())
            ui->comboBoxSlipRelation->addItem(sri->pluginName());
    }

    //Calculation methods
    CalculateRegimeInterface *cri = NULL;
    qDebug() << "calculateInterfacePlugins.size() = " << calculateInterfacePlugins.size();
    for (unsigned int i = 0; i < calculateInterfacePlugins.size(); i++) {
        cri = calculateInterfacePlugins.at(i);
        if (cri->isAnnular())
            ui->comboBoxRegimeAnnular->addItem(cri->pluginName());
        if (cri->isDispersed())
            ui->comboBoxRegimeDispersed->addItem(cri->pluginName());
        if (cri->isStratified())
            ui->comboBoxRegimeStratified->addItem(cri->pluginName());
        if (cri->isSlug())
            ui->comboBoxRegimeSlug->addItem(cri->pluginName());
    }
}

/** Populates the predefined gas and liquid ComboBoxes for,
    @param flowType Specifies which comboBox to populate, M_LIQUID, M_GAS or M_BOTH, M_BOTH ignores newItem-arguments
    @param newItem Was a new item added? If so this item will be selected
    @param focusComboBoxName Name of combobox item to select (if newItem == true)
*/
void MainWindow::addPredefinesComboBoxItems(flowtype_t flowType, bool newItem, QString focusComboBoxName)
{
    qDebug() << "MainWindow::addPredefinesComboBoxItems()";
    QComboBox *comboBox = NULL;
    QString settingsName;

    //Set variables based on flowType
    if (flowType == M_LIQUID) {
        comboBox = ui->comboBoxLiquidPredefines;
        settingsName = "LiquidDefinitions";
    } else {
        if (flowType == M_GAS) {
            comboBox = ui->comboBoxGasPredefines;
            settingsName = "GasDefinitions";
        } else {
            if (flowType == M_BOTH) {
                addPredefinesComboBoxItems(M_GAS, false);
                flowType = M_LIQUID;
                newItem = false;
                comboBox = ui->comboBoxLiquidPredefines;
                settingsName = "LiquidDefinitions";
            } else {
                qDebug() << "Sent errorous flowType to MainWindow::addPredefinesComboBoxItems()";
            }
        }
    }

    //get current selection to focus it later
    int currentSelection =  comboBox->currentIndex();
    //remove old combobox items
    comboBox->clear();

    QSettings settings(COMPANYNAME, PRODUCTNAME);
    QMap<QString, QVariant> map = settings.value(settingsName).toMap();
    QMap<QString, QVariant>::const_iterator i = map.constBegin();
    while (i != map.constEnd()) {
         comboBox->addItem(i.key());
         ++i;
    }
    //Focus the corrent item again
    comboBox->setCurrentIndex(currentSelection);
    //Arguments to focus the new item if a new one is added
    if (newItem) {
        comboBox->setCurrentIndex(comboBox->findText(focusComboBoxName));
    }
}
/** Populates the CheckBox list over regime transition types
*/
void MainWindow::addRegimeCheckBoxes()
{
    qDebug() << "MainWindow::addRegimeCheckBoxes()";
    std::vector<QGroupBox *> groupBoxes;
    QGroupBox *groupBox;
    std::vector<QLayout *> layouts;
    QLayout *layout;
    QCheckBox *checkBox;
    QVBoxLayout *vLayout;
    bool newGroupBox;
    for (int i = 0; i < regimeInterfacePlugins.size(); i++) {
        QString pluginName = regimeInterfacePlugins.at(i)->pluginName();
        QString regimeDesc = regimeInterfacePlugins.at(i)->transitionDescription();
        newGroupBox = true;
        //Walk through current groupboxes to see if there already is one with correct title
        for (int j = 0; j < groupBoxes.size(); j++) {
            if (groupBoxes.at(j)->title().compare(regimeDesc) == 0) {
                newGroupBox = false;
                groupBox = groupBoxes.at(j);
                layout = layouts.at(j);
                break;
            }
        }
        //Make new group box since there isn't one already with correct title
        if (newGroupBox) {
            groupBox = new QGroupBox(regimeDesc, ui->tabWidget);
            layout = new QVBoxLayout(groupBox);
            layout->setMargin(6);
            groupBox->setLayout(vLayout);
            ui->verticalLayoutRegime->addWidget(groupBox);
            groupBoxes.push_back(groupBox);
            layouts.push_back(layout);
        }
        //Add item
        checkBox = new QCheckBox(pluginName, groupBox);
        checkBox->setToolTip(regimeInterfacePlugins.at(i)->tooltip());
        checkBox->setStatusTip(regimeInterfacePlugins.at(i)->statustip());
        _regimeCheckBoxes.push_back(checkBox);
        layout->addWidget(checkBox);
    }
    //Make them stick to top of view
    ui->verticalLayoutRegime->addStretch(1);
}

/** Loads values saved in settings into widgets, values that were saved when closing window previously
*/
void MainWindow::loadWidgetValues()
{
    qDebug() << "MainWindow::loadWidgetValues()";
    QSettings settings(COMPANYNAME, PRODUCTNAME);

    //LineEdit
    for (int i = 0; i < _lineEditsAll.size(); i++) {
        _lineEditsAll.at(i)->setText(settings.value(_settingNamesLineEditsAll.at(i), _settingDefaultsLineEditsAll.at(i)).toString());
    }
    //variable radio boxes
    bool def = true;
    for (unsigned int i = 0; i < _radioButtonsVariable.size(); i++) {
        if (i == 1)
            def = false;
        _radioButtonsVariable.at(i)->setChecked(settings.value(_settingNamesRadioBoxes.at(i), def).toBool());
    }
    //enable and disable the correct lineEdits corresponding to the selected radio button
    on_radioButtonVariable_clicked();
    //Regime Radio boxes
    ui->radioButtonRegimeAnnular->setChecked(settings.value("RegimeAnnular", true).toBool());
    ui->radioButtonRegimeStratified->setChecked(settings.value("RegimeStratified", false).toBool());
    ui->radioButtonRegimeDispersed->setChecked(settings.value("RegimeDispersed", false).toBool());
    ui->radioButtonRegimeSlug->setChecked(settings.value("RegimeSlug", false).toBool());
    //Regime Check boxes
    ui->checkBoxRegimeAnnular->setChecked(settings.value("UseRegimeAnnular", true).toBool());
    ui->checkBoxRegimeStratified->setChecked(settings.value("UseRegimeStratified", true).toBool());
    ui->checkBoxRegimeDispersed->setChecked(settings.value("UseRegimeDispersed", true).toBool());
    ui->checkBoxRegimeSlug->setChecked(settings.value("UseRegimeSlug", true).toBool());
    ui->checkBoxRegimeUseMinimumHoldup->setChecked(settings.value("UseMinimumHoldup", false).toBool());
    //enable and disable the correct Radio and checkboxes
    on_checkBoxRegimeUseMinimumHoldup_clicked();
    //Scale Radio boxes
    ui->radioButtonScaleExp->setChecked(settings.value("ScaleExp", true).toBool());
    ui->radioButtonScaleLin->setChecked(settings.value("ScaleLin", false).toBool());
    ui->radioButtonScaleLog->setChecked(settings.value("ScaleLog", false).toBool());
    //SpinBox resolution
    ui->spinBoxResolution->setValue(QVariant(settings.value("Resolution", 30)).toInt());
    //ComboBoxes requires searching for plugin name
    ui->comboBoxGasWallFriction->setCurrentIndex(ui->comboBoxGasWallFriction->findText(settings.value("GasWallFriction").toString()));
    ui->comboBoxLiquidWallFriction->setCurrentIndex(ui->comboBoxLiquidWallFriction->findText(settings.value("LiquidWallFriction").toString()));
    ui->comboBoxInterfaceFriction->setCurrentIndex(ui->comboBoxInterfaceFriction->findText(settings.value("InterfaceFriction").toString()));
    ui->comboBoxDispersedFriction->setCurrentIndex(ui->comboBoxDispersedFriction->findText(settings.value("DispersedSlugFriction").toString()));
    ui->comboBoxSlipRelation->setCurrentIndex(ui->comboBoxSlipRelation->findText(settings.value("DispersedSlugSlipRelation").toString()));
    //In case no items are selected.
    if (ui->comboBoxGasWallFriction->currentIndex() == -1 && ui->comboBoxGasWallFriction->count() > 0)
        ui->comboBoxGasWallFriction->setCurrentIndex(0);
    if (ui->comboBoxLiquidWallFriction->currentIndex() == -1 && ui->comboBoxLiquidWallFriction->count() > 0)
        ui->comboBoxLiquidWallFriction->setCurrentIndex(0);
    if (ui->comboBoxInterfaceFriction->currentIndex() == -1 && ui->comboBoxInterfaceFriction->count() > 0)
        ui->comboBoxInterfaceFriction->setCurrentIndex(0);
    if (ui->comboBoxDispersedFriction->currentIndex() == -1 && ui->comboBoxDispersedFriction->count() > 0)
        ui->comboBoxDispersedFriction->setCurrentIndex(0);
    if (ui->comboBoxSlipRelation->currentIndex() == -1 && ui->comboBoxSlipRelation->count() > 0)
        ui->comboBoxSlipRelation->setCurrentIndex(0);

    //get last used id
    _simID = settings.value("SimulationID", 100).toInt();
    _firstNewSimID = _simID + 1;
    _simRegimeID = settings.value("SimulationRegimeID", 100).toInt();
    _firstNewSimRegimeID = _simRegimeID + 1;

    //Place window where it was when last closed
    if (settings.contains("MainWindowRect")) {
        this->restoreGeometry(settings.value("MainWindowRect").toByteArray());
    }

    //QSplitters
    int sizeBottom = settings.value("FlowSplitterListSize", -1).toInt();
    if (sizeBottom != -1) {
        QList<int> listSizes;
        //Fixes bug
        sizeBottom -= 26;
        listSizes.push_back(flowSplitter->size().height()-sizeBottom);
        listSizes.push_back(sizeBottom);
        flowSplitter->setSizes(listSizes);
    }
    sizeBottom = settings.value("RegimeSplitterListSize", -1).toInt();
    if (sizeBottom != -1) {
        QList<int> listSizes;
        //Fixes bug
        sizeBottom -= 50;
        listSizes.push_back(regimeSplitter->size().height()-sizeBottom);
        listSizes.push_back(sizeBottom);
        regimeSplitter->setSizes(listSizes);
    }
    //Regime transition checkboxes
    for (int i = 0; i < _regimeCheckBoxes.size(); i++) {
        QString name = "setting";
        name.prepend(_regimeCheckBoxes.at(i)->text());
        _regimeCheckBoxes.at(i)->setChecked(QVariant(settings.value(name, false)).toBool());
    }
}

/** Saves current values of widgets, this allows the program to keep the variables for when the program is restarted
*/
void MainWindow::saveWidgetValues()
{
    qDebug() << "MainWindow::saveWidgetValues()";
    QSettings settings(COMPANYNAME, PRODUCTNAME);
    //LineEdits
    for (unsigned int i = 0; i < _lineEditsAll.size(); i++) {
        settings.setValue(_settingNamesLineEditsAll.at(i), _lineEditsAll.at(i)->text());
    }
    //Variable Radio boxes
    for (unsigned int i = 0; i < _settingNamesRadioBoxes.size(); i++) {
        settings.setValue(_settingNamesRadioBoxes.at(i), _radioButtonsVariable.at(i)->isChecked());
    }
    //Regime Radio boxes
    settings.setValue("RegimeAnnular", ui->radioButtonRegimeAnnular->isChecked());
    settings.setValue("RegimeStratified", ui->radioButtonRegimeStratified->isChecked());
    settings.setValue("RegimeDispersed", ui->radioButtonRegimeDispersed->isChecked());
    settings.setValue("RegimeSlug", ui->radioButtonRegimeSlug->isChecked());
    //Regime Check boxes
    settings.setValue("UseRegimeAnnular", ui->checkBoxRegimeAnnular->isChecked());
    settings.setValue("UseRegimeStratified", ui->checkBoxRegimeStratified->isChecked());
    settings.setValue("UseRegimeDispersed", ui->checkBoxRegimeDispersed->isChecked());
    settings.setValue("UseRegimeSlug", ui->checkBoxRegimeSlug->isChecked());
    settings.setValue("UseMinimumHoldup", ui->checkBoxRegimeUseMinimumHoldup->isChecked());
    //Scale Radio boxes
    settings.setValue("ScaleExp", ui->radioButtonScaleExp->isChecked());
    settings.setValue("ScaleLin", ui->radioButtonScaleLin->isChecked());
    settings.setValue("ScaleLog", ui->radioButtonScaleLog->isChecked());
    //SpinBox resolution
    settings.setValue("Resolution", ui->spinBoxResolution->value());
    //ComboBoxes
    settings.setValue("GasWallFriction", ui->comboBoxGasWallFriction->currentText());
    settings.setValue("LiquidWallFriction", ui->comboBoxLiquidWallFriction->currentText());
    settings.setValue("InterfaceFriction", ui->comboBoxInterfaceFriction->currentText());
    settings.setValue("DispersedSlugFriction", ui->comboBoxDispersedFriction->currentText());
    settings.setValue("DispersedSlugSlipRelation", ui->comboBoxSlipRelation->currentText());
    //Simulation ID
    settings.setValue("SimulationID", _simID);
    settings.setValue("SimulationRegimeID", _simRegimeID);
    //WindowGeometry
    settings.setValue("MainWindowRect", this->saveGeometry());
    //QSplitters
    QList<int> sizeList = flowSplitter->sizes();
    if (sizeList.size() == 2) {
        settings.setValue("FlowSplitterListSize", sizeList.at(1));
    }
    sizeList = regimeSplitter->sizes();
    if (sizeList.size() == 2) {
        settings.setValue("RegimeSplitterListSize", sizeList.at(1));
    }
    //Regime transition checkboxes
    for (int i = 0; i < _regimeCheckBoxes.size(); i++) {
        QString name = "Setting";
        name.prepend(_regimeCheckBoxes.at(i)->text());
        settings.setValue(name, _regimeCheckBoxes.at(i)->isChecked());
    }
}

/** Saves gas variables as a new definition
    @param name Name of definition
    @param density Density of gas
    @param viscosity Viscosity of gas
    @param temperature Temperature of gas
    @param pressure Pressure of gas
    @param densityMin Minimum density of gas
    @param viscosityMin Minimum viscosity of gas
    @param temperatureMin Minimum temperature of gas
    @param pressureMin Minimum density of gas
    @param densityMax Maximum density of gas
    @param viscosityMax Maximum viscosity of gas
    @param temperatureMax Maximum temperature of gas
    @param pressureMax Maximum density of gas
*/
void MainWindow::saveGasDefinition(QString name, QString density, QString viscosity, QString temperature, QString pressure,
                                   QString densityMin, QString viscosityMin, QString temperatureMin, QString pressureMin,
                                   QString densityMax, QString viscosityMax, QString temperatureMax, QString pressureMax)
{
    qDebug() << "MainWindow::saveGasDefinition()";
    QMap<QString, QVariant> gasMap;
    QSettings settings(COMPANYNAME, PRODUCTNAME);
    gasMap = settings.value("GasDefinitions").toMap();
    QStringList gasValues;
    gasValues.push_back(density);
    gasValues.push_back(viscosity);
    gasValues.push_back(temperature);
    gasValues.push_back(pressure);
    if (densityMin.compare("") != 0) {
        gasValues.push_back(densityMin);
        gasValues.push_back(viscosityMin);
        gasValues.push_back(temperatureMin);
        gasValues.push_back(pressureMin);
        gasValues.push_back(densityMax);
        gasValues.push_back(viscosityMax);
        gasValues.push_back(temperatureMax);
        gasValues.push_back(pressureMax);
    }

    gasMap.insert(name, gasValues);
    settings.setValue("GasDefinitions", gasMap);
    addPredefinesComboBoxItems(M_GAS, true, name);
}

/** Saves current gas parameters as a new definition
*   @param definitionName Name of new definition
*/
void MainWindow::saveGasDefinition(QString definitionName)
{
    qDebug() << "MainWindow::saveGasDefinition()";
    saveGasDefinition(definitionName, ui->lineEditGasDensity->text(), ui->lineEditGasViscosity->text(), ui->lineEditGasTemperature->text(), ui->lineEditGasPressure->text(),
                      ui->lineEditGasDensityMin->text(), ui->lineEditGasViscosityMin->text(), ui->lineEditGasTemperatureMin->text(), ui->lineEditGasPressureMin->text(),
                      ui->lineEditGasDensityMax->text(), ui->lineEditGasViscosityMax->text(), ui->lineEditGasTemperatureMax->text(), ui->lineEditGasPressureMax->text());
}

/** Saves liquid variables as a new definition
    @param name Name of definition
    @param density Density of liquid
    @param viscosity Viscosity of liquid
    @param densityMin Minimum density of liquid
    @param viscosityMin Minimum viscosity of liquid
    @param densityMax Maximum density of liquid
    @param viscosityMax Maximum viscosity of liquid
*/
void MainWindow::saveLiquidDefinition(QString name, QString density, QString viscosity, QString densityMin, QString viscosityMin,
                                      QString densityMax, QString viscosityMax)
{
    qDebug() << "MainWindow::saveLiquidDefinition(QStrings)";
    QMap<QString, QVariant> liquidMap;
    QSettings settings(COMPANYNAME, PRODUCTNAME);
    liquidMap = settings.value("LiquidDefinitions").toMap();
    QStringList liquidValues;
    liquidValues.push_back(density);
    liquidValues.push_back(viscosity);
    if (densityMin.compare("") != 0) {
        liquidValues.push_back(densityMin);
        liquidValues.push_back(viscosityMin);
        liquidValues.push_back(densityMax);
        liquidValues.push_back(viscosityMax);
    }
    liquidMap.insert(name, liquidValues);
    settings.setValue("LiquidDefinitions", liquidMap);
    addPredefinesComboBoxItems(M_LIQUID, true, name);
}

/** Saves current liquid parameters as a new definition
*   @param definitionName Name of new definition
*/
void MainWindow::saveLiquidDefinition(QString definitionName)
{
    qDebug() << "MainWindow::saveLiquidDefinition()";
    saveLiquidDefinition(definitionName, ui->lineEditLiquidDensity->text(), ui->lineEditLiquidDensityMin->text(),
                         ui->lineEditLiquidDensityMax->text(), ui->lineEditLiquidViscosity->text(), ui->lineEditLiquidViscosityMin->text(),
                         ui->lineEditLiquidViscosityMax->text());
}

/** Save Gas Predefines button clicked, open dialog asking for name of definition
*   @see on_pushButtonSaveLiquidPredefine_clicked()
*/
void MainWindow::on_pushButtonSaveGasPredefine_clicked()
{
    qDebug() << "MainWindow::on_pushButtonSaveGasPredefine_clicked()";
    bool ok;
    QString text = QInputDialog::getText(this, tr("Definition Name"),
                                        tr("Definition Name:"), QLineEdit::Normal, "", &ok);
    if (ok && !text.isEmpty())
                saveGasDefinition(text);
}

/** Save Liquid Predefines button clicked, open dialog asking for name of definition
*   @see on_pushButtonSaveGasPredefine_clicked()
*/
void MainWindow::on_pushButtonSaveLiquidPredefine_clicked()
{
    qDebug() << "MainWindow::on_pushButtonSaveLiquidPredefine_clicked()";
    bool ok;
    QString text = QInputDialog::getText(this, tr("Definition Name"),
                                        tr("Definition Name:"), QLineEdit::Normal, "", &ok);
    if (ok && !text.isEmpty())
                saveLiquidDefinition(text);
}

/** Load Gas Predefines button clicked, load currently selected gas
*   @see on_pushButtonLoadLiquidPredefines_clicked()
*/
void MainWindow::on_pushButtonLoadGasPredefines_clicked()
{
    qDebug() << "MainWindow::on_pushButtonLoadGasPredefines_clicked()";
    QMap<QString, QVariant> map;
    QSettings settings(COMPANYNAME, PRODUCTNAME);
    map = settings.value("GasDefinitions").toMap();
    QMap<QString, QVariant>::const_iterator i = map.constBegin();
    while (i != map.constEnd()) {
        if (i.key() == ui->comboBoxGasPredefines->currentText()) {
            QVariant qv = i.value();
            QStringList qsl = qv.toStringList();
            if (qsl.length() == 12) {
                ui->lineEditGasDensity->setText(qsl.at(0));
                ui->lineEditGasViscosity->setText(qsl.at(1));
                ui->lineEditGasTemperature->setText(qsl.at(2));
                ui->lineEditGasPressure->setText(qsl.at(3));
                if (qsl.at(4).compare("") != 0)
                    ui->lineEditGasDensityMin->setText(qsl.at(4));
                if (qsl.at(5).compare("") != 0)
                    ui->lineEditGasViscosityMin->setText(qsl.at(5));
                if (qsl.at(6).compare("") != 0)
                    ui->lineEditGasTemperatureMin->setText(qsl.at(6));
                if (qsl.at(7).compare("") != 0)
                    ui->lineEditGasPressureMin->setText(qsl.at(7));
                if (qsl.at(8).compare("") != 0)
                    ui->lineEditGasDensityMax->setText(qsl.at(8));
                if (qsl.at(9).compare("") != 0)
                    ui->lineEditGasViscosityMax->setText(qsl.at(9));
                if (qsl.at(10).compare("") != 0)
                    ui->lineEditGasTemperatureMax->setText(qsl.at(10));
                if (qsl.at(11).compare("") != 0)
                    ui->lineEditGasPressureMax->setText(qsl.at(11));

                ui->tabWidget->setCurrentIndex(0);
            } else {
                if (qsl.length() == 4) {
                    ui->lineEditGasDensity->setText(qsl.at(0));
                    ui->lineEditGasViscosity->setText(qsl.at(1));
                    ui->lineEditGasTemperature->setText(qsl.at(2));
                    ui->lineEditGasPressure->setText(qsl.at(3));

                    ui->tabWidget->setCurrentIndex(0);
                } else {
                    qDebug() << "- Loading of gas predefined values failed";
                }
            }
            //end while
            i = map.constEnd()-1;
        }
        ++i;
    }
}

/** Load Liquid Predefines button clicked, load currently selected liquid
*   @see on_pushButtonLoadGasPredefines_clicked()
*/
void MainWindow::on_pushButtonLoadLiquidPredefines_clicked()
{
    qDebug() << "MainWindow::on_pushButtonLoadLiquidPredefines_clicked()";
    QMap<QString, QVariant> map;
    QSettings settings(COMPANYNAME, PRODUCTNAME);
    map = settings.value("LiquidDefinitions").toMap();
    QMap<QString, QVariant>::const_iterator i = map.constBegin();
    while (i != map.constEnd()) {
        if (i.key() == ui->comboBoxLiquidPredefines->currentText()) {
            QVariant qv = i.value();
            QStringList qsl = qv.toStringList();
            if (qsl.length() == 6) {
                ui->lineEditLiquidDensity->setText(qsl.at(0));
                ui->lineEditLiquidViscosity->setText(qsl.at(1));
                if (qsl.at(2).compare("") != 0)
                    ui->lineEditLiquidDensityMin->setText(qsl.at(2));
                if (qsl.at(3).compare("") != 0)
                    ui->lineEditLiquidViscosityMin->setText(qsl.at(3));
                if (qsl.at(4).compare("") != 0)
                    ui->lineEditLiquidDensityMax->setText(qsl.at(4));
                if (qsl.at(5).compare("") != 0)
                    ui->lineEditLiquidViscosityMax->setText(qsl.at(5));

                ui->tabWidget->setCurrentIndex(0);
            } else {
                if (qsl.length() == 2) {
                    ui->lineEditLiquidDensity->setText(qsl.at(0));
                    ui->lineEditLiquidViscosity->setText(qsl.at(1));

                    ui->tabWidget->setCurrentIndex(0);
                } else {
                    qDebug() << "- Loading of liquid predefined values failed";
                }
            }
            //end while
            i = map.constEnd()-1;
        }
        ++i;
    }
}

/** Delete Gas Predefines button clicked, delete currently selected gas
*   @see on_pushButtonDeleteLiquidPredefines_clicked()
*/
void MainWindow::on_pushButtonDeleteGasPredefines_clicked()
{
    qDebug() << "MainWindow::on_pushButtonDeleteGasPredefines_clicked()";
    QSettings settings(COMPANYNAME, PRODUCTNAME);
    QMap<QString, QVariant> map = settings.value("GasDefinitions").toMap();
    map.remove(ui->comboBoxGasPredefines->currentText());
    ui->comboBoxGasPredefines->removeItem(ui->comboBoxGasPredefines->currentIndex());
    settings.setValue("GasDefinitions", map);
}

/** Delete Liquid Predefines button clicked, delete currently selected liquid
*   @see on_pushButtonDeleteGasPredefines_clicked()
*/
void MainWindow::on_pushButtonDeleteLiquidPredefines_clicked()
{
    qDebug() << "MainWindow::on_pushButtonDeleteLiquidPredefines_clicked() ";
    QSettings settings(COMPANYNAME, PRODUCTNAME);
    QMap<QString, QVariant> map = settings.value("LiquidDefinitions").toMap();
    map.remove(ui->comboBoxLiquidPredefines->currentText());
    ui->comboBoxLiquidPredefines->removeItem(ui->comboBoxLiquidPredefines->currentIndex());
    settings.setValue("LiquidDefinitions", map);
}

/** Exit called form menu, exit program */
void MainWindow::on_actionExit_triggered()
{
    qDebug() << "MainWindow::on_actionExit_triggered()";
    this->close();
}

/** Sets values in data for variables inputed
*   @param data Pointer to where Data instance to which values should be written
*   @see setWidgetValues()
*/
void MainWindow::getWidgetValues(Data *data)
{
    qDebug() << "MainWindow::getWidgetValues()";
    data->setDensityGas(ui->lineEditGasDensity->text().toDouble());
    data->setDensityLiquid(ui->lineEditLiquidDensity->text().toDouble());
    data->setTemperatureGas(ui->lineEditGasTemperature->text().toDouble());
    data->setPressureGas(ui->lineEditGasPressure->text().toDouble());
    data->setViscosityGas(ui->lineEditGasViscosity->text().toDouble());
    data->setViscosityLiquid(ui->lineEditLiquidViscosity->text().toDouble());
    data->setUsg(ui->lineEditUsg->text().toDouble());
    data->setUsl(ui->lineEditUsl->text().toDouble());
    data->setSurfaceTension(ui->lineEditSurfaceTension->text().toDouble());
    data->setAngle(ui->lineEditAngle->text().toDouble());
    data->setDiameter(ui->lineEditDiameter->text().toDouble()*pow(10, -3));
    data->setRoughness(ui->lineEditRoughness->text().toDouble()*pow(10, -6));
    data->setRegimeType(getRegimeType());

    for (unsigned int i = 0; i < _radioButtonsVariable.size(); i++) {
        if (_radioButtonsVariable.at(i)->isChecked() == true && _variableScaler.size() > i) {
            qDebug() << "Radio button " << i << "is checked";
            data->setVariableType((variable_t)i);
            qDebug() << (variable_t)i;
            data->setVariableMin(_lineEditsMin.at(i)->text().toDouble() * _variableScaler.at(i));
            data->setVariableMax(_lineEditsMax.at(i)->text().toDouble() * _variableScaler.at(i));
        }
    }
}

/** Set the value of widgets based on the values of the supplied argument
*   @param data Pointer to Data instance with values
*   @see getWidgetValues()
*/
void MainWindow::setWidgetValues(const Data *data)
{
    ui->lineEditUsg->setText(QVariant(data->Usg()).toString());
    ui->lineEditUsl->setText(QVariant(data->Usl()).toString());

    ui->lineEditGasDensity->setText(QVariant(data->densityGas()).toString());
    ui->lineEditGasViscosity->setText(QVariant(data->viscosityGas()).toString());
    ui->lineEditGasTemperature->setText(QVariant(data->temperatureGas()).toString());
    ui->lineEditGasPressure->setText(QVariant(data->pressureGas()).toString());
    ui->lineEditLiquidDensity->setText(QVariant(data->densityLiquid()).toString());
    ui->lineEditLiquidViscosity->setText(QVariant(data->viscosityLiquid()).toString());
    ui->lineEditSurfaceTension->setText(QVariant(data->surfaceTension()).toString());

    ui->lineEditAngle->setText(QVariant(data->angle()).toString());
    ui->lineEditDiameter->setText(QVariant(data->diameter()*pow(10, 3)).toString());
    ui->lineEditRoughness->setText(QVariant(data->roughness()*pow(10, 6)).toString());

    int varNum = (int)data->variableType();
    if (varNum > 0 && varNum < _lineEditsMin.size()) {
        _lineEditsMin.at(varNum)->setText(QVariant(data->variableMin()).toString());
        _lineEditsMax.at(varNum)->setText(QVariant(data->variableMax()).toString());
    }
}

/** Returns a regimetype_t value indicating the regime type to simulate */
regimetype_t MainWindow::getRegimeType()
{
    qDebug() << "MainWindow::getRegimeType()";
    regimetype_t regimeType;

    //If no specific regime since we use minimum holdup criterion
    if (ui->checkBoxRegimeUseMinimumHoldup->isChecked())
        return REGIME_DYNAMIC_MINIMUM_HOLDUP;

    for (unsigned int i = 0; i < _radioButtonsRegime.size(); i++) {
        if (_radioButtonsRegime.at(i)->isChecked() == true) {
            regimeType = (regimetype_t)i;
        }
    }
    return regimeType;
}

/** Returns a scale_t value indicating the scaling of x values */
scale_t MainWindow::getScaleType()
{
    qDebug() << "MainWindow::getScaleType()";
    scale_t scaleType;

    if (ui->radioButtonScaleExp->isChecked()) {
        scaleType = SCALE_EXP;
    } else {
        if (ui->radioButtonScaleLin->isChecked()) {
            scaleType = SCALE_LIN;
        } else {
            scaleType = SCALE_LOG;
        }
    }
    return scaleType;
}

/** Simulation button has been clicked, start simulation */
void MainWindow::on_pushButtonSimulateRegime_clicked()
{
    qDebug() << "MainWindow::on_pushButtonSimulateRegime_clicked()";

    if (!isInputValid(SIMULATION_REGIMETRANSITION)) {
        return;
    }

    //set values
    Data *data = new Data();
    getWidgetValues(data);
    //Get regime type and variable variable
    FrictionFunctions *frictionFunctions;
    frictionFunctions = getSelectedFrictionPlugins(frictionInterfacePlugins, interfaceFrictionInterfacePlugins, slipRelationInterfacePlugins);
    CalculateFunctions *calculateFunctions;
    calculateFunctions = getSelectedCalculatePlugins(calculateInterfacePlugins);

    ui->pushButtonSimulateRegime->setDisabled(true);
    ui->pushButtonStopSimulationRegime->setEnabled(true);

    //See which plugins correspond to checkboxes currently checked, save pointer to these plugins in 'plugins'
    std::vector<RegimeTransitionInterface *> plugins;
    for (int i = 0; i < _regimeCheckBoxes.size(); i++) {
        if (_regimeCheckBoxes.at(i)->isChecked()) {
            plugins.push_back(regimeInterfacePlugins.at(i));
        }
    }
    this->setStatusTip("Simulating...");
    _regimeCalculationManagers.clear();
    //Walk through 'plugins' and start a CalculateManager for each one
    for (int i = 0; i < plugins.size(); i++) {
        ++_simRegimeID;
        Data *sendData = new Data(data);
        QString regimeTransitionName = plugins.at(i)->pluginName();
        regimeTransitionName.replace("&", "");
        sendData->setRegimeTransitionName(regimeTransitionName);
        cm = new CalculateManager(this, _simRegimeID, sendData, frictionFunctions, plugins.at(i), calculateFunctions, ui->spinBoxResolution->value(), THREADS, getScaleType(), 0.0000001, regimeTransitionName);
        _regimeCalculationManagers.push_back(cm);
        connect(cm, SIGNAL(simulationProgressed(int)), this, SLOT(on_regimeSimulation_progressed(int)));
        connect(cm, SIGNAL(finished()), this, SLOT(on_regimeSimulation_finished()));
        connect(cm, SIGNAL(terminated()), this, SLOT(on_regimeSimulation_terminated()));
        connect(cm, SIGNAL(simulationError(QString)), this, SLOT(on_simulationError(QString)));
        if (i == 0) {
            cm->run();
        }
    }
}

/** Enter What's this mode */
void MainWindow::on_actionWhats_this_triggered()
{
    qDebug() << "MainWindow::on_actionWhats_this_triggered()";
    QWhatsThis::enterWhatsThisMode();
}

/** Radio button selecting simulation variable clicked, enable appropriate LineEdits */
void MainWindow::on_radioButtonVariable_clicked()
{
    qDebug() << "MainWindow::on_radioButtonVariable_clicked()";
    for (int i = 0; i < _radioButtonsVariable.size(); i++) {
        if (_radioButtonsVariable.at(i)->isChecked() == true) {
            _lineEdits.at(i)->setDisabled(true);
            _lineEditsMin.at(i)->setEnabled(true);
            _lineEditsMax.at(i)->setEnabled(true);
        } else {
            _lineEdits.at(i)->setEnabled(true);
            _lineEditsMin.at(i)->setDisabled(true);
            _lineEditsMax.at(i)->setDisabled(true);
        }
    }
}

/** Simulation button has been clicked, start simulation */
void MainWindow::on_pushButtonSimulateFlow_clicked()
{
    qDebug() << "MainWindow::on_pushButtonSimulateFlow_clicked()";
    ++_simID;
    saveWidgetValues();

    if (!isInputValid(SIMULATION_FLOW)) {
        return;
    }

    //set values
    Data *data = new Data();
    getWidgetValues(data);
    qDebug() << "Printing data";
    qDebug() << *data;

    ui->pushButtonSimulateFlow->setDisabled(true);
    ui->pushButtonStopSimulationFlow->setEnabled(true);
    ui->progressBarSimulation->setValue(0);

    //Get regime type and variable variable
    FrictionFunctions *frictionFunctions;
    frictionFunctions = getSelectedFrictionPlugins(frictionInterfacePlugins, interfaceFrictionInterfacePlugins, slipRelationInterfacePlugins);
    CalculateFunctions *calculateFunctions;
    calculateFunctions = getSelectedCalculatePlugins(calculateInterfacePlugins);
    cm = new CalculateManager(this, _simID, data, frictionFunctions, calculateFunctions, ui->checkBoxRegimeUseMinimumHoldup->isChecked(), ui->spinBoxResolution->value(), THREADS, getScaleType(), 0.00000002);
    this->setStatusTip("Simulating...");
    cm->run();
    qDebug() << "cm started";
    connect(cm, SIGNAL(simulationProgressed(int)), this, SLOT(on_flowSimulation_progressed(int)));
    connect(cm, SIGNAL(finished()), this, SLOT(on_flowSimulation_finished()));
    connect(cm, SIGNAL(terminated()), this, SLOT(on_flowSimulation_terminated()));
    connect(cm, SIGNAL(simulationError(QString)), this, SLOT(on_simulationError(QString)));
}

/** Slot which updates the progressBar to reflect the simulation progression based on feedback from CalculateManager through progressed() signal
*   @param percent Integer value from 0 to 100 indicating the level of completion of the simulation
*/
void MainWindow::on_flowSimulation_progressed(int percent)
{
    ui->progressBarSimulation->setValue(percent);
}

/** Simulation finished, plot and store results
*/
void MainWindow::on_flowSimulation_finished()
{
    qDebug() << "MainWindow::on_flowSimulation_finished()";
    ui->pushButtonSimulateFlow->setEnabled(true);
    ui->pushButtonStopSimulationFlow->setDisabled(true);
    //Retrieve values
    std::vector<double> xList = cm->xList();
    std::vector<double> holdupList = cm->holdupList();
    std::vector<double> dpdxList = cm->dpdxList();
    std::vector<double> alphaList = cm->alphaList();
    //QString xVariableName = cm->xVariableName();
    //cm->quit();

    //Test storageclass
    QDir nameDir(QDir::currentPath());
    nameDir.cd(SIMULATIONSFOLDER);

    //Save to file
    SimulationStorage *ss = new SimulationStorage(SIMULATION_FLOW, cm->simulationID(), nameDir, cm->data());
    ss->addData("X", xList);
    ss->addData(holdupPlot->objectName(), holdupList);
    ss->addData(pressurePlot->objectName(), dpdxList);
    ss->addData(alphaPlot->objectName(), alphaList);
    ss->store();
    qDebug() << "Storing done";
    delete ss;
    ss = NULL;

    //Add to treeview
    int row = simulationsFlowModel->rowCount();
    simulationsFlowModel->setItem(row, 0, new QStandardItem(QVariant(cm->simulationID()).toString()));
    simulationsFlowModel->setItem(row, 1, new QStandardItem(cm->data()->regimeNameShort(4)));
    simulationsFlowModel->setItem(row, 2, new QStandardItem(cm->xVariableName()));
    simulationsFlowModel->setItem(row, 3, new QStandardItem(QVariant(cm->data()->variableMin()).toString()));
    simulationsFlowModel->setItem(row, 4, new QStandardItem(QVariant(cm->data()->variableMax()).toString()));
    simulationsFlowModel->setItem(row, 5, new QStandardItem(QVariant(cm->data()->Usg()).toString()));
    simulationsFlowModel->setItem(row, 6, new QStandardItem(QVariant(cm->data()->Usl()).toString()));
    simulationsFlowModel->setItem(row, 7, new QStandardItem(QVariant(cm->data()->angle()).toString()));
    simulationsFlowModel->setItem(row, 8, new QStandardItem(QVariant(cm->data()->roughness()).toString()));
    simulationsFlowModel->setItem(row, 9, new QStandardItem(QVariant(cm->data()->diameter()).toString()));

    //Add curve to plots
    holdupPlot->Plot()->clear();
    holdupPlot->Plot()->addCurve(xList, holdupList, QVariant(_simID).toString());
    pressurePlot->Plot()->clear();
    pressurePlot->Plot()->addCurve(xList, dpdxList, QVariant(_simID).toString());
    alphaPlot->Plot()->clear();
    alphaPlot->Plot()->addCurve(xList, alphaList, QVariant(_simID).toString());
    //Set x axis and plot title
    for (int i = 0; i < _XYPlotAreas.size(); i++) {
        //axisTitle(QwtPlot::yLeft)
        _XYPlotAreas.at(i)->Plot()->setTitle(QString("%1 vs %2").arg(_XYPlotAreas.at(i)->objectName()).arg(cm->xVariableName()));
        _XYPlotAreas.at(i)->Plot()->setAxisTitle(QwtPlot::xBottom, cm->xVariableName());
    }
    delete cm;

    ui->tabWidgetScheme->setCurrentIndex(0);
}

/** Returns true if the LineEdit and DropdownMenu values are valid and usable in flow simulation calculations
    If something is wrong, it will focus that item */
bool MainWindow::isInputValid(simulationtype_t simulationType)
{
    bool valid = true;
    //All Lineedits
    for (unsigned int i = 0; i < _lineEditsAll.size(); i++) {
        double d = _lineEditsAll.at(i)->text().toDouble();
        if ((d > 1000000000 || d <= 0) && _lineEditsAll.at(i)->isEnabled()) {
            //Angle can be negative
            if (_settingNamesLineEditsAll.at(i).contains("Angle", Qt::CaseInsensitive) == 0) {
                valid = false;
                _lineEditsAll.at(i)->setStyleSheet("* { background-color: rgb(255, 0, 0); } ");
                ui->tabWidget->setCurrentIndex(0);
            }
        } else {
            _lineEditsAll.at(i)->setStyleSheet(QString::Null());
        }
    }
    //Friction dropdowns
    if (ui->comboBoxGasWallFriction->currentIndex() == -1) {
        ui->comboBoxGasWallFriction->setStyleSheet("* { background-color: rgb(255, 0, 0); } ");
        ui->tabWidget->setCurrentIndex(2);
    } else {
        ui->comboBoxGasWallFriction->setStyleSheet(QString::Null());
    }
    if (ui->comboBoxDispersedFriction->currentIndex() == -1) {
        ui->comboBoxDispersedFriction->setStyleSheet("* { background-color: rgb(255, 0, 0); } ");
        ui->tabWidget->setCurrentIndex(2);
    } else {
        ui->comboBoxDispersedFriction->setStyleSheet(QString::Null());
    }
    if (ui->comboBoxInterfaceFriction->currentIndex() == -1) {
        ui->comboBoxInterfaceFriction->setStyleSheet("* { background-color: rgb(255, 0, 0); } ");
        ui->tabWidget->setCurrentIndex(2);
    } else {
        ui->comboBoxInterfaceFriction->setStyleSheet(QString::Null());
    }
    if (ui->comboBoxLiquidWallFriction->currentIndex() == -1) {
        ui->comboBoxLiquidWallFriction->setStyleSheet("* { background-color: rgb(255, 0, 0); } ");
        ui->tabWidget->setCurrentIndex(2);
    } else {
        ui->comboBoxLiquidWallFriction->setStyleSheet(QString::Null());
    }
    if (ui->comboBoxSlipRelation->currentIndex() == -1) {
        ui->comboBoxSlipRelation->setStyleSheet("* { background-color: rgb(255, 0, 0); } ");
        ui->tabWidget->setCurrentIndex(2);
    } else {
        ui->comboBoxSlipRelation->setStyleSheet(QString::Null());
    }

    if (simulationType == SIMULATION_REGIMETRANSITION) {
        //Make sure USG is set as variable
        if (!ui->radioButtonVariableUsg->isChecked()) {
            ui->radioButtonVariableUsg->setStyleSheet("* { background-color: rgb(255, 0, 0); } ");
            ui->tabWidget->setCurrentIndex(0);
            valid = false;
        } else {
            ui->radioButtonVariableUsg->setStyleSheet(QString::Null());
        }
        //Check that when trying to run a simulation a regime transition calculation method is selected
        int checked = 0;
        for (int i  = 0; i < _regimeCheckBoxes.size(); i++) {
            checked += _regimeCheckBoxes.at(i)->isChecked();
        }
        if (checked == 0) {
            if (_regimeCheckBoxes.size() > 0) {
                _regimeCheckBoxes.at(0)->setStyleSheet("* { background-color: rgb(255, 0, 0); } ");
                ui->tabWidget->setCurrentIndex(3);
                valid = false;
            }
        } else {
            _regimeCheckBoxes.at(0)->setStyleSheet(QString::Null());
        }
    }
    return valid;
}

/** Command that does the neccecery tasks for the application to run properly first time around */
void MainWindow::firstRunSetup()
{
    //Add predefined materials
    //Water
    saveLiquidDefinition("Water", "1000", "8.901e-4");
    saveLiquidDefinition("ConocoPhillips 32", "907", "25.396e-3");
    saveLiquidDefinition("ConocoPhillips 68", "870", "7.569e-3");
    saveLiquidDefinition("ConocoPhillips 100", "871", "87.1e-3");
    saveLiquidDefinition("Exxon Mobil 80", "885", "115.050e-3");
    saveLiquidDefinition("Exxon Mobil 65", "880", "79.904e-3");
    saveLiquidDefinition("Exxon Mobil 100", "893", "17.4135e-3");
    //Gas
    saveGasDefinition("Air", "1.275", "1.845e-5", "298.16", "1.013");
    saveGasDefinition("CO2", "1.84212", "1.493e-5", "298.16", "1.013");
}

void MainWindow::on_pushButtonPlotSimulations_clicked()
{
    qDebug() <<  "MainWindow::on_pushButtonPlotSimulations_clicked()";
    plotSelectedSimulations(SIMULATION_FLOW);
}

void MainWindow::on_pushButtonPlotSimulationsRegime_clicked()
{
    qDebug() <<  "MainWindow::on_pushButtonPlotSimulationsRegime_clicked()";
    plotSelectedSimulations(SIMULATION_REGIMETRANSITION);
}

/** Some items in the history list view have been selected and should be plotted
    @param simulationType Specifies if function applies to flow or regime transition **/
void MainWindow::plotSelectedSimulations(simulationtype_t simulationType)
{
    qDebug() <<  "MainWindow::plotSelectedSimulations()";
    QDir folderPath(QDir::currentPath());
    folderPath.cd(SIMULATIONSFOLDER);

    QModelIndexList qmil;
    QModelIndex qmIndex;

    int id;
    std::vector<QString> dataNames;
    std::vector<double> xValues;
    std::vector<std::vector<double> > simulationData;

    if (simulationType == SIMULATION_FLOW) {
        holdupPlot->Plot()->clear();
        pressurePlot->Plot()->clear();
        alphaPlot->Plot()->clear();
        qmil = simulationFlowItemSelectionModel->selectedRows(2);
    } else {
        regimePlot->Plot()->clear();
        qmil = simulationRegimeItemSelectionModel->selectedRows(2);
    }

    QString variableString = "";
    for (int i = 0; i < qmil.size(); i++) {
        qmIndex = qmil.at(i);
        if (variableString.compare("") == 0) {
            variableString = qmIndex.data().toString();
        } else {
            if (variableString .compare(qmIndex.data().toString()) != 0) {
                QString errorMsg = "Plotting simulations with different variables in same diagram.\n\n";
                errorMsg.append(QString("(Variables %1 and %2)").arg(variableString).arg(qmIndex.data().toString()));
                errorMessage->showMessage(errorMsg);
            }
        }
    }

    if (simulationType == SIMULATION_FLOW) {
        qmil = simulationFlowItemSelectionModel->selectedRows(0);
    } else {
        qmil = simulationRegimeItemSelectionModel->selectedRows(0);
    }
    for (int i = 0; i < qmil.size(); i++) {
        qmIndex = qmil.at(i);
        if (qmIndex.column() == 0) {
            id = qmIndex.data().toInt();
            if (SimulationStorage::loadSimulation(simulationType, id, folderPath,  dataNames, simulationData)) {
                for (unsigned int j = 0; j < dataNames.size(); j++) {
                    if (dataNames.at(j).compare("X") == 0) {
                        xValues = simulationData.at(j);
                        break;
                    }
                }
                if (simulationType == SIMULATION_FLOW) {
                    //Flow
                    for (unsigned int j = 0; j < dataNames.size(); j++) {
                        for (unsigned int k = 0; k < _XYPlotAreas.size(); k++) {
                            if (_XYPlotAreas.at(k) != NULL) {
                                if (dataNames.at(j).contains(_XYPlotAreas.at(k)->objectName(), Qt::CaseInsensitive)) {
                                    qDebug() << "COMPAREOK" << dataNames.at(j) << " " << _XYPlotAreas.at(k)->objectName();
                                    _XYPlotAreas.at(k)->Plot()->setTitle(QString("%1 vs %2").arg(_XYPlotAreas.at(k)->objectName()).arg(variableString));
                                    _XYPlotAreas.at(k)->Plot()->setAxisTitle(QwtPlot::xBottom, variableString);
                                    _XYPlotAreas.at(k)->Plot()->addCurve(xValues, simulationData.at(j), QVariant(id).toString());
                                    break;
                                } else {
                                    qDebug() << "COMPARE" << dataNames.at(j) << " " << _XYPlotAreas.at(k)->Plot()->objectName();
                                }
                            }
                        }
                    }
                } else {
                    //Regime Transition
                    regimePlot->Plot()->setTitle(QString("%1 vs %2").arg(regimePlot->Plot()->axisTitle(QwtPlot::yLeft).text()).arg(variableString));
                    regimePlot->Plot()->setAxisTitle(QwtPlot::xBottom, variableString);
                    regimePlot->Plot()->addCurve(xValues, simulationData.at(1), QVariant(id).toString());
                }
            }
        }
    }
}

/** Add/remove old simulations from flow simulations list
*/
void MainWindow::on_checkBoxIncludeOldSimulationsFlow_clicked()
{
    qDebug() << "MainWindow::on_checkBoxIncludeOldSimulationsFlow_clicked()";
    addRemoveOldItemsSimulationList(SIMULATION_FLOW);
}

/** Add/remove old simulations from flow simulations list
*/
void MainWindow::on_checkBoxIncludeOldSimulationsRegime_clicked()
{
    qDebug() << "MainWindow::on_checkBoxIncludeOldSimulationsRegime_clicked()";
    addRemoveOldItemsSimulationList(SIMULATION_REGIMETRANSITION);
}

/** Add/remove old simulations from flow simulations list
*/
void MainWindow::addRemoveOldItemsSimulationList(simulationtype_t simulationType)
{
    qDebug() << "MainWindow::addRemoveOldItemsSimulationList()";

    QStandardItemModel *model;
    Ui::ListFrame *frame;
    int firstNewSimID;
    if (simulationType == SIMULATION_FLOW) {
        model = simulationsFlowModel;
        frame = uiFlowListFrame;
        firstNewSimID = _firstNewSimID;
    } else {
        model = simulationsRegimeModel;
        frame = uiRegimeListFrame;
        firstNewSimID = _firstNewSimRegimeID;
    }

    QDir nameDir(QDir::currentPath());
    nameDir.cd(SIMULATIONSFOLDER);

    std::vector<int> ids;
    std::vector<QString> regimeNames;
    std::vector<QString> variableNames;
    std::vector<double> variablesMin;
    std::vector<double> variablesMax;
    std::vector<double> Usgs;
    std::vector<double> Usls;
    std::vector<double> angles;
    std::vector<double> diameters;
    std::vector<double> roughnesses;

    if (frame->checkBoxIncludeOldSimulations->isChecked()) {
        SimulationStorage::listSimulationParameters(simulationType, 0, firstNewSimID-1, nameDir, ids, regimeNames, variableNames, variablesMin, variablesMax, Usgs, Usls, angles, diameters, roughnesses);
        for (int i = 0; i < ids.size(); i++) {
            QList<QStandardItem *> standardItemList;
            standardItemList.push_back(new QStandardItem(QVariant(ids.at(i)).toString()));
            standardItemList.push_back(new QStandardItem(regimeNames.at(i)));
            standardItemList.push_back(new QStandardItem(variableNames.at(i)));
            standardItemList.push_back(new QStandardItem(QVariant(variablesMin.at(i)).toString()));
            standardItemList.push_back(new QStandardItem(QVariant(variablesMax.at(i)).toString()));
            standardItemList.push_back(new QStandardItem(QVariant(Usgs.at(i)).toString()));
            standardItemList.push_back(new QStandardItem(QVariant(Usls.at(i)).toString()));
            standardItemList.push_back(new QStandardItem(QVariant(angles.at(i)).toString()));
            standardItemList.push_back(new QStandardItem(QVariant(roughnesses.at(i)).toString()));
            standardItemList.push_back(new QStandardItem(QVariant(diameters.at(i)).toString()));
            model->insertRow(i, standardItemList);
        }
    } else {
        int rows = model->rowCount();
        int id;
        std::vector<int> deleteRowIndexes;
        for (int i = 0; i < rows; i++) {
            QString asdf = model->data(model->index(i, 0)).toString();
            id = QVariant(asdf).toInt();
            if (id < firstNewSimID) {
                //deleting rows right here would offset row indexes for rest of for loop, instead
                //just list all rows and delete afterwards
                deleteRowIndexes.push_back(i);
            } else {
                //After first row which has an id higher than firstNewSimID, all rows will have so we can just break loop
                break;
            }
        }
        //walk through list of indexes to delete backwards to avoid offsetting them
        for (int i = deleteRowIndexes.size()-1; i >= 0; i--) {
            model->removeRow(i);
        }
    }
}

/** Clear selection of simulation list
*/
void MainWindow::on_pushButtonClearSelectionFlow_clicked()
{
    simulationFlowItemSelectionModel->clearSelection();
}

/** Clear selection of regime simulationlist
*/
void MainWindow::on_pushButtonClearSelectionRegime_clicked()
{
    simulationRegimeItemSelectionModel->clearSelection();
}

/** Enable and disable buttons based on selection changes in simulations list
*/
void MainWindow::on_simulationFlowItemSelectionModel_selectionChanged(QItemSelection selected, QItemSelection deselected)
{
    qDebug() << "MainWindow::on_simulationFlowItemSelectionModel_selectionChanged()";
    enableDisableSimulationListButtons(SIMULATION_FLOW);
}

/** Enable and disable buttons based on selection changes in simulations list
*/
void MainWindow::on_simulationRegimeItemSelectionModel_selectionChanged(QItemSelection selected, QItemSelection deselected)
{
    qDebug() << "MainWindow::on_simulationRegimeItemSelectionModel_selectionChanged()";
    enableDisableSimulationListButtons(SIMULATION_REGIMETRANSITION);
}

/** Enable and disable buttons based on selection changes in simulations list
*/
void MainWindow::enableDisableSimulationListButtons(simulationtype_t simulationType)
{
    Ui::ListFrame *frame;
    QItemSelectionModel *itemSelectionModel;
    if (simulationType == SIMULATION_FLOW) {
        frame = uiFlowListFrame;
        itemSelectionModel = simulationFlowItemSelectionModel;
    } else {
        frame = uiRegimeListFrame;
        itemSelectionModel = simulationRegimeItemSelectionModel;
    }
    qDebug() << "MainWindow::enableDisableSimulationListButtons()";
    QModelIndexList qmil = itemSelectionModel->selectedRows(0);
    int nSel = qmil.size();
    if (nSel == 1) {
        frame->pushButtonLoadSimulationParameters->setEnabled(true);
    } else {
        frame->pushButtonLoadSimulationParameters->setDisabled(true);
    }
    if (nSel > 0) {
        frame->pushButtonMoreInfo->setEnabled(true);
        frame->pushButtonPlotSimulations->setEnabled(true);
        frame->pushButtonRemoveSelected->setEnabled(true);
        frame->pushButtonClearSelection->setEnabled(true);
    } else {
        frame->pushButtonMoreInfo->setDisabled(true);
        frame->pushButtonPlotSimulations->setDisabled(true);
        frame->pushButtonRemoveSelected->setDisabled(true);
        frame->pushButtonClearSelection->setDisabled(true);
    }
    frame->pushButtonClearSelection->setText(QString("Clear Selection (%1)").arg(nSel));
    frame->pushButtonRemoveSelected->setText(QString("Delete Selected (%1)").arg(nSel));
}

/** Load parameters of currently selected simulation
*/
void MainWindow::on_pushButtonLoadSimulationParametersFlow_clicked()
{
    qDebug() << "MainWindow::on_pushButtonLoadSimulationParametersFlow_clicked()";
    loadSimulationListParameters(SIMULATION_FLOW);
}

/** Load parameters of currently selected simulation
*/
void MainWindow::on_pushButtonLoadSimulationParametersRegime_clicked()
{
    qDebug() << "MainWindow::on_pushButtonLoadSimulationParametersRegime_clicked()";
    loadSimulationListParameters(SIMULATION_REGIMETRANSITION);
}

/** Load parameters of currently selected simulation
*/
void MainWindow::loadSimulationListParameters(simulationtype_t simulationType)
{

    QModelIndexList qmil;
    if (simulationType == SIMULATION_FLOW) {
        qmil = simulationFlowItemSelectionModel->selectedRows(0);
    } else {
        qmil = simulationRegimeItemSelectionModel->selectedRows(0);
    }
    QModelIndex qmi = qmil.at(0);
    int simID = qmi.data().toInt();
    QDir nameDir(QDir::currentPath());
    nameDir.cd(SIMULATIONSFOLDER);
    Data * data = SimulationStorage::loadSimulationParameters(simulationType, simID, nameDir);
    if (data != NULL) {
        setWidgetValues(data);
    }
}

/** Show a dialog with more information on currently selected items in simulation list
*/
void MainWindow::on_pushButtonFlowMoreInfo_clicked()
{
    showSimulationListInformation(SIMULATION_FLOW);
}

/** Show a dialog with more information on currently selected items in simulation list
*/
void MainWindow::on_pushButtonRegimeMoreInfo_clicked()
{
    showSimulationListInformation(SIMULATION_REGIMETRANSITION);
}

/** Show a dialog with more information on currently selected items in simulation list
*/
void MainWindow::showSimulationListInformation(simulationtype_t simulationType)
{
    qDebug() << "MainWindow::showSimulationListInformation()";
    QDir nameDir(QDir::currentPath());
    nameDir.cd(SIMULATIONSFOLDER);

    int simID = -1;
    Data *data = NULL;
    QModelIndexList qmil;
    if (simulationType == SIMULATION_FLOW) {
        qmil = simulationFlowItemSelectionModel->selectedRows(0);
    } else {
        qmil = simulationRegimeItemSelectionModel->selectedRows(0);
    }
    QModelIndex qmi;
    QString message = "<html><body><font size=\"-1\"";
    for (int i = 0; i < qmil.size(); i++) {
        qmi = qmil.at(i);
        simID = qmi.data().toInt();
        data = SimulationStorage::loadSimulationParameters(simulationType, simID, nameDir);
        if (data != NULL) {
            message.append(QString("<b><u>Simulation %1</u></b><br>").arg(simID));
            message.append(data->dataInformation());
            message.append("<br>");
        }
    }
    message.append("</body></html>");

    Ui::MoreInfo *uiMoreInfo = new Ui::MoreInfo;
    QFrame *frame = new QFrame(NULL);
    uiMoreInfo->setupUi(frame);
    uiMoreInfo->webView->setHtml(message);
    frame->show();
}

/** Remove currently selected items from simulations list, also delete their sourcefiles
*/
void MainWindow::on_pushButtonRemoveSelectedFlow_clicked()
{
    qDebug() <<  "MainWindow::on_pushButtonRemoveSelected_clicked()";
    removeSelectedItemsFromList(SIMULATION_FLOW);
}

/** Remove currently selected items from simulations list, also delete their sourcefiles
*/
void MainWindow::on_pushButtonRemoveSelectedRegime_clicked()
{
    qDebug() <<  "MainWindow::on_pushButtonRemoveSelectedRegime_clicked()";
    removeSelectedItemsFromList(SIMULATION_REGIMETRANSITION);
}

void MainWindow::removeSelectedItemsFromList(simulationtype_t simulationType)
{
    qDebug() <<  "MainWindow::removeSelectedItemsFromList()";

    QModelIndexList qmil;
    QStandardItemModel *model;
    QModelIndex qmIndex;

    QItemSelectionModel *itemSelModel;
    if (simulationType == SIMULATION_FLOW) {
        itemSelModel = simulationFlowItemSelectionModel;
        model = simulationsFlowModel;
    } else {
        itemSelModel = simulationRegimeItemSelectionModel;
        model = simulationsRegimeModel;
    }

    QDir folderPath(QDir::currentPath());
    folderPath.cd(SIMULATIONSFOLDER);

    int id = -1;
    qmil = itemSelModel->selectedRows(0);
    while (qmil.size() > 0) {
        qmIndex = qmil.at(0);
        if (qmIndex.column() == 0) {
            id = qmIndex.data().toInt();
            SimulationStorage::deleteSimulation(simulationType, id, folderPath);
            model->removeRow(qmIndex.row());
        }
        qmil = itemSelModel->selectedRows(0);
    }
}

/** Show about information on MultiPhasePlot
*/
void MainWindow::on_actionAbout_MultiPhasePlot_triggered()
{
    qDebug() << "MainWindow::on_actionAbout_MultiPhasePlot_triggered()";
    QString aboutText = "<h3>MultiPhasePlot</h3><br>\n";
    aboutText.append(QString("Version %1 <br>\n").arg(VERSION));
    aboutText.append("By Marius Stene (marius.stene@gmail.com)<br>\n");
    aboutText.append("Licenced under GPL2 or later<br>\n");
    aboutText.append("http://code.google.com/p/multiphaseplot");
    QMessageBox::about(this, "About MultiPhasePlot", aboutText);
}

/** Tell calculationmanagers to quit */
void MainWindow::on_pushButtonStopSimulationFlow_clicked()
{
    cm->quit();
}

/** Simulation has been terminated, enable/disable correct buttons */
void MainWindow::on_flowSimulation_terminated()
{
    delete cm;
    ui->pushButtonSimulateFlow->setEnabled(true);
    ui->pushButtonStopSimulationFlow->setDisabled(true);
    ui->progressBarSimulation->setValue(0);
}

/** Simulation finished, if all CalculateManagers are finished, plot results
*/
void MainWindow::on_regimeSimulation_finished()
{
    for (int i = 0; i < _regimeCalculationManagers.size(); i++) {
        if (!_regimeCalculationManagers.at(i)->isFinished()) {
            _regimeCalculationManagers.at(i)->run();
            return;
        }
    }
    this->setStatusTip("Simulation finished");
    ui->progressBarSimulation->setStatusTip("Simulation finished");

    //OK, all simulations are finished
    regimePlot->Plot()->clear();
    QDir nameDir(QDir::currentPath());
    nameDir.cd(SIMULATIONSFOLDER);

    for (int i = 0; i < _regimeCalculationManagers.size(); i++) {
        //Plot
        regimePlot->Plot()->addCurve(_regimeCalculationManagers.at(i)->xList(), _regimeCalculationManagers.at(i)->yList(), _regimeCalculationManagers.at(i)->calculationName());
        ui->tabWidgetScheme->setCurrentIndex(1);
        //Store data
        SimulationStorage *ss = new SimulationStorage(SIMULATION_REGIMETRANSITION, _regimeCalculationManagers.at(i)->simulationID(),
                                                      nameDir, _regimeCalculationManagers.at(i)->data());
        ss->addData("X", _regimeCalculationManagers.at(i)->xList());
        ss->addData("Usl", _regimeCalculationManagers.at(i)->yList());
        ss->store(false);
        delete ss;
        ss = NULL;

        //Add to treeview
        int row = simulationsRegimeModel->rowCount();
        simulationsRegimeModel->setItem(row, 0, new QStandardItem(QVariant(_regimeCalculationManagers.at(i)->simulationID()).toString()));
        simulationsRegimeModel->setItem(row, 1, new QStandardItem(_regimeCalculationManagers.at(i)->pluginName()));
        simulationsRegimeModel->setItem(row, 2, new QStandardItem(_regimeCalculationManagers.at(i)->xVariableName()));
        simulationsRegimeModel->setItem(row, 3, new QStandardItem(QVariant(_regimeCalculationManagers.at(i)->data()->variableMin()).toString()));
        simulationsRegimeModel->setItem(row, 4, new QStandardItem(QVariant(_regimeCalculationManagers.at(i)->data()->variableMax()).toString()));
        simulationsRegimeModel->setItem(row, 5, new QStandardItem(QVariant(_regimeCalculationManagers.at(i)->data()->Usg()).toString()));
        simulationsRegimeModel->setItem(row, 6, new QStandardItem(QVariant(_regimeCalculationManagers.at(i)->data()->Usl()).toString()));
        simulationsRegimeModel->setItem(row, 7, new QStandardItem(QVariant(_regimeCalculationManagers.at(i)->data()->angle()).toString()));
        simulationsRegimeModel->setItem(row, 8, new QStandardItem(QVariant(_regimeCalculationManagers.at(i)->data()->roughness()).toString()));
        simulationsRegimeModel->setItem(row, 9, new QStandardItem(QVariant(_regimeCalculationManagers.at(i)->data()->diameter()).toString()));
    }
    for (int i = 0; i < _regimeCalculationManagers.size(); i++) {
        delete _regimeCalculationManagers.at(i);
    }
    _regimeCalculationManagers.clear();
    ui->pushButtonSimulateRegime->setEnabled(true);
    ui->pushButtonStopSimulationRegime->setDisabled(true);
}

/** Slot which updates the progressBar to reflect the simulation progression based on feedback from CalculateManager through progressed() signal
*   @param asdf Since regime transition has multiple CalculateManager we can't use this value directly but have to call each CalculateManager to retrieve current progress
*/
void MainWindow::on_regimeSimulation_progressed(int asdf)
{
    int progress = 0;
    QStringList qsl;
    for (int i = 0; i < _regimeCalculationManagers.size(); i++) {
        progress += _regimeCalculationManagers.at(i)->progress();
        if (_regimeCalculationManagers.at(i)->progress() < 100) {
            qsl.append(_regimeCalculationManagers.at(i)->pluginName());
        }
    }
    ui->progressBarSimulation->setValue(progress/_regimeCalculationManagers.size());
    QString list = qsl.join(", ");
    this->setStatusTip(QString("Simulating (%1 not yet finished)...").arg(list));
    ui->progressBarSimulation->setStatusTip(QString("Simulating (%1 not yet finished)...").arg(list));
}

/** Tell all CalculateManagers to quit */
void MainWindow::on_pushButtonStopSimulationRegime_clicked()
{
    if (_regimeCalculationManagers.size() == 0) {
        on_regimeSimulation_terminated();
    }
    for (int i = 0; i < _regimeCalculationManagers.size(); i++) {
        _regimeCalculationManagers.at(i)->quit();
    }
}

/** Simulation has been terminated, enable/disable correct buttons */
void MainWindow::on_regimeSimulation_terminated()
{
    for (int i = 0; i < _regimeCalculationManagers.size(); i++) {
        if (!_regimeCalculationManagers.at(i)->isTerminated())
            return;
    }
    for (int i = 0; i < _regimeCalculationManagers.size(); i++) {
        delete _regimeCalculationManagers.at(i);
    }
    ui->pushButtonSimulateRegime->setEnabled(true);
    ui->pushButtonStopSimulationRegime->setDisabled(true);
    ui->progressBarSimulation->setValue(0);
}

/** Show error message sent from calculation thread */
void MainWindow::on_simulationError(QString errorMsg)
{
    //errorMessage->showMessage(errorMsg);
    errorMsg.replace("&", "");
    QMessageBox::critical(NULL, tr("MultiPhasePlot"), errorMsg, QMessageBox::Ok, QMessageBox::Ok);
}

/** Open default system browser with source documentation */
void MainWindow::on_actionSource_code_documentation_triggered()
{
    //QString dirS = QDir(QDir::currentPath()).absolutePath();
    //dirS.append("docs/index.html");
    //dirS.prepend("file:///");
    QDesktopServices::openUrl(QUrl(QString("http://ibar.sf.net")));
}

void MainWindow::on_checkBoxRegimeUseMinimumHoldup_clicked()
{
    if (ui->checkBoxRegimeUseMinimumHoldup->isChecked()) {
        ui->checkBoxRegimeAnnular->setEnabled(true);
        ui->checkBoxRegimeDispersed->setEnabled(true);
        ui->checkBoxRegimeStratified->setEnabled(true);
        ui->checkBoxRegimeSlug->setEnabled(true);
        ui->radioButtonRegimeAnnular->setDisabled(true);
        ui->radioButtonRegimeDispersed->setDisabled(true);
        ui->radioButtonRegimeStratified->setDisabled(true);
        ui->radioButtonRegimeSlug->setDisabled(true);
    } else {
        ui->checkBoxRegimeAnnular->setDisabled(true);
        ui->checkBoxRegimeDispersed->setDisabled(true);
        ui->checkBoxRegimeStratified->setDisabled(true);
        ui->checkBoxRegimeSlug->setDisabled(true);
        ui->radioButtonRegimeAnnular->setEnabled(true);
        ui->radioButtonRegimeDispersed->setEnabled(true);
        ui->radioButtonRegimeStratified->setEnabled(true);
        ui->radioButtonRegimeSlug->setEnabled(true);
    }
}
