/**
 * @file
 * @author  Marius Stene <marius.stene at gmail.com>
 * @version 1.0
 *
 * @section LICENSE
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details at
 * http://www.gnu.org/copyleft/gpl.html
 *
 * @section DESCRIPTION
 * Window class for main window
 */

#ifndef MAINWINDOW_H
#define MAINWINDOW_H


#include <QtGui>
#include <QDir>
#include <QPluginLoader>
#include <QFileInfoList>
#include <QDebug>
#include <QSettings>
#include <QStringList>
#include <QMap>
#include <QWhatsThis>
#include <QPlastiqueStyle>
#include <QWindowsVistaStyle>
#include <QStandardItem>
#include <QStandardItemModel>
#include <QModelIndexList>
#include <QItemSelectionModel>
#include <QDesktopServices>
#include <QErrorMessage>
#include <QVBoxLayout>
#include <iostream>
#include <stdio.h>
#include <vector>
#include <stack>

#include "defines.h"
#include "calculatemanager.h"

#include "simulationstorage.h"
#include "xyplot.h"
#include "xyplotarea.h"
#include "ui_listframe.h"
#include "ui_moreinfo.h"
#include "../../Support/calculatefunctions.h"
#include "../../Support/data.h"
#include "../../Support/frictioninterface.h"
#include "../../Support/calculateregimeinterface.h"
#include "../../Support/interfacefrictioninterface.h"
#include "../../Support/frictionfunctions.h"
#include "../../Support/regimetransitioninterface.h"

namespace Ui {
    class MainWindow;
}

class MainWindow : public QMainWindow {
    Q_OBJECT
public:
    MainWindow(QWidget *parent = 0);
    ~MainWindow();

protected:
    void changeEvent(QEvent *e);

private:
    Ui::MainWindow *ui;
    Ui::ListFrame *uiRegimeListFrame;
    Ui::ListFrame *uiFlowListFrame;
    CalculateManager *cm;
    QErrorMessage *errorMessage;

    QSplitter *regimeSplitter;
    QSplitter *flowSplitter;
    QTabWidget *tabWidgetFlow;

    XYPlotArea *regimePlot;
    XYPlotArea *holdupPlot;
    XYPlotArea *pressurePlot;
    XYPlotArea *alphaPlot;

    QStandardItemModel *simulationsFlowModel;
    QStandardItemModel *simulationsRegimeModel;
    QItemSelectionModel *simulationFlowItemSelectionModel;
    QItemSelectionModel *simulationRegimeItemSelectionModel;

    void addPluginComboBoxItems();
    void addPredefinesComboBoxItems(flowtype_t flowType, bool newItem = false, QString focusComboBoxName = "");
    void addRegimeCheckBoxes();
    void loadWidgetValues();
    void saveWidgetValues();
    void saveGasDefinition(QString definitionName);
    void saveLiquidDefinition(QString definitionName);

    void enableDisableSimulationListButtons(simulationtype_t simulationType);
    void plotSelectedSimulations(simulationtype_t simulationType);
    void showSimulationListInformation(simulationtype_t simulationType);
    void removeSelectedItemsFromList(simulationtype_t simulationType);
    void loadSimulationListParameters(simulationtype_t simulationType);
    void addRemoveOldItemsSimulationList(simulationtype_t simulationType);

    void getWidgetValues(Data *data);
    void setWidgetValues(const Data *data);
    void saveGasDefinition(QString name, QString density, QString viscosity, QString temperature, QString pressure,
                           QString densityMin = "", QString viscosityMin = "", QString temperatureMin = "", QString pressureMin = "",
                           QString densityMax = "", QString viscosityMax = "", QString temperatureMax = "", QString pressureMax = "");
    void saveLiquidDefinition(QString name, QString density, QString viscosity, QString densityMin = "", QString viscosityMin = "",
                              QString densityMax = "", QString viscosityMax = "");
    bool isInputValid(simulationtype_t simulationType);
    void firstRunSetup();
    regimetype_t getRegimeType();
    scale_t getScaleType();

    FrictionFunctions *getSelectedFrictionPlugins(std::vector<FrictionInterface *> frictionPlugins,
                                                  std::vector<InterfaceFrictionInterface *> interfaceFrictionPlugins,
                                                  std::vector<SlipRelationInterface *> slipRelationFrictionPlugins);
    CalculateFunctions *getSelectedCalculatePlugins(std::vector<CalculateRegimeInterface *> calculatePlugins);
    std::vector<FrictionInterface *> frictionInterfacePlugins;
    std::vector<InterfaceFrictionInterface*> interfaceFrictionInterfacePlugins;
    std::vector<SlipRelationInterface*> slipRelationInterfacePlugins;
    std::vector<RegimeTransitionInterface *> regimeInterfacePlugins;
    std::vector<CalculateRegimeInterface *> calculateInterfacePlugins;

    std::vector<QRadioButton *> _radioButtonsVariable;
    std::vector<double> _variableScaler;

    std::vector<QLineEdit *> _lineEdits;
    std::vector<QLineEdit *> _lineEditsMin;
    std::vector<QLineEdit *> _lineEditsMax;
    std::vector<QLineEdit *> _lineEditsAll;
    std::vector<QCheckBox *> _regimeCheckBoxes;
    std::vector<CalculateManager *> _regimeCalculationManagers;
    QStringList _settingNamesLineEdits;
    QStringList _settingNamesLineEditsMin;
    QStringList _settingNamesLineEditsMax;
    QStringList _settingNamesLineEditsAll;
    QStringList _settingDefaultsLineEdits;
    QStringList _settingDefaultsLineEditsMin;
    QStringList _settingDefaultsLineEditsMax;
    QStringList _settingDefaultsLineEditsAll;
    QStringList _settingNamesRadioBoxes;


    std::vector<QRadioButton *> _radioButtonsRegime;
    std::vector<XYPlotArea *> _XYPlotAreas;

    unsigned int _simID;
    unsigned int _firstNewSimID;
    unsigned int _simRegimeID;
    unsigned int _firstNewSimRegimeID;

private slots:

    void on_checkBoxRegimeUseMinimumHoldup_clicked();
    void on_actionSource_code_documentation_triggered();
    void on_actionAbout_MultiPhasePlot_triggered();
    void on_actionExit_triggered();
    void on_actionWhats_this_triggered();
    void on_pushButtonRemoveSelectedFlow_clicked();
    void on_pushButtonRemoveSelectedRegime_clicked();
    void on_pushButtonFlowMoreInfo_clicked();
    void on_pushButtonRegimeMoreInfo_clicked();
    void on_pushButtonLoadSimulationParametersFlow_clicked();
    void on_pushButtonLoadSimulationParametersRegime_clicked();
    void on_simulationFlowItemSelectionModel_selectionChanged(QItemSelection, QItemSelection );
    void on_simulationRegimeItemSelectionModel_selectionChanged(QItemSelection , QItemSelection );
    void on_checkBoxIncludeOldSimulationsFlow_clicked();
    void on_checkBoxIncludeOldSimulationsRegime_clicked();
    void on_pushButtonPlotSimulations_clicked();
    void on_pushButtonPlotSimulationsRegime_clicked();
    void on_pushButtonSimulateFlow_clicked();
    void on_radioButtonVariable_clicked();
    void on_flowSimulation_progressed(int percent);
    void on_flowSimulation_finished();
    void on_pushButtonStopSimulationFlow_clicked();
    void on_flowSimulation_terminated();
    void on_pushButtonSimulateRegime_clicked();
    void on_pushButtonClearSelectionFlow_clicked();
    void on_pushButtonClearSelectionRegime_clicked();    
    void on_pushButtonDeleteLiquidPredefines_clicked();
    void on_pushButtonDeleteGasPredefines_clicked();
    void on_pushButtonLoadLiquidPredefines_clicked();
    void on_pushButtonSaveLiquidPredefine_clicked();
    void on_pushButtonLoadGasPredefines_clicked();
    void on_pushButtonSaveGasPredefine_clicked();
    void on_regimeSimulation_finished();
    void on_regimeSimulation_progressed(int asdf);
    void on_pushButtonStopSimulationRegime_clicked();
    void on_regimeSimulation_terminated();
    void on_simulationError(QString);


};

#endif // MAINWINDOW_H
