/**
 * @file
 * @author  Marius Stene <marius.stene at gmail.com>
 * @version 1.0
 *
 * @section LICENSE
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details at
 * http://www.gnu.org/copyleft/gpl.html
 *
 * @section DESCRIPTION
 * Template class for loading plugins
 */

#ifndef PLUGINLOADER_H
#define PLUGINLOADER_H

#include <QDir>
#include <QFile>
#include <QDebug>
#include <QFileInfo>
#include <QPluginLoader>
#include <QString>
#include <QStringList>

#include <vector>

#include "../../Support/plugininterface.h"

template<class T>
/** @brief Template class for loading plugins */
class PluginLoader
{
public:
    static void     loadPlugins(QDir pluginDir, std::vector<T> &retVector, QStringList filter = (QStringList("*")));
    static QObject* loadPlugin(QFileInfo filePath);
};

template<class T>
/** Static function that loads the plugins in pluginDir to retVector
    @param pluginDir QDir with path to directory where all files with names that if in filter List will be loaded
    @param retVector Vector of pointers to plugin class inherited from PluginInterface, e.g. <FrictionInterface *>
    @param filter QStringList of filters for what files to load
*/
void PluginLoader<T>::loadPlugins(QDir pluginDir, std::vector<T> &retVector, QStringList filter)
{
    qDebug() << "PluginLoader<T>::loadPlugins()";
    pluginDir.setFilter(QDir::Files);
    pluginDir.setNameFilters(filter);

    std::vector<T> pluginsFriction;

    QFileInfoList fileList = pluginDir.entryInfoList();
    QObject *plugin;
    for (int i = 0; i < fileList.size(); ++i) {
        QFileInfo fileInfo = fileList.at(i);
        plugin = loadPlugin(fileInfo);
        if (plugin != NULL) {
            pluginsFriction.push_back(qobject_cast<T>(plugin));
        }
    }
    retVector.swap(pluginsFriction);

}

template<class T>
QObject * PluginLoader<T>::loadPlugin(QFileInfo fileInfo)
{
    QObject *plugin;
    QPluginLoader pluginLoader(fileInfo.absoluteFilePath());
    pluginLoader.load();
    if (pluginLoader.isLoaded()) {
        plugin = pluginLoader.instance();
        if (plugin) {
            qDebug() << "Plugin " << fileInfo.fileName() << " successfully loaded";
            return plugin;
        } else {
            qDebug() << "- Loading of plugin " << fileInfo.fileName() << " failed, no instance";
        }
    } else {
        qDebug() << "- Loading of plugin " << fileInfo.fileName() << " failed, not successfully loaded";
        qDebug() << "Reason: " << pluginLoader.errorString();
    }
    return NULL;
}

#endif // PLUGINLOADER_H
