/*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License as
* published by the Free Software Foundation; either version 2 of
* the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* General Public License for more details at
* http://www.gnu.org/copyleft/gpl.html
*
* Author: Marius Stene
* Email: marius.stene at gmail dot com
*
*/

/*! @class XYPlot
 *  @brief Class for storing and retrieving simulation data as csv files
 */

#include "simulationstorage.h"

/** Constructor for storing simulations
*   @param simulationType What type of simulation has been run, of type simulationtype_t (SIMULATION_FLOW or SIMULATION_REGIMETRANSITION)
*   @param simulationID Unique identification number of simulation
*   @param folderPath Folder path to the file where the simulation will be stored
*   @param data Pointer to Data class with the parameters used to make simulation, const
*/
SimulationStorage::SimulationStorage(simulationtype_t simulationType, int simulationID, QDir folderPath, const Data *data)
                    : _simulationType(simulationType), _simulationID(simulationID), _folderPath(folderPath)
{
    qDebug() << "SimulationStorage::SimulationStorage()";
    _numData = 0;
    _lengthData = -1;
    _data = new Data(data);
    _fileName = fileNamePrefix(_simulationType);
    _fileName.append(QVariant(_simulationID).toString());
    _fileName.append(".csv");
}

/** Clean up, delete Data */
SimulationStorage::~SimulationStorage()
{
    delete _data;
}

/** Add data to the simulation storage
*   Only store() will actually write data to disk
*   @param dataName Name (description) on what the data is
*   @param simulationData Vector of doubles with the data to store
*/
bool SimulationStorage::addData(QString dataName,
                                std::vector<double> simulationData)
{
    qDebug() << "SimulationStorage::addData()";
    if (_lengthData < 0) {
        _lengthData = simulationData.size();
    } else {
        if (_lengthData != simulationData.size()) {
            //Data not the same length, will not accept it
            qDebug() << "Invalid data submitted";
            return false;
        }
    }
    _dataName.push_back(dataName);
    _simulationData.push_back(simulationData);
    _numData += 1;
    return true;
}

/** Write the data to disk
*   Disk format is a semicolon separated  csv file */
void SimulationStorage::store(bool flow)
{
    qDebug() << "SimulationStorage::store()";
    QFile file(_folderPath.filePath(_fileName));
    if (!file.open(QIODevice::WriteOnly | QIODevice::Text))
        return;

    QTextStream out(&file);

    //Heading
    QString _name(_fileName);
    _name.replace(fileNamePrefix(_simulationType), "");
    _name.replace(".csv", "");
    out << "\"" << _name << "\"";
    for (unsigned int i = 0; i < _numData-1; i++) {
        out << ";";
    }
    out << "\n";

    //Simulation specifications and parameters
    _data->writeData(out, flow);

    //data names
    std::vector<QString>::const_iterator iterName;
    iterName = _dataName.begin();
    for (std::vector<std::vector<double> >::size_type u = 0; u < _simulationData.size(); ) {
        out << "\"" << *iterName << "\"";
        u++;
        if (u != _simulationData.size()) {
            out << ";";
        }
        ++iterName;
    }
    out << "\n";

    //data numbers
    for (unsigned int i = 0; i < _lengthData; ) {
        for (unsigned int j = 0; j < _numData; ) {
            out << _simulationData[j][i];
            j++;
            if (j != _numData)
                out << ";";
        }
        i++;
        if (i != _lengthData)
            out << "\n";
    }
//for (std::vector<double>::size_type v = 0; v < _simulationData[u].size(); v++)
    file.close();
}

/** Read all simulations saved with identification numbers between idStart and idEnd and puts the data into std::vectors of double and QString
*   This function is intended to be used for loading data into listview in MainWindow
*   @param simulationType What type of simulation has been run, of type simulationtype_t (SIMULATION_FLOW or SIMULATION_REGIMETRANSITION)
*   @param idStart Integer id of first simulation to load (list)
*   @param idEnd Integer id of last simulation to load (list)
*   @param folderPath Folder path to the file where the simulation will be stored
*   @param ids Vector of integers to where all ids will be written, data already in vector will be removed (Applies to all following vectors as well)
*   @param regimeNames Vector of QString to where all regime names will be written (shortened to 4 chars)
*   @param variableNames Vector of QString to where all variable names will be written
*   @param variablesMin Vector of double to where all minimum variables values will be written
*   @param variablesMax Vector of double to where all maximum variables values will be written
*   @param Usgs Vector of double to where all Usg values will be written
*   @param Usls Vector of double to where all Usl values will be written
*   @param angles Vector of double to where all angles values will be written
*   @param diameters Vector of double to where all diameters values will be written
*   @param roughnesses Vector of double to where all roughnesses values will be written
*/
void SimulationStorage::listSimulationParameters(simulationtype_t simulationType, int idStart, int idEnd, QDir folderPath,
                                                 std::vector<int> &ids, std::vector<QString> &regimeNames,
                                                 std::vector<QString> &variableNames, std::vector<double> &variablesMin, std::vector<double> &variablesMax,
                                                 std::vector<double> &Usgs, std::vector<double> &Usls, std::vector<double> &angles,
                                                 std::vector<double> &diameters, std::vector<double> &roughnesses)
{
    qDebug() << "SimulationStorage::listSimulationParameters()";
    Data *data = NULL;
    ids.clear();
    regimeNames.clear();
    variableNames.clear();
    variablesMin.clear();
    variablesMax.clear();
    Usgs.clear();
    Usls.clear();
    angles.clear();
    diameters.clear();
    roughnesses.clear();

    bool isflow = false;
    if (simulationType == SIMULATION_FLOW) {
        isflow = true;
    }

    for (int i = idStart; i <= idEnd; i++) {
        QString fileName = fileNamePrefix(simulationType);
        fileName.append(QVariant(i).toString());
        fileName.append(".csv");
        QFile file(folderPath.filePath(fileName));
        if (file.open(QIODevice::ReadOnly | QIODevice::Text)) {
            QTextStream in(&file);
            in.readLine();
            in.readLine();
            data = Data::readData(in.readLine(), isflow);
            if (data != NULL) {
                ids.push_back(i);
                if (simulationType == SIMULATION_FLOW) {
                    regimeNames.push_back(data->regimeNameShort(4));
                } else {
                    regimeNames.push_back(data->regimeTransitionName());
                }
                variableNames.push_back(data->variableName());
                variablesMin.push_back(data->variableMin());
                variablesMax.push_back(data->variableMax());
                Usgs.push_back(data->Usg());
                Usls.push_back(data->Usl());
                angles.push_back(data->angle());
                diameters.push_back(data->diameter());
                roughnesses.push_back(data->roughness());
                delete data;
                data = NULL;
            }
        }
    }
}

/**  */
bool SimulationStorage::loadSimulation(simulationtype_t simulationType, int simulationID, QDir folderPath, std::vector<QString> &dataNames, std::vector<std::vector<double> > &simulationData)
{
    qDebug() << "SimulationStorage::loadSimulation()";

    dataNames.clear();
    simulationData.clear();

    QString fileName = fileNamePrefix(simulationType);;
    fileName.append(QVariant(simulationID).toString());
    fileName.append(".csv");
    QFile file(folderPath.filePath(fileName));
    if (!file.open(QIODevice::ReadOnly | QIODevice::Text))
        return false;

    //Read file and throw away first three lines
    QTextStream in(&file);
    in.readLine();
    in.readLine();
    in.readLine();

    //Read data names
    QString st = in.readLine();
    st.replace("\"", "");
    QStringList qsl = st.split(";");
    int nData = qsl.size();
    for (int i = 0; i < qsl.size(); i++) {
        dataNames.push_back(qsl.at(i).toLocal8Bit().constData());
    }

    //Read data
    std::vector<QString> tempQ;
    do {
        st = in.readLine();
        if (!st.isNull())
            tempQ.push_back(st);
    } while (!st.isNull());

    std::vector<double> tempV;
    for (int i = 0; i < nData; i++) {
        for (unsigned int j = 0; j < tempQ.size(); j++) {
            st = tempQ.at(j);
            qsl = st.split(";");
            tempV.push_back(QVariant(qsl.at(i)).toDouble());
        }
        simulationData.push_back(tempV);
        tempV.clear();
    }
    return true;
}

bool SimulationStorage::deleteSimulation(simulationtype_t simulationType, int simulationID, QDir folderPath)
{
    qDebug() << "SimulationStorage::deleteSimulation()";
    QString fileName = fileNamePrefix(simulationType);;
    fileName.append(QVariant(simulationID).toString());
    fileName.append(".csv");
    QFile file(folderPath.filePath(fileName));
    if (!file.exists())
        return true;
    return (file.remove());
}

Data * SimulationStorage::loadSimulationParameters(simulationtype_t simulationType, int simulationID, QDir folderPath)
{
    qDebug() << "SimulationStorage::loadSimulation()";
    Data *data = NULL;
    QString fileName = fileNamePrefix(simulationType);;
    fileName.append(QVariant(simulationID).toString());
    fileName.append(".csv");
    QFile file(folderPath.filePath(fileName));
    if (!file.open(QIODevice::ReadOnly | QIODevice::Text))
        return NULL;

    //Read file and throw away first two lines
    QTextStream in(&file);
    in.readLine();
    in.readLine();
    data = Data::readData(in.readLine());
    return data;
}

QString SimulationStorage::fileNamePrefix(simulationtype_t simulationType)
{
    if (simulationType == SIMULATION_FLOW) {
        return QString("sim");
    } else {
        return QString("regime");
    }
}
