/*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License as
* published by the Free Software Foundation; either version 2 of
* the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* General Public License for more details at
* http://www.gnu.org/copyleft/gpl.html
*
* Author: Marius Stene
* Email: marius.stene at gmail dot com
*
*/

#include "xyplot.h"

/*! @class XYPlotZoomer
 *  @brief Class which facilitates zooming in XYPlots
 */
class XYPlotZoomer: public QwtPlotZoomer
{
public:
    XYPlotZoomer(QwtPlotCanvas *canvas):
        QwtPlotZoomer(canvas)
    {
        setTrackerMode(AlwaysOn);
    }

    virtual QwtText trackerText(const QPointF &pos) const
    {
        QColor bg(Qt::white);
        bg.setAlpha(200);

        QwtText text = QwtPlotZoomer::trackerText(pos);
        text.setBackgroundBrush( QBrush( bg ));
        return text;
    }
};

/*! @class XYPlot
 *  @brief Class for creating simple plots, based on QwtPlot
 */

/** Constructor that creates plot base with title and labels
*   @param parent Parent Widget (parent window)
*   @param title Title of plot
*   @param xAxisLabel Label of x axis
*   @param yAxisLabel Title of y axis
*/
XYPlot::XYPlot(QWidget *parent, QString title, QString xAxisLabel, QString yAxisLabel) : QwtPlot(parent)
{
    qDebug() << "XYPlot::XYPlot()";
    //Object name is used in mainwindow to figure out what plot this is, to know which plot to plot holdup to and so on.
    this->setObjectName(title);
    //Assign a title
    setTitle(title);
    //Assign labels
    if (xAxisLabel.compare("") != 0)
        setAxisTitle(QwtPlot::xBottom, xAxisLabel);
    if (yAxisLabel.compare("") != 0)
        setAxisTitle(QwtPlot::yLeft, yAxisLabel);

    _curves = 0;
    //Qt::SolidLine, Qt::DashLine, Qt::DotLine, Qt::DashDotLine, Qt::DashDotDotLine
    //Qt::BevelJoin, Qt::RoundJoin

    _pens.push_back(QPen(Qt::black, 1, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin));
    _pens.push_back(QPen(Qt::red, 1, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin));
    _pens.push_back(QPen(Qt::green, 1, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin));
    _pens.push_back(QPen(Qt::blue, 1, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin));
    _pens.push_back(QPen(Qt::darkMagenta, 1, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin));
    _pens.push_back(QPen(Qt::gray, 1, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin));
    _pens.push_back(QPen(Qt::darkGreen, 1, Qt::DotLine, Qt::RoundCap, Qt::RoundJoin));
    _pens.push_back(QPen(Qt::darkYellow, 1, Qt::DotLine, Qt::RoundCap, Qt::RoundJoin));
    _pens.push_back(QPen(Qt::magenta, 1, Qt::DotLine, Qt::RoundCap, Qt::RoundJoin));
    _pens.push_back(QPen(Qt::darkBlue, 1, Qt::DotLine, Qt::RoundCap, Qt::RoundJoin));
    _pens.push_back(QPen(Qt::cyan, 1, Qt::DotLine, Qt::RoundCap, Qt::RoundJoin));
    _pens.push_back(QPen(Qt::black, 1, Qt::DashLine, Qt::RoundCap, Qt::RoundJoin));
    _pens.push_back(QPen(Qt::red, 1, Qt::DashLine, Qt::RoundCap, Qt::RoundJoin));
    _pens.push_back(QPen(Qt::green, 1, Qt::DashLine, Qt::RoundCap, Qt::RoundJoin));
    _pens.push_back(QPen(Qt::blue, 1, Qt::DashLine, Qt::RoundCap, Qt::RoundJoin));
    _pens.push_back(QPen(Qt::darkMagenta, 1, Qt::DashLine, Qt::RoundCap, Qt::RoundJoin));
    _pens.push_back(QPen(Qt::gray, 1, Qt::DashLine, Qt::RoundCap, Qt::RoundJoin));
    _pens.push_back(QPen(Qt::darkRed, 1, Qt::DotLine, Qt::RoundCap, Qt::RoundJoin));
    _pens.push_back(QPen(Qt::darkGray, 1, Qt::DashDotDotLine, Qt::RoundCap, Qt::RoundJoin));
    _pens.push_back(QPen(Qt::lightGray, 1, Qt::DashDotDotLine, Qt::RoundCap, Qt::RoundJoin));
    /*
    // We don't need the cache here
    canvas()->setPaintAttribute(QwtPlotCanvas::PaintCached, false);
    canvas()->setPaintAttribute(QwtPlotCanvas::PaintPacked, false);
*/

    insertLegend(new QwtLegend(), QwtPlot::RightLegend);
    setAutoReplot(true);

    zoomer = new XYPlotZoomer(canvas());

    zoomer->setMousePattern(QwtEventPattern::MouseSelect2,
        Qt::RightButton, Qt::ControlModifier);
    zoomer->setMousePattern(QwtEventPattern::MouseSelect3,
        Qt::RightButton);

    QwtPlotPanner *panner = new QwtPlotPanner(canvas());
    panner->setAxisEnabled(QwtPlot::yRight, false);
    panner->setMouseButton(Qt::MidButton);

    /*
    //  ...a horizontal line at y = 0...
    QwtPlotMarker *mY = new QwtPlotMarker();
    mY->setLabel(QString::fromLatin1("y = 0"));
    mY->setLabelAlignment(Qt::AlignRight|Qt::AlignTop);
    mY->setLineStyle(QwtPlotMarker::HLine);
    mY->setYValue(0.0);
    mY->attach(this);

    //  ...a vertical line at x = 2 * pi
    QwtPlotMarker *mX = new QwtPlotMarker();
    mX->setLabel(QString::fromLatin1("x = 2 pi"));
    mX->setLabelAlignment(Qt::AlignLeft | Qt::AlignBottom);
    mX->setLabelOrientation(Qt::Vertical);
    mX->setLineStyle(QwtPlotMarker::VLine);
    mX->setLinePen(QPen(Qt::black, 0, Qt::DashDotLine));
    mX->setXValue(2.0 * M_PI);
    mX->attach(this);
    */
    setAxisMaxMajor(QwtPlot::xBottom, 6);
    setAxisMaxMinor(QwtPlot::xBottom, 10);
    //QwtLog10ScaleEngine * qloge = new QwtLog10ScaleEngine;
    //setAxisScaleEngine(QwtPlot::xBottom, qloge);
    this->setCanvasBackground(Qt::white);

}

/** Adds a curve to plot, a plot can have an unlimited number of curves
*   @param xValuesList A vector of doubles of x values for the curve
*   @param yValuesList A vector of doubles of y values for the curve
*   @param legend Curve legend (
*/
void XYPlot::addCurve(std::vector<double> xValuesList, std::vector<double> yValuesList, QString legend)
{
    qDebug() << "XYPlot::addCurve()";
    QwtPlotCurve *plotCurve = new QwtPlotCurve(legend);
    plotCurve->setRenderHint(QwtPlotItem::RenderAntialiased);
    plotCurve->setLegendAttribute(QwtPlotCurve::LegendShowLine, true);
    if (_curves < _pens.size()) {
        plotCurve->setPen(_pens.at(_curves));
        _curves++;
    }
    plotCurve->attach(this);

    unsigned int size = xValuesList.size();
    if (size > yValuesList.size())
        size = yValuesList.size();

    double *x = new double[size];
    double *y = new double[size];

    std::vector<double>::const_iterator iter;
    int i = 0;
    for(iter = xValuesList.begin(); iter != xValuesList.end(); ++iter) {
        x[i] = *iter;
        i++;
    }
    i = 0;
    for(iter = yValuesList.begin(); iter != yValuesList.end(); ++iter) {
        y[i] = *iter;
        i++;
    }

    // Create sin and cos data
    if (size > 0)
        plotCurve->setRawSamples(x, y, size);

    //Make sure you can't zoom out more than to cover the entire curve.
    //Next zoom out level would be zoom entirely out = BAD
    zoomer->setZoomBase(true);
    this->setAxisAutoScale(QwtPlot::xBottom);
    this->setAxisAutoScale(QwtPlot::yLeft);
}

/** Remove all curves plotted.
    Reimplemented to reset _curves to zero so that curve colours are the same when starting a new plot */
void XYPlot::clear()
{
    qDebug() << "XYPlot::clear()";
    _curves = 0;
    QwtPlot::clear();
}

/** Returns a pointer to the QwtPlot */
QwtPlot * XYPlot::Plot()
{
    qDebug() << "XYPlot::Plot()";
    return this;
}
