/*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License as
* published by the Free Software Foundation; either version 2 of
* the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* General Public License for more details at
* http://www.gnu.org/copyleft/gpl.html
*
* Author: Marius Stene
* Email: marius.stene at gmail dot com
*
*/

/*! @class XYPlotArea
 *  @brief Widget class which adds a XYPlot together with buttons for manipulating plot
 */
#include "xyplotarea.h"

/** Constructor that creates plot area with title and labels
*   @param parent Parent Widget (parent window)
*   @param title Title of plot
*   @param xAxisLabel Label of x axis
*   @param yAxisLabel Title of y axis
*/
XYPlotArea::XYPlotArea(QWidget *parent, QString title, QString xAxisLabel, QString yAxisLabel)
{
    this->setObjectName(title);

    //Plot stuff
    plot = new XYPlot(parent, title, xAxisLabel, yAxisLabel);
    plot->setAutoReplot(true);
    plot->insertLegend(new QwtLegend(), QwtPlot::BottomLegend);

    //Actual Widgets
    QRadioButton *radioButtonYLog = new QRadioButton("&Logarithmic", parent);
    QRadioButton *radioButtonYLin = new QRadioButton("L&inear", parent);
    radioButtonYLin->setChecked(true);
    QRadioButton *radioButtonXLog = new QRadioButton("Lo&garithmic", parent);
    QRadioButton *radioButtonXLin = new QRadioButton("Li&near", parent);
    radioButtonXLin->setChecked(true);
    QRadioButton *radioButtonLegendRight = new QRadioButton("&Right", parent);
    QRadioButton *radioButtonLegendBottom = new QRadioButton("&Bottom", parent);
    radioButtonLegendBottom->setChecked(true);
    lineEditXSize = new QLineEdit("800", parent);
    lineEditYSize = new QLineEdit("600", parent);
    QLabel *labelX = new QLabel("X:", parent);
    QLabel *labelY = new QLabel("Y:", parent);
    QPushButton *exportButton = new QPushButton(tr("&Save as"));
    QMenu *menu = new QMenu(this);
    QAction *saveActPng = new QAction(tr("&Save as png..."), this);
    saveActPng->setShortcuts(QKeySequence::Save);
    QAction *saveActSvg = new QAction(tr("&Save as svg..."), this);
    menu->addAction(saveActPng);
    menu->addAction(saveActSvg);
    exportButton->setMenu(menu);

    //GroupBoxes
    QGroupBox *groupBoxY = new QGroupBox("Y Axis Scale", parent);
    QGroupBox *groupBoxX = new QGroupBox("X Axis Scale", parent);
    QGroupBox *groupBoxLegend = new QGroupBox("Legend", parent);
    QGroupBox *groupBoxExport = new QGroupBox("Export Plot", parent);
    QGroupBox *groupBoxExportSize = new QGroupBox("Image Size", parent);
    QVBoxLayout *vYLayout = new QVBoxLayout(parent);
    QVBoxLayout *vXLayout = new QVBoxLayout(parent);
    QVBoxLayout *legendLayout = new QVBoxLayout(parent);
    QVBoxLayout *exportLayout = new QVBoxLayout(parent);
    QGridLayout *exportSizeLayout = new QGridLayout(parent);

    //add add add add
    QHBoxLayout *hMainLayout = new QHBoxLayout(parent);
    QVBoxLayout *vLayout = new QVBoxLayout(parent);
    groupBoxY->setLayout(vYLayout);
    groupBoxX->setLayout(vXLayout);
    groupBoxLegend->setLayout(legendLayout);
    groupBoxExport->setLayout(exportLayout);
    groupBoxExportSize->setLayout(exportSizeLayout);
    vYLayout->addWidget(radioButtonYLog);
    vYLayout->addWidget(radioButtonYLin);
    vXLayout->addWidget(radioButtonXLog);
    vXLayout->addWidget(radioButtonXLin);
    legendLayout->addWidget(radioButtonLegendRight);
    legendLayout->addWidget(radioButtonLegendBottom);
    exportLayout->addWidget(groupBoxExportSize);
    exportSizeLayout->addWidget(labelX, 0, 0);
    exportSizeLayout->addWidget(labelY, 1, 0);
    exportSizeLayout->addWidget(lineEditXSize, 0, 1);
    exportSizeLayout->addWidget(lineEditYSize, 1, 1);
    exportLayout->addWidget(exportButton);
    vLayout->addWidget(groupBoxX);
    vLayout->addWidget(groupBoxY);
    vLayout->addWidget(groupBoxLegend);
    vLayout->addWidget(groupBoxExport);
    //vLayout->addSpacing(200);
    vLayout->addStretch(1);
    hMainLayout->addWidget(plot);
    hMainLayout->addLayout(vLayout);
    hMainLayout->setStretch(0, 300);
    hMainLayout->setStretch(1, 12);
    this->setLayout(hMainLayout);
    this->setBaseSize(500, 500);
    this->setMinimumSize(250, 382);
    this->setSizeIncrement(10, 1);

    //Set tool and status tip
    radioButtonYLog->setStatusTip(tr("Switch Y axis to logaritmic scale"));
    radioButtonYLog->setToolTip(radioButtonYLog->statusTip());
    radioButtonYLin->setStatusTip(tr("Switch Y axis to linear scale"));
    radioButtonYLin->setToolTip(radioButtonYLin->statusTip());
    radioButtonXLog->setStatusTip(tr("Switch X axis to logaritmic scale"));
    radioButtonXLog->setToolTip(radioButtonXLog->statusTip());
    radioButtonXLin->setStatusTip(tr("Switch X axis to linear scale"));
    radioButtonXLin->setToolTip(radioButtonXLin->statusTip());
    radioButtonLegendRight->setStatusTip(tr("Move legend to right hand side of plot"));
    radioButtonLegendRight->setToolTip(radioButtonLegendRight->statusTip());
    radioButtonLegendBottom->setStatusTip(tr("Move legend to bottom of plot"));
    radioButtonLegendBottom->setToolTip(radioButtonLegendBottom->statusTip());
    lineEditXSize->setStatusTip(tr("Size in pixels in X direction of exported image"));
    lineEditXSize->setToolTip(lineEditXSize->statusTip());
    lineEditYSize->setStatusTip(tr("Size in pixels in T direction of exported image"));
    lineEditYSize->setToolTip(lineEditYSize->statusTip());
    exportButton->setStatusTip(tr("Export plot as image"));
    exportButton->setToolTip(exportButton->statusTip());
    saveActPng->setStatusTip(tr("Save as PNG bitmap image file"));
    saveActPng->setToolTip(saveActPng->statusTip());
    saveActSvg->setStatusTip(tr("Save as SVG scalable vector graphics file"));
    saveActSvg->setToolTip(saveActSvg->statusTip());

    lineEditXSize->setValidator(new QIntValidator(0, 10000, this));
    lineEditYSize->setValidator(new QIntValidator(0, 10000, this));

    connect(radioButtonYLog, SIGNAL(clicked()), this, SLOT(on_radioButtonYLog_clicked()));
    connect(radioButtonYLin, SIGNAL(clicked()), this, SLOT(on_radioButtonYLin_clicked()));
    connect(radioButtonXLog, SIGNAL(clicked()), this, SLOT(on_radioButtonXLog_clicked()));
    connect(radioButtonXLin, SIGNAL(clicked()), this, SLOT(on_radioButtonXLin_clicked()));
    connect(radioButtonLegendRight, SIGNAL(clicked()), this, SLOT(on_radioButtonLegendRight_clicked()));
    connect(radioButtonLegendBottom, SIGNAL(clicked()), this, SLOT(on_radioButtonLegendBottom_clicked()));
    connect(saveActPng, SIGNAL(triggered()), this, SLOT(on_menuActionSaveAsPng_triggered()));
    connect(saveActSvg, SIGNAL(triggered()), this, SLOT(on_menuActionSaveAsSvg_triggered()));



    QSettings settings(COMPANYNAME, PRODUCTNAME);
    lineEditXSize->setText(settings.value("ExportImageSizeX", 800).toString());
    lineEditYSize->setText(settings.value("ExportImageSizeY", 600).toString());
}

/** Save settings for later */
void XYPlotArea::saveSettings(QString folderPathSave)
{
    QSettings settings(COMPANYNAME, PRODUCTNAME);
    settings.setValue("ExportImageSizeX", lineEditXSize->text());
    settings.setValue("ExportImageSizeY", lineEditYSize->text());
    if (folderPathSave.compare("") != 0) {
        settings.setValue("SaveImageFolderPath", folderPathSave);
    }
}

/** Returns pointer to XYPlot */
XYPlot * XYPlotArea::Plot()
{
    return plot;
}

/** Slot called when radioButtonYLog is clicked, set scale engine to Log10 for Y Axis */
void XYPlotArea::on_radioButtonYLog_clicked(){
    qDebug() << "XYPlotArea::on_radioButtonYLog_clicked()";
    QwtLog10ScaleEngine * qloge = new QwtLog10ScaleEngine;
    plot->setAxisScaleEngine(QwtPlot::yLeft, qloge);
}

/** Slot called when radioButtonYLin is clicked, set scale engine to Lin for Y Axis */
void XYPlotArea::on_radioButtonYLin_clicked(){
    qDebug() << "XYPlotArea::on_radioButtonYLin_clicked()";
    QwtLinearScaleEngine * qline = new QwtLinearScaleEngine;
    plot->setAxisScaleEngine(QwtPlot::yLeft, qline);
}

/** Slot called when radioButtonXLog is clicked, set scale engine to Log10 for X Axis */
void XYPlotArea::on_radioButtonXLog_clicked(){
    qDebug() << "XYPlotArea::on_radioButtonXLog_clicked()";
    QwtLog10ScaleEngine * qloge = new QwtLog10ScaleEngine;
    plot->setAxisScaleEngine(QwtPlot::xBottom, qloge);
}

/** Slot called when radioButtonXLin is clicked, set scale engine to Lin for X Axis */
void XYPlotArea::on_radioButtonXLin_clicked(){
    qDebug() << "XYPlotArea::on_radioButtonXLin_clicked()";
    QwtLinearScaleEngine * qline = new QwtLinearScaleEngine;
    plot->setAxisScaleEngine(QwtPlot::xBottom, qline);
}

/** Slot called when radioButtonLegendRight is clicked, move legend to the right */
void XYPlotArea::on_radioButtonLegendRight_clicked() {
    qDebug() << "XYPlotArea::on_radioButtonLegendRight_clicked()";
    plot->insertLegend(new QwtLegend(), QwtPlot::RightLegend);
}

/** Slot called when radioButtonLegendBottom is clicked, move legend to the bottom */
void XYPlotArea::on_radioButtonLegendBottom_clicked() {
    qDebug() << "XYPlotArea::on_radioButtonLegendBottom_clicked()";
    plot->insertLegend(new QwtLegend(), QwtPlot::BottomLegend);
}

void XYPlotArea::on_menuActionSaveAsPng_triggered()
{
    QString selectedFilter;
    QSettings settings(COMPANYNAME, PRODUCTNAME);
    QDir saveDir(settings.value("SaveImageFolderPath", QDir::currentPath()).toString());
    QString fileName = QFileDialog::getSaveFileName(this, tr("Save file as"), saveDir.absolutePath(),
                                     tr("PNG Image Files (*.png);;JPEG Image Files (*.jpg);;BMP Image Files (*.bmp);;All Files (*)"), &selectedFilter);

    if ( !fileName.isEmpty() ) {
        QDir saveDirectory(fileName);
        saveSettings(saveDirectory.absolutePath());
        QPixmap pixmap(QVariant(lineEditXSize->text()).toInt(), QVariant(lineEditYSize->text()).toInt());
        pixmap.fill(Qt::white); // Qt::transparent ?

        QwtPlotPrintFilter filter;
        int options = QwtPlotPrintFilter::PrintAll;
        options &= ~QwtPlotPrintFilter::PrintBackground;
        options |= QwtPlotPrintFilter::PrintFrameWithScales;
        filter.setOptions(options);

        plot->print(pixmap, filter);

        if ( pixmap.save(fileName, "png" )) {
            qDebug()<<"File saved successfully";
        } else {
            QMessageBox::information(this, tr("QMessageBox::information()"), QString("File save failed!\nFile: %1").arg(fileName));
            qDebug()<<"File save failed!";
        }
    }
}

void XYPlotArea::on_menuActionSaveAsSvg_triggered()
{
    QString selectedFilter;
    QSettings settings(COMPANYNAME, PRODUCTNAME);
    QDir saveDir(settings.value("SaveImageFolderPath", QDir::currentPath()).toString());
    QString fileName = QFileDialog::getSaveFileName(this, tr("Save file as"), saveDir.absolutePath(),
                                     tr("SVG Image Files (*.svg);;All Files (*)"), &selectedFilter);

    if ( !fileName.isEmpty() ) {
        QDir saveDirectory(fileName);
        saveSettings(saveDirectory.absolutePath());

        QSvgGenerator generator;
        generator.setFileName(fileName);
        generator.setSize(QSize(QVariant(lineEditXSize->text()).toInt(), QVariant(lineEditYSize->text()).toInt()));
        QwtPlotPrintFilter filter;
        int options = QwtPlotPrintFilter::PrintAll;
        options |= QwtPlotPrintFilter::PrintBackground;
        options |= QwtPlotPrintFilter::PrintFrameWithScales| QwtPlotPrintFilter::PrintMargin;
        filter.setOptions(options);

        plot->print(generator);
    }
}


