/*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version.
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* Copyright (C) 2010 Marius Stene (marius.stene at gmail dot com)
*
*/

#include "calculateregimeshared.h"

CalculateRegimeShared::CalculateRegimeShared()
{
qDebug() << "CalculateRegimeShared::CalculateRegimeShared()";
}

CalculateRegimeShared::~CalculateRegimeShared()
{
qDebug() << "CalculateRegimeShared::~CalculateRegimeShared()";
}

/** Workhorse function which takes in pointer to function in with which a correct variable should be found, used by all four regimes
*   @param data Flow data
*   @param frictionFunctions Pointer to class with pointer to functions
*   @param evalFunction Pointer to function with which to evaluate the variable, takes an array of doubles as argument allowing it to pass as many arguments as it wants to
*   @see voidAnnular Function to calculate void fraction for annular flow, can be passed as argument as evalFunction
*   @see alphaDispersed Function to calculate alpha for dispersed flow, can be passed as argument as evalFunction
*   @see alphaStatified Function to calculate alpha for stratified flow, can be passed as argument as evalFunction
*   @see voidBubble Function to calculate void fraction for bubble in slug flow, can be passed as argument as evalFunction
*   @param args pointer to double array of arguments to pass to evalFunction
*   @param nArg number of arguments in args array
*   @param tolerance numerical error tolerance
*   @param x initial value, should be center value
*/
double CalculateRegimeShared::bisectvoid(Data *data, FrictionFunctions *frictionFunctions,
                        double (*evalFunction)(Data*, FrictionFunctions*, double variable, double *args, int nArg),
                        double *args, int nArg, double tolerance, double x)
{
    qDebug() << "CalculateRegimeShared::bisectvoid()";
    double dxMultiplier = 1.3;
    //problem: velger ikke konsekvent stabil lsning. nskelig at en returnerer den lsning med minst holdup.
    //double dx = x / (data.diameter()/2 + 0.01);
    double dx = x/10;
    double a = x - dx;
    double fa, fb;
    fa = evalFunction(data, frictionFunctions, a, args, nArg);
    double b = x + dx;
    fb = evalFunction(data, frictionFunctions, b, args, nArg);

    dx=x/100;

    //% Finner skift i fortegn
    while ((fa > 0) == (fb > 0)) {
        qDebug() << "inside loop CalculateRegimeShared::bisectvoid n1" << fa << ":" << fb;
        qDebug() << "a=" << a << ", b=" << b;
        dx = dxMultiplier*dx;
        a = x - dx;
        if (a > 1.0)
            a = 0.9999;
        if (a < 0.0)
            a = 0.0001;
        fa = evalFunction(data, frictionFunctions, a, args, nArg);

        if ((fa > 0) != (fb > 0))
            break;

        b = x + dx;
        if (b > 1.0)
            b = 0.9999;
        if (b < 0.0)
            b = 0.0001;
        fb = evalFunction(data, frictionFunctions, b, args, nArg);
    }

    //if trace, steps=[a fa; b fb],end
    //% hovedlkke
    double fc;
    while (fabs(b-a) > 2.0 * tolerance * std::max(fabs(b), 1.0)) {
        double c = a + 0.5 * (b-a);
        fc = evalFunction(data, frictionFunctions, c, args, nArg);
        qDebug() << "inside loop CalculateRegimeShared::bisectvoid n2" << fa << ":" << fb << ":" << fc;
        qDebug() << "inside loop CalculateRegimeShared::bisectvoid n2" << a << ":" << b << ":" << c;
        //if trace, steps=[steps; [c fc]];,end
        if ((fb > 0) == (fc > 0)) {
            b = c;
            fb = fc;
        } else {
            a = c;
            fa = fc;
        }
        qDebug() << a << b << c;
    }
    return b;
}

/** Calculates reynolds number
*   @param density Density
*   @param hdiameter Hydraulic diameter
*   @param velocity Speed of transport
*   @param viscosity Viscosity (Pa*s)
*/
double CalculateRegimeShared::reynolds(double density, double hdiameter, double velocity, double viscosity)
{
    qDebug() << "CalculateRegimeShared::reynolds()";
    if (viscosity <= 0 || viscosity > 2000000000)
        return 0.0;

    if (velocity < 0)
        velocity = -velocity;
    double re = density*velocity*hdiameter/viscosity;
    //qDebug() << "returning = " << re;
    return re;
}

/** Calculate hydraulic diameter
*   @param alpha Alpha value to use
*   @param area Area
*   @param surface Surface length
*/
double CalculateRegimeShared::hydraulicDiameter(double alpha, double area, double surface)
{
    qDebug() << "CalculateRegimeShared::hydraulicDiameter()";
    if (surface < 0 || surface > 2000000000)
        return 0.0;

    double dh = (4*alpha*area)/surface;
    //qDebug() << "returning = " << dh;
    return dh;
}

/** Calculates the rising velocity  of gas bubbles in dispersed/slug flow
*   @param alpha Alpha value
*   @param surfaceTension Surface Tension
*   @param densityLiquid Density of liquid phase
*   @param densityGas Density of gas phase
*   @param angle Pipe angle
*/
double CalculateRegimeShared::relativeVelocityRisingBubbles(double alpha, double surfaceTension, double densityLiquid, double densityGas, double angle)
{
    qDebug() << "CalculateRegimeDispersed::relativeVelocityRisingBubbles()";
    double re = 1.18 * pow(9.81*surfaceTension*(densityLiquid-densityGas)/pow(densityLiquid, 2), 0.25) * pow((1-alpha)*fabs(sin(angle*PI/180)), 0.5);
    //qDebug() << "returning=" << re;
    return re;
}
