/**
 * @file
 * @author  Marius Stene <marius.stene at gmail.com>
 * @version 1.0
 *
 * @section LICENSE
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * @section DESCRIPTION
 * Class which handles the data storage
 */

#ifndef DATA_H
#define DATA_H

#include <QDebug>
#include <QDataStream>
#include <QStringList>
#include <QTextStream>
#include <QVariant>
#include <math.h>
#include <iostream>

#include "../MultiPhase/src/defines.h"

//Used as argument when telling what the variable is
enum variable_t {
    VARIABLE_USG = 0,
    VARIABLE_USL = 1,
    VARIABLE_LIQUIDDENSITY = 2,
    VARIABLE_LIQUIDVISCOSITY = 3,
    VARIABLE_GASDENSITY = 4,
    VARIABLE_GASVISCOSITY = 5,
    VARIABLE_GASTEMPERATURE = 6,
    VARIABLE_GASPRESSURE = 7,
    VARIABLE_SURFACETENSION = 8,
    VARIABLE_ANGLE = 9,
    VARIABLE_DIAMETER = 10,
    VARIABLE_ROUGHNESS = 11
};

enum regimetype_t {
    REGIME_ANNULAR = 0,
    REGIME_DISPERSED = 1,
    REGIME_SLUG = 2,
    REGIME_STRATIFIED = 3,
    //Just some extra regimes not used currently
    REGIME_CHURN = 4,
    REGIME_PLUG = 5,
    REGIME_BUBBLE = 6,
    REGIME_WISPY_ANNULAR = 7,
    REGIME_STRATIFIED_WAVE = 8,
    REGIME_ANNULAR_DISPERSED = 9,
    //Not a real regime, just a clue to use the minimum holdup of all the regimes
    REGIME_DYNAMIC_MINIMUM_HOLDUP = 10
};

class Data {
public:
    Data();
    Data(const Data *copyData);
    bool equals(Data *compareWith) const;
    //Data(Data &copyData);

    inline double alpha() const { return _alpha;}
    inline double angle() const { return _angle;}
    inline double angleRad() const { return (_angle * PI / 180); }
    inline double viscosityLiquid() const { return _viscosityLiquid;}
    inline double viscosityGas() const { return _viscosityGas;}
    inline double densityLiquid() const { return _densityLiquid;}
    inline double densityGas() const { return _densityGas;}
    inline double pressureGas() const { return _pressureGas;}
    inline double temperatureGas() const { return _temperatureGas;}
    inline double surfaceTension() const { return _surfaceTension;}
    inline double Usl() const { return _Usl;}
    inline double Usg() const { return _Usg;}
    inline double diameter() const { return _diameter;}
    inline double roughness() const { return _roughness;}
    inline double variable() const { return *_variable; }
    inline double variableMax() const { return _variableMax; }
    inline double variableMin() const { return _variableMin; }
    inline regimetype_t regimeType() const { return _regimeType; }
    inline variable_t variableType() const { return _variableType; }


    inline double slipRelationParameter() const { if (angle() > 60) { return 1.0; } else { return 0.95; } };

    inline QString alphaName() const { return QString("Alpha"); }
    inline QString angleName() const { return QString("Angle");}
    inline QString viscosityLiquidName() const { return QString("Liquid Viscosity");}
    inline QString viscosityGasName() const { return QString("Gas Viscosity");}
    inline QString densityLiquidName() const { return QString("Liquid Density");}
    inline QString densityGasName() const { return QString("Gas Density");}
    inline QString pressureGasName() const { return QString("Gas Pressure");}
    inline QString temperatureGasName() const { return QString("Gas Temperature");}
    inline QString surfaceTensionName() const { return QString("Surface Tension");}
    inline QString UslName() const { return QString("Usl");}
    inline QString UsgName() const { return QString("Usg");}
    inline QString diameterName() const { return QString("Diameter");}
    inline QString roughnessName() const { return QString("Roughness");}
    inline QString regimeName() const { return _regimeName; }
    inline QString variableName() const { return _variableName; }
    QString regimeNameShort(int length) const;
    inline QString regimeTransitionName() const { return _regimeTransitionName; }

    inline double Ul() const { return _Usl/(1-_alpha); }
    inline double Ug() const { return _Usg/_alpha; }
    inline double area() const { return 3.1415*_diameter*_diameter/4; };
    //FIX
    double reynoldsLiquid();
    inline double reynoldsGas() {return densityGas()*Ug()*hdiameterGas()/viscosityGas();}
    inline double hdiameterLiquid() { return 1.0; }
    inline double hdiameterGas() { return 1.0; }
    inline double hdiameter() { return 1.0; }

    void setAlpha(double alpha);
    void setVariable(double variable);
    void setAngle(double angle);
    void setViscosityLiquid(double viscosityLiquid);
    void setViscosityGas(double viscosityGas);
    void setDensityLiquid(double densityLiquid);
    void setDensityGas(double densityGas);
    void setPressureGas(double pressureGas);
    void setTemperatureGas(double temperatureGas);
    void setSurfaceTension(double surfaceTension);
    inline void setUsl(double Usl) { _Usl = Usl; };
    void setUsg(double Usg);
    void setDiameter(double diameter);
    void setRoughness(double roughness);

    void setVariableMin(double variableMin);
    void setVariableMax(double variableMax);
    void setVariableType(variable_t variableType);
    void setRegimeType(regimetype_t regimeType);
    void setRegimeTransitionName(QString regimeTransitionName);

    void writeData(QTextStream &out, bool flow = true) const;
    static Data * readData(QString dataStream, bool flow = true);

    QString dataInformation() const;

private:
    double _alpha;
    double _angle;
    double _viscosityLiquid;
    double _viscosityGas;
    double _densityLiquid;
    double _densityGas;
    double _pressureGas;
    double _temperatureGas;
    double _surfaceTension;
    double _Usl;
    double _Usg;
    double _diameter;
    double _roughness;
    double *_variable;
    double _variableMin;
    double _variableMax;
    QString _variableName;
    QString _regimeTransitionName;
    variable_t _variableType;
    QString _regimeName;
    regimetype_t _regimeType;
};

#endif // DATA_H

#ifndef QT_NO_DEBUG_STREAM
QDebug operator<<(QDebug dbg, const Data &d);
#endif
