/*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License as
* published by the Free Software Foundation; either version 2 of
* the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* General Public License for more details at
* http://www.gnu.org/copyleft/gpl.html
*
* Author: Marius Stene
* Email: marius.stene at gmail dot com
*
*/

#include "barnea.h"

Barnea::Barnea()
{
qDebug() << "Barnea::Barnea()";
}

Barnea::~Barnea()
{
qDebug() << "Barnea::~Barnea()";
}

double Barnea::calculateRegimeTransition(Data *data, FrictionFunctions *frictionFunctions, CalculateFunctions *calculateFunctions, double tolerance, double Usg)
{
    qDebug() << "Barnea::calculateRegimeTransition()";
    double lowerbound = 0.0;
    double higherbound = 100*Usg;
    double alphaAnnular;
    double Y, Z;
    double UslTemp;
    data->setVariable(Usg);
    while ((higherbound - lowerbound) > tolerance) {
        UslTemp = 0.5*(lowerbound+higherbound);
        data->setUsl(UslTemp);
        alphaAnnular = calculateFunctions->pluginAnnular()->calculateAlpha(data, frictionFunctions, tolerance);
        double holdup = 1-alphaAnnular;
        qWarning() << "WELCOMEHERE holdup:" << holdup << " lowerbound:" << lowerbound << " higherbound:" << higherbound << " Usl:" << data->Usl() << " Usg:" << data->Usg();
        trans(data, frictionFunctions, alphaAnnular, Y, Z);
        if (Y >= Z) {
            //slug flow detected
            higherbound = UslTemp;
        } else {
            lowerbound = UslTemp;
        }
    }
    UslTemp = 0.5*(lowerbound+higherbound);
    return UslTemp;
}

regimetype_t Barnea::calculateRegime(Data *data, FrictionFunctions *frictionFunctions, CalculateFunctions *calculateFunctions, double tolerance, double Usg, double Usl)
{
    double transitionUsl = calculateRegimeTransition(data, frictionFunctions, calculateFunctions, tolerance, Usg);
    if (transitionUsl > Usl) {
        return REGIME_ANNULAR;
    } else {
        return REGIME_SLUG;
    }
}

void Barnea::trans(Data *data, FrictionFunctions *frictionFunctions, double alpha, double &Y, double &Z)
{
    //local fluid velocities
    double Ul = data->Usl()/(1-alpha);
    double Ug = data->Usg()/alpha;

    double surfaceLiquid;
    double surfaceInterface;
    double thicknessFilm;
    geometryAnnular(alpha, data->diameter(), surfaceLiquid, surfaceInterface, thicknessFilm);
    //hydraulic diameter for each specified phase
    double hdiameterLiquid = CalculateRegimeShared::hydraulicDiameter(1-alpha, data->area(), surfaceLiquid);
    double hdiameterGas = data->diameter()-2*thicknessFilm;
    //reynolds number for each specified phase
    double reynoldsLiquid = CalculateRegimeShared::reynolds(data->densityLiquid(), hdiameterLiquid, Ul, data->viscosityLiquid());
    double reynoldsGas = CalculateRegimeShared::reynolds(data->densityGas(), hdiameterGas, Ug, data->viscosityGas());
    //friction
    double frictionFactorWallLiquid = frictionFunctions->pluginWallLiquid()->calculateLiquidFrictionFactor(*data, reynoldsLiquid, hdiameterLiquid);
    double frictionFactorWallGas = frictionFunctions->pluginWallGas()->calculateGasFrictionFactor(*data, reynoldsGas, hdiameterGas);
    //calcs
    double holdup = 1-alpha;
    double X = (frictionFactorWallLiquid * data->densityLiquid() * data->Usl())/(frictionFactorWallGas*data->densityGas()*data->Usg());
    Y =(1+75)*holdup/( pow((1-holdup), 2.5) * holdup)-(1/pow(holdup, 3))*X;
    Z =(2-3/2)*holdup/(pow(holdup, 3)*(1-(3/2)*holdup))*X;
}

/** The function evaluates the characteristic geometrical parameters for annular flow
*   @param alpha Alpha value
*   @param diameter Effective diameter
*   @param surfaceLiquid Length of interface between liquid phase and wall
*   @param surfaceInterface Length of interface between liquid phase and gas phase
*   @param thicknessFilm Thickness of film
*/
void Barnea::geometryAnnular(double alpha, double diameter,
                            double &surfaceLiquid, double &surfaceInterface, double &thicknessFilm)
{
    qDebug() << "Barnea::geometryAnnular()";
    //momentary length of circular surface between liquid film and tube wall
    surfaceLiquid = diameter*PI;
    //length of circular interfase surface between gas- and liquid phase.
    surfaceInterface = diameter*PI*pow(alpha, 0.5);
    //thickness of liquid film
    //From appendix 6
    thicknessFilm = (diameter/2)*(1-pow(alpha, 0.5));

    qDebug() << "returning = surfaceLiquid =" << surfaceLiquid << ", surfaceInterface =" << surfaceInterface << ", thicknessFilm =" << thicknessFilm;
}

QString Barnea::pluginName() const
{
    qDebug() << "Barnea::pluginName()";
    return QString("&Barnea");
}

QString Barnea::transitionDescription() const
{
    qDebug() << "Barnea::transitionDescription()";
    return QString("Annular - Slug");
}

QString Barnea::tooltip() const
{
    return QString("");
}

QString Barnea::statustip() const
{
    return QString("");
}

bool Barnea::transitionAnnular() const
{
    return true;
}

bool Barnea::transitionStratified() const
{
    return false;
}

bool Barnea::transitionDispersed() const
{
    return false;
}

bool Barnea::transitionSlug() const
{
    return true;
}

Q_EXPORT_PLUGIN2(../../../application/MultiPhase/debug/lib/regimetransition/barnea, Barnea);
