/*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License as
* published by the Free Software Foundation; either version 2 of
* the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* General Public License for more details at
* http://www.gnu.org/copyleft/gpl.html
*
* Author: Marius Stene
* Email: marius.stene at gmail dot com
*
*/

#include "barneabubble.h"

BarneaBubble::BarneaBubble()
{
qDebug() << "BarneaBubble::BarneaBubble()";
}

BarneaBubble::~BarneaBubble()
{
qDebug() << "BarneaBubble::~BarneaBubble()";
}

double BarneaBubble::calculateRegimeTransition(Data *data, FrictionFunctions *frictionFunctions, CalculateFunctions *calculateFunctions, double tolerance, double Usg)
{
    qDebug() << "BarneaBubble::calculateRegimeTransition()";
    double lowerbound = 0.0;
    double higherbound = 20*Usg;
    double UslTemp;
    double alphaDisp;
    double dBubble, dBubbleB, dBubbleD, dBubbleMin;
    data->setVariable(Usg);
    while ((higherbound - lowerbound) > tolerance) {
        UslTemp = 0.5*(lowerbound+higherbound);
        data->setUsl(UslTemp);
        alphaDisp = calculateFunctions->pluginDispersed()->calculateAlpha(data, frictionFunctions, tolerance);
        dBubble = bubbleSize(data);
        dBubbleB = BarneaBuoyancyCriterion(data, frictionFunctions, alphaDisp);
        dBubbleD = BarneaDeformationCriterion(data);
        //Get lowest of B and D
        (dBubbleD < dBubbleB) ? dBubbleMin = dBubbleD : dBubbleMin = dBubbleB;
        if (dBubble <= dBubbleMin) {
            //slug flow initiated
            higherbound = UslTemp;
        } else {
            lowerbound = UslTemp;
        }
    }
    UslTemp= 0.5*(lowerbound+higherbound);
    return UslTemp;
}

regimetype_t BarneaBubble::calculateRegime(Data *data, FrictionFunctions *frictionFunctions, CalculateFunctions *calculateFunctions, double tolerance, double Usg, double Usl)
{
    double transitionUsl = calculateRegimeTransition(data, frictionFunctions, calculateFunctions, tolerance, Usg);
    if (transitionUsl > Usl) {
        return REGIME_SLUG;
    } else {
        return REGIME_DISPERSED;
    }
}

double BarneaBubble::BarneaBuoyancyCriterion(Data *data, FrictionFunctions *frictionFunctions, double alpha)
{
    qDebug() << "BarneaBubble::BarneaBuoyancyCriterion()";
    double Ul = data->Usl()/(1-alpha);
    double Ug = data->Usg()/alpha;
    double hdiameter = CalculateRegimeShared::hydraulicDiameter(alpha, data->area(), data->diameter()*PI);
    double reynolds = CalculateRegimeShared::reynolds(data->densityLiquid(), hdiameter, Ul+Ug, data->viscosityLiquid());
    double frictionFactorMixed = frictionFunctions->pluginDispersed()->calculateDispersedFrictionFactor(*data, alpha, reynolds, hdiameter, Ul+Ug, data->viscosityLiquid());
    //critical bubble diameter due to gravity
    double bubbleBuoyancy=0.375*(data->densityLiquid()/(data->densityLiquid()-data->densityGas()))*((frictionFactorMixed*pow(data->Usg()+data->Usl(), 2))/(GRAVITY*cos(data->angleRad())));
    qWarning() << "Ul=" << Ul << ", Ug =" << Ug << ", hdiameter =" << hdiameter << ", reynolds =" << reynolds << ", frictionFactorMixed =" << frictionFactorMixed << ", bubbleBuoyancy =" << bubbleBuoyancy;
    qWarning() << "cos(data->angleRad()) = " << cos(data->angleRad()) << ", data->angleRad() = " << data->angleRad()<< ", data->densityLiquid() = " << data->densityLiquid() << ", data->densityGas() = " << data->densityGas() << ", pow(data->Usg()+data->Usl(), 2) = "<< pow(data->Usg()+data->Usl(), 2);
    return bubbleBuoyancy;
}

double BarneaBubble::BarneaDeformationCriterion(Data *data)
{
    qDebug() << "BarneaBubble::BarneaDeformationCriterion()";
    double bubbleDeformation = 2 * pow((0.4*data->surfaceTension()/((data->densityLiquid()-data->densityGas())*GRAVITY)), 0.5);
    return bubbleDeformation;
}

double BarneaBubble::bubbleSize(Data *data)
{
    qDebug() << "BarneaBubble::bubbleSize()";
    //double densityAverage = alpha * data->densityGas() + (1-alpha) * data->densityLiquid();
    //double reynolds = CalculateRegimeShared::reynolds(densityAverage, data->diameter(), (data->Usg() + data->Usl()), data->viscosityLiquid());
    //double frictionFactorMixed = frictionFunctions->pluginDispersed()->calculateDispersedFrictionFactor(*data, alpha, reynolds, data->diameter(), (data->Usg() + data->Usl()), data->viscosityLiquid());
    //rate of energy dissipation per unit mass
    //double kappa = (2*frictionFactorMixed/data->diameter()) * pow((data->Usg() + data->Usl()), 3);
    //critical bubble diameter
    double bubbleCrit = 8 * data->surfaceTension() * data->Usg()/(0.046*pow(data->diameter() * data->Usl() / (data->viscosityLiquid()/data->densityLiquid()), -0.2)*data->densityLiquid()*pow(data->Usl(),3));
    return bubbleCrit;
}

QString BarneaBubble::pluginName() const
{
    qDebug() << "BarneaBubble::pluginName()";
    return QString("Bar&nea Bubble Size");
}

QString BarneaBubble::transitionDescription() const
{
    qDebug() << "BarneaBubble::transitionDescription()";
    return QString("Dispersed - Slug");
}

QString BarneaBubble::tooltip() const
{
    return QString("Barnea Bubble Size compares critical deformation bubble size and critical bubble migration to upper part of pipeline size.\n\nD. Barnea, A unified model for predicting flow-pattern transitions for the whole range of pipe inclinations, International Journal of Multiphase Flow, Volume 13, Issue 1, January-February 1987");
}

QString BarneaBubble::statustip() const
{
    return QString("Barnea Bubble Size compares critical deformation bubble size and critical bubble migration to upper part of pipeline size.");
}

bool BarneaBubble::transitionAnnular() const
{
    return false;
}

bool BarneaBubble::transitionStratified() const
{
    return false;
}

bool BarneaBubble::transitionDispersed() const
{
    return true;
}

bool BarneaBubble::transitionSlug() const
{
    return true;
}

Q_EXPORT_PLUGIN2(../../../application/MultiPhase/debug/lib/regimetransition/barneabubble, BarneaBubble);
