/*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License as
* published by the Free Software Foundation; either version 2 of
* the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* General Public License for more details at
* http://www.gnu.org/copyleft/gpl.html
*
* Author: Marius Stene
* Email: marius.stene at gmail dot com
*
*/

#include "defaultcalculateregimeannular.h"


DefaultCalculateRegimeAnnular::DefaultCalculateRegimeAnnular()
{
    qDebug() << "DefaultCalculateRegimeAnnular::DefaultCalculateRegimeAnnular()";
}

DefaultCalculateRegimeAnnular::~DefaultCalculateRegimeAnnular()
{
    qDebug() << "DefaultCalculateRegimeAnnular::~DefaultCalculateRegimeAnnular()";
}

double DefaultCalculateRegimeAnnular::calculateAlpha(Data *data, FrictionFunctions *frictionFunctions, double tolerance)
{
    qDebug() << "DefaultCalculateRegimeAnnular::calculateAlpha()";
    int nArgs = 2;
    double *args = new double[nArgs];
    args[0] = data->Usl();
    args[1] = data->Usg();
    double diameterGas = CalculateRegimeShared::bisectvoid(data, frictionFunctions, diameterGasAnnular, args, nArgs, tolerance, data->diameter()/2);
    delete [] args;
    double alpha = pow(diameterGas/data->diameter(), 2);
    qDebug() << "returning =: " << alpha;
    return alpha;
}

QList<QPair<QString, double> > DefaultCalculateRegimeAnnular::calculate(Data *data, FrictionFunctions *frictionFunctions, double tolerance)
{
    double alpha = calculateAlpha(data, frictionFunctions, tolerance);
    QList<QPair<QString, double> > list;
    list.append(qMakePair(QString("Alpha"), alpha));
    return list;
}

/** Calculate pressure loss in annular flow
*   @param data Flow data
*   @param frictionFunctions Pointer to class with pointer to functions
*/
double DefaultCalculateRegimeAnnular::calculatePressureDrop(Data *data, FrictionFunctions *frictionFunctions, double tolerance)
{
    qDebug() << "DefaultCalculateRegimeAnnular::calculatePressureDrop()";
    double frictionWall;
    double frictionInterface;
    double frictionGravityLiquid;
    double frictionGravityGas;
    frictionAnnular(data, frictionFunctions, data->Usl(), data->Usg(), data->alpha(), frictionWall, frictionInterface, frictionGravityLiquid, frictionGravityGas);
    double re = - (frictionWall + (1-data->alpha()) * frictionGravityLiquid + data->alpha() * frictionGravityGas);
    qDebug() << "returning =: " << re;
    return re;
}

QString DefaultCalculateRegimeAnnular::pluginName() const
{
    qDebug() << "DefaultCalculateRegimeAnnular::pluginName()";
    return QString("Default Annular");
}

bool DefaultCalculateRegimeAnnular::isAnnular() const
{
    qDebug() << "DefaultCalculateRegimeAnnular::isAnnular()";
    return true;
}

bool DefaultCalculateRegimeAnnular::isStratified() const
{
    qDebug() << "DefaultCalculateRegimeAnnular::isStratified()";
    return false;
}

bool DefaultCalculateRegimeAnnular::isDispersed() const
{
    qDebug() << "DefaultCalculateRegimeAnnular::isDispersed()";
    return false;
}

bool DefaultCalculateRegimeAnnular::isSlug() const
{
    qDebug() << "DefaultCalculateRegimeAnnular::isSlug()";
    return false;
}

/** Function meant to be passed as argument evalFunction to bisectvoid, finds the correct void fraction for annular flow
*   @param data Flow data
*   @param frictionFunctions Pointer to class with pointer to functions
*   @param diameterGas Diameter of gas phase guess
*   @param args pointer to double array of arguments to pass to evalFunction
*   @param nArg number of arguments in args array
*/
double DefaultCalculateRegimeAnnular::diameterGasAnnular(Data *data, FrictionFunctions *frictionFunctions, double diameterGas, double *args, int nArg)
{
    qDebug() << "DefaultCalculateRegimeAnnular::diameterGasAnnular";
    if (nArg != 2) {
        qDebug() << "incorrect number of arguments passed, bailing out!";
        return -1.0;
    }
    double Usl = args[0];
    double Usg = args[1];

    double alpha = pow((diameterGas/data->diameter()), 2);
    double H = 1 - alpha;
    //% @calculation:     Wall friction, interphase friction and gravity friction for the representative phases.
    double frictionWall;
    double frictionInterface;
    double frictionGravityLiquid;
    double frictionGravityGas;
    frictionAnnular(data, frictionFunctions, Usl, Usg, alpha, frictionWall, frictionInterface, frictionGravityLiquid, frictionGravityGas);

    //calculation:     voidfraction iterated, void_annular=0!!!
    qDebug() << "frictionWall =" << frictionWall << ", frictionInterface =" << frictionInterface << ", alpha=" << alpha;
    qDebug() << "(frictionGravityLiquid-frictionGravityGas) =" << frictionGravityLiquid << "-" << frictionGravityGas;
    double re = -frictionWall/H + frictionInterface*((1/H) + (1/alpha)) + (frictionGravityLiquid-frictionGravityGas);
    qDebug() << "returning =" << re;
    return re;
}

/** Calculate friction in annular flow
*   @param data Flow data
*   @param frictionFunctions Pointer to class with pointer to functions
*   @param Usl Superficial velocity liquid phase
*   @param Usg Superficial velocity gas phase
*   @param frictionWall Friction caused by wall
*   @param frictionInterface Friction caused by interface
*   @param frictionGravityLiquid Friction caused by gravity on liquid phase
*   @param frictionGravityGas Friction caused by gravity on gas phase
*/
void DefaultCalculateRegimeAnnular::frictionAnnular(Data *data, FrictionFunctions *frictionFunctions, double Usl, double Usg, double alpha,
                                       double &frictionWall, double &frictionInterface, double &frictionGravityLiquid, double &frictionGravityGas)
{
    qDebug() << "DefaultCalculateRegimeAnnular::frictionAnnular()";
    //@calculation:     local fluid velocities
    //@calculation:     length of circular interphase between phases and length of
    //                  circular contact surface between liquid phase and tube wall.
    double surfaceLiquid;
    double surfaceInterface;
    double thicknessFilm;
    double Ul = Usl / (1-alpha);
    double Ug = Usg/alpha;
    geometryAnnular(alpha, data->diameter(), surfaceLiquid, surfaceInterface, thicknessFilm);

    //% @calculation:     hydraulic diameter for each specified phase
    double hdiameterLiquid = CalculateRegimeShared::hydraulicDiameter(1-alpha, data->area(), surfaceLiquid);
    double hdiameterGas = data->diameter()-2*thicknessFilm;

    //% @calculation:     reynolds number for each specified phase
    qDebug() << data << "Ul=" << Ul;
    double reynoldsLiquid = CalculateRegimeShared::reynolds(data->densityLiquid(), hdiameterLiquid, Ul, data->viscosityLiquid());
    double reynoldsGas = CalculateRegimeShared::reynolds(data->densityGas(), hdiameterGas, Ug, data->viscosityGas());

    //% @call:            switch chosen wall friction model for liquid phase
    //% @calculation:     friction coefficient for annular flow.
    qDebug() << "reynoldsLiquid=" << reynoldsLiquid;
    qDebug() << data;
    qDebug() << "thicknessFilm=" << thicknessFilm << "hdiameterGas" << hdiameterGas;
    double frictionFactorWallLiquid = frictionFunctions->pluginWallLiquid()->calculateLiquidFrictionFactor(*data, reynoldsLiquid, hdiameterLiquid);
    double frictionFactorWallGas = frictionFunctions->pluginWallGas()->calculateGasFrictionFactor(*data, reynoldsGas, hdiameterGas);
    double frictionFactorInterface = frictionFunctions->pluginInterface()->calculateFrictionFactor(*data, alpha, reynoldsGas, hdiameterGas, thicknessFilm, frictionFactorWallGas);

    //Flow friction.
    qDebug() << "data->densityLiquid()=" << data->densityLiquid()<< ", frictionFactorWallLiquid=" << frictionFactorWallLiquid;
    qDebug() << "Ul=" << Ul << ", surfaceLiquid=" << surfaceLiquid <<  ", data->area()=" << data->area();
    qDebug() << "frictionFactorInterface=" << frictionFactorInterface << ", fabs(Ug-Ul)=" << fabs(Ug-Ul);
    qDebug() << "surfaceInterface=" << surfaceInterface;
    qDebug() << "(1/2)*data->densityLiquid()*frictionFactorWallLiquid*Ul*Ul*surfaceLiquid/(4*data->area())=" << 0.5*data->densityLiquid()*frictionFactorWallLiquid*Ul*Ul*surfaceLiquid/(4*data->area());
    frictionWall = 0.5*data->densityLiquid()*frictionFactorWallLiquid*fabs(Ul)*Ul*surfaceLiquid/(4*data->area());
    frictionInterface = 0.5*data->densityLiquid()*frictionFactorInterface*fabs(Ug-Ul)*(Ug-Ul)*surfaceInterface/(4*data->area());

    //Gravity friction on liquid and gas phase
    frictionGravityLiquid=data->densityLiquid()*GRAVITY*sin(data->angleRad());
    frictionGravityGas=data->densityGas()*GRAVITY*sin(data->angleRad());
    qDebug() << "returning = frictionWall =" << frictionWall << ", frictionInterface =" << frictionInterface;
    qDebug() << "\t\tfrictionGravityLiquid =" << frictionGravityLiquid << ", frictionGravityGas =" << frictionGravityGas;
}

/** The function evaluates the characteristic geometrical parameters for annular flow
*   @param alpha Alpha value
*   @param diameter Effective diameter
*   @param surfaceLiquid Length of interface between liquid phase and wall
*   @param surfaceInterface Length of interface between liquid phase and gas phase
*   @param thicknessFilm Thickness of film
*/
void DefaultCalculateRegimeAnnular::geometryAnnular(double alpha, double diameter,
                            double &surfaceLiquid, double &surfaceInterface, double &thicknessFilm)
{
    qDebug() << "DefaultCalculateRegimeAnnular::geometryAnnular()";
    //momentary length of circular surface between liquid film and tube wall
    surfaceLiquid = diameter*PI;
    //length of circular interfase surface between gas- and liquid phase.
    surfaceInterface = diameter*PI*pow(alpha, 0.5);
    //thickness of liquid film
    //From appendix 6
    thicknessFilm = (diameter/2)*(1-pow(alpha, 0.5));

    qDebug() << "returning = surfaceLiquid =" << surfaceLiquid << ", surfaceInterface =" << surfaceInterface << ", thicknessFilm =" << thicknessFilm;
}

Q_EXPORT_PLUGIN2(../../../application/MultiPhase/debug/lib/calculate/defaultcalculateregimeannular, DefaultCalculateRegimeAnnular);
