/*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License as
* published by the Free Software Foundation; either version 2 of
* the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* General Public License for more details at
* http://www.gnu.org/copyleft/gpl.html
*
* Author: Marius Stene
* Email: marius.stene at gmail dot com
*
*/

#include "defaultcalculateregimedispersed.h"


DefaultCalculateRegimeDispersed::DefaultCalculateRegimeDispersed()
{
    qDebug() << "DefaultCalculateRegimeDispersed::DefaultCalculateRegimeDispersed()";
}

DefaultCalculateRegimeDispersed::~DefaultCalculateRegimeDispersed()
{
    qDebug() << "DefaultCalculateRegimeDispersed::~DefaultCalculateRegimeDispersed()";
}

double DefaultCalculateRegimeDispersed::calculateAlpha(Data *data, FrictionFunctions *frictionFunctions, double tolerance)
{
    qDebug() << "DefaultCalculateRegimeDispersed::calculateAlpha()";
    int nArgs = 2;
    double *args = new double[nArgs];
    args[0] = data->Usl();
    args[1] = data->Usg();
    double alpha = CalculateRegimeShared::bisectvoid(data, frictionFunctions, alphaDispersed, args, nArgs, tolerance, data->diameter()/2);
    delete [] args;
    qDebug() << "returning =: " << alpha;
    return alpha;
}

QList<QPair<QString, double> > DefaultCalculateRegimeDispersed::calculate(Data *data, FrictionFunctions *frictionFunctions, double tolerance)
{
    double alpha = calculateAlpha(data, frictionFunctions, tolerance);
    QList<QPair<QString, double> > list;
    list.append(qMakePair(QString("Alpha"), alpha));
    return list;
}

/** Calculate pressure loss in dispersed flow
*   @param data Flow data
*   @param frictionFunctions Pointer to class with pointer to functions
*/
double DefaultCalculateRegimeDispersed::calculatePressureDrop(Data *data, FrictionFunctions *frictionFunctions, double tolerance)
{
    qDebug() << "DefaultCalculateRegimeDispersed::calculatePressureDrop()";
    double UlDisp;
    double UgDisp;
    velocityDispersed(data, frictionFunctions, UlDisp, UgDisp);

    qDebug() << "data->alpha()" << data->alpha();
    //@calculation:     average density of gas/liquid mixture in dispersed flow
    double densityAverage = data->alpha()*data->densityGas() + (1-data->alpha())*data->densityLiquid();
    //D_hs=hydraulic_dfun(alfa_s, pi*data->diameter);
    double reynoldsNumber = CalculateRegimeShared::reynolds(densityAverage, data->diameter(), UlDisp, data->viscosityLiquid());
    //Get friction
    double fricFactor = frictionFunctions->pluginDispersed()->calculateDispersedFrictionFactor(*data, data->alpha(), reynoldsNumber, data->diameter(), UlDisp, data->viscosityLiquid());
    qDebug() << "fricFactor=" << fricFactor;
    //@calculation:     rising velocity of gas bubbles in dispersed flow.
    double URel = CalculateRegimeShared::relativeVelocityRisingBubbles(data->alpha(), data->surfaceTension(), data->densityLiquid(), data->densityGas(), data->angle());
    qDebug() << "URel=" << URel;
    //@calculation:     'two-phase multiplicator' due to interphase action
    double fricIndicator = 1 + 15.3 * data->alpha() * URel / (UlDisp*(1-data->alpha()));
    qDebug() << "fricIndicator=" << fricIndicator;
    //@calculation:     total pressure drop over dispersed regime
    double dpdxDisp = fricFactor * data->densityLiquid() * (1-data->alpha()) * fabs(UlDisp) * UlDisp * fricIndicator/(2*data->diameter());
    qDebug() << "dpdxDisp=" << dpdxDisp;
    //%*********************Pressure drop due to gravity forces*************************************************************************
    //% @calculation:     total pressure drop due to acceleration forces
    double dpdxGrav=GRAVITY*sin(data->angleRad()) * (data->alpha() * data->densityGas() + (1-data->alpha()) * data->densityLiquid());
    //% @calculation:     total pressure drop over a characteristic slug unit
    double dpdxSlug=-(dpdxDisp+dpdxGrav);
    qDebug() << "dpdxDisp=" << dpdxDisp << " dpdxGrav=" << dpdxGrav << " angle=" << data->angle() << " sin" << sin(data->angleRad()) << " data->angleRad()" << data->alpha();
    qDebug() << "returning=" << dpdxSlug;
    return dpdxSlug;
}

QString DefaultCalculateRegimeDispersed::pluginName() const
{
    qDebug() << "DefaultCalculateRegimeDispersed::pluginName()";
    return QString("Default Dispersed");
}

bool DefaultCalculateRegimeDispersed::isAnnular() const
{
    qDebug() << "DefaultCalculateRegimeDispersed::isAnnular()";
    return false;
}

bool DefaultCalculateRegimeDispersed::isDispersed() const
{
    qDebug() << "DefaultCalculateRegimeDispersed::isDispersed()";
    return true;
}

bool DefaultCalculateRegimeDispersed::isStratified() const
{
    qDebug() << "DefaultCalculateRegimeDispersed::isStratified()";
    return false;
}

bool DefaultCalculateRegimeDispersed::isSlug() const
{
    qDebug() << "DefaultCalculateRegimeDispersed::isSlug()";
    return false;
}

/** Function meant to be passed as argument evalFunction to bisectvoid, finds the correct alpha value for dispersed flow
*   @param data Flow data
*   @param frictionFunctions Pointer to class with pointer to functions
*   @param alpha Alpha value to try, returns a corrected value
*   @param args pointer to double array of arguments, for this function should be Usl and Usg
*   @param nArg number of arguments in args array, should be 2, otherwise function will bail out
*/
double DefaultCalculateRegimeDispersed::alphaDispersed(Data *data, FrictionFunctions *frictionFunctions, double alpha, double *args, int nArg)
{
    qDebug() << "DefaultCalculateRegimeDispersed::alphaDispersed()";
    if (nArg != 2) {
        qDebug() << "incorrect number of arguments passed, bailing out!";
        return -1.0;
    }
    double Usl = args[0];
    double Usg = args[1];

    double re = frictionFunctions->pluginSlipRelation()->calculateAlpha(*data, alpha, Usl, Usg);
    qDebug() << "returning: " << re;
    return re;
}

/** Calculates liquid and gas velocities in dispersed flow
*   @param data Flow data
*   @param frictionFunctions Pointer to class with pointer to functions
*   @param UlDisp Reference to double to which where liquid velocity should be saved
*   @param UgDisp Reference to double to which where gas velocity should be saved
*/
void DefaultCalculateRegimeDispersed::velocityDispersed(const Data *data, FrictionFunctions *frictionFunctions, double &UlDisp, double &UgDisp)
{
    qDebug() << "DefaultCalculateRegimeDispersed::velocityDispersed";
    UlDisp = frictionFunctions->pluginSlipRelation()->calculateLiquidVelocity(*data, data->Usl(), data->Usg(), data->alpha());
    UgDisp = frictionFunctions->pluginSlipRelation()->calculateGasVelocity(*data, UlDisp, data->alpha());
}


Q_EXPORT_PLUGIN2(../../../application/MultiPhase/debug/lib/calculate/defaultcalculateregimedispersed, DefaultCalculateRegimeDispersed);
