/*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License as
* published by the Free Software Foundation; either version 2 of
* the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* General Public License for more details at
* http://www.gnu.org/copyleft/gpl.html
*
* Author: Marius Stene
* Email: marius.stene at gmail dot com
*
*/

#include "defaultcalculateregimeslug.h"


DefaultCalculateRegimeSlug::DefaultCalculateRegimeSlug()
{
    qDebug() << "DefaultCalculateRegimeSlug::DefaultCalculateRegimeSlug()";
    _data = NULL;
}

DefaultCalculateRegimeSlug::~DefaultCalculateRegimeSlug()
{
    qDebug() << "DefaultCalculateRegimeSlug::~DefaultCalculateRegimeSlug()";
}

double DefaultCalculateRegimeSlug::calculateAlpha(Data *data, FrictionFunctions *frictionFunctions, double tolerance)
{
    qDebug() << "DefaultCalculateRegimeSlug::calculateAlpha()";
    slugUnit(data, frictionFunctions, tolerance, 0.5,
                        _alphaSlug, _alphaBubble, _UlSlug,
                        _velocityBubbleFront,
                        _UlInBubble, _UgInBubble,
                        _slugFraction, _slugLength, _bubbleLength);
    double holdup = _slugFraction*(1-_alphaSlug)+(1-_slugFraction)*(1-_alphaBubble);
    double alpha = 1 - holdup;
    _data = data;
    //Save for later to see if we actually need to recalculate slugUnit for pressureLoss
    _data->setAlpha(alpha);
    _tolerance = tolerance;
    _frictionFunctions = frictionFunctions;

    qDebug() << "returning =" << alpha;
    return alpha;
}

QList<QPair<QString, double> > DefaultCalculateRegimeSlug::calculate(Data *data, FrictionFunctions *frictionFunctions, double tolerance)
{
    double alpha = calculateAlpha(data, frictionFunctions, tolerance);
    QList<QPair<QString, double> > list;
    list.append(qMakePair(QString("Alpha"), alpha));
    list.append(qMakePair(QString("SlugFraction"), _slugFraction));
    list.append(qMakePair(QString("AlphaSlug"), _alphaSlug));
    list.append(qMakePair(QString("AlphaBubble"), _alphaBubble));
    list.append(qMakePair(QString("SlugLength"), _slugLength));
    list.append(qMakePair(QString("UlInBubble"), _UlInBubble));
    list.append(qMakePair(QString("UgInBubble"), _UgInBubble));
    list.append(qMakePair(QString("UlSlug"), _UlSlug));
    list.append(qMakePair(QString("UgInBubble"), _UgInBubble));
    return list;
}

/** Calculate pressure loss in slug flow
*   @param data Flow data
*   @param frictionFunctions Pointer to class with pointer to functions
*/
double DefaultCalculateRegimeSlug::calculatePressureDrop(Data *data, FrictionFunctions *frictionFunctions, double tolerance)
{
    qDebug() << "DefaultCalculateRegimeSlug::calculatePressureDrop()";
    if (!data->equals(_data) || _tolerance != tolerance || !frictionFunctions->equals(_frictionFunctions)) {
        slugUnit(data, frictionFunctions, tolerance, 0.5,
                            _alphaSlug, _alphaBubble, _UlSlug,
                            _velocityBubbleFront,
                            _UlInBubble, _UgInBubble,
                            _slugFraction, _slugLength, _bubbleLength);
    }
    double dpdx = dpdxSlug(data, frictionFunctions, _alphaSlug, _alphaBubble, _UlSlug, _velocityBubbleFront,
                                       _UlInBubble, _UgInBubble, _slugFraction, _slugLength, _bubbleLength);
    qDebug() << "returning =" << dpdx;
    return dpdx;
}

QString DefaultCalculateRegimeSlug::pluginName() const
{
    qDebug() << "DefaultCalculateRegimeSlug::pluginName()";
    return QString("Default Slug");
}

bool DefaultCalculateRegimeSlug::isAnnular() const
{
    qDebug() << "DefaultCalculateRegimeSlug::isAnnular()";
    return false;
}

bool DefaultCalculateRegimeSlug::isStratified() const
{
    qDebug() << "DefaultCalculateRegimeSlug::isStratified()";
    return false;
}

bool DefaultCalculateRegimeSlug::isDispersed() const
{
    qDebug() << "DefaultCalculateRegimeSlug::isDispersed()";
    return false;
}

bool DefaultCalculateRegimeSlug::isSlug() const
{
    qDebug() << "DefaultCalculateRegimeSlug::isSlug()";
    return true;
}


/** Calculates the total pressure drop along a modelled slug unit.
*   @param data Flow data
*   @param frictionFunctions Pointer to class with pointer to functions
*   @param alphaSlug Alpha value for slug
*   @param alphaBubble Alpha value for bubble
*   @param UlSlug Velocity of liquid in slug
*   @param velocityBubbleFront Velocity of front of bubble
*   @param UlInBubble Liquid velocity in bubble
*   @param UgInBubble Gas velocity in bubble
*   @param slugFraction Slug fraction
*   @param slugLength Length of slug
*   @param bubbleLength Length of bubble
*/
double DefaultCalculateRegimeSlug::dpdxSlug(Data *data, FrictionFunctions *frictionFunctions, double alphaSlug,
                               double alphaBubble, double UlSlug,
                           double velocityBubbleFront, double UlInBubble, double UgInBubble, double slugFraction, double slugLength,
                           double bubbleLength)
{
    qDebug() << "DefaultCalculateRegimeSlug::dpdxSlug()";
    //Pressure drop over bubble
    //angle relating slug unit interfaces.
    double angleBubble = interfaceAngle(alphaBubble);
    //trigonometric calculations og geometrical properties of stratified flow.
    double surfaceLiquid;
    double surfaceGas;
    double surfaceInterface;
    geometry(data, angleBubble, surfaceLiquid, surfaceGas, surfaceInterface);
    //Hydraulic diameter
    double hdiameterLiquid = CalculateRegimeShared::hydraulicDiameter(1-alphaBubble, data->area(), surfaceLiquid);
    double hdiameterGas = CalculateRegimeShared::hydraulicDiameter(alphaBubble, data->area(), surfaceGas + surfaceInterface);
    //reynolds number
    double reynoldsLiquid = CalculateRegimeShared::reynolds(data->densityLiquid(), hdiameterLiquid, UlInBubble, data->viscosityLiquid());
    double reynoldsGas = CalculateRegimeShared::reynolds(data->densityGas(), hdiameterGas, UgInBubble, data->viscosityGas());
    //Friction factor gas and liquid
    double frictionFactorLiquid = frictionFunctions->pluginWallLiquid()->calculateLiquidFrictionFactor(*data, reynoldsLiquid, hdiameterLiquid);
    double frictionFactorGas = frictionFunctions->pluginWallGas()->calculateGasFrictionFactor(*data, reynoldsGas, hdiameterGas);
    //total pressure drop over bubble part of slug unit
    double dpdxBubble = (1-slugFraction) * (frictionFactorGas * data->densityGas() * fabs(UgInBubble) * UgInBubble * surfaceGas / (8*data->area()) + frictionFactorLiquid * data->densityLiquid() * fabs(UlInBubble) * UlInBubble * surfaceInterface / (8*data->area()));

    //Pressure drop over slug
    //average density of gas/liquid mixture in slug
    double densityAverage = alphaSlug * data->densityGas() + (1-alphaSlug) * data->densityLiquid();
    //Reynolds
    double reynoldsAverage = CalculateRegimeShared::reynolds(densityAverage, data->diameter(), UlSlug, data->viscosityLiquid());
    double fricFactorSlug = frictionFunctions->pluginDispersed()->calculateDispersedFrictionFactor(*data, alphaSlug, reynoldsAverage, data->diameter(), UlSlug, data->viscosityLiquid());
    //rising velocity of gas bubbles in slug flow.
    double URising = CalculateRegimeShared::relativeVelocityRisingBubbles(alphaSlug, data->surfaceTension(), data->densityLiquid(), data->densityGas(), data->angle());
    //'two-phase multiplicator' due to interphase action
    double kTwoPhase = pow((1+15.3*alphaSlug*(URising/(UlSlug*(1-alphaSlug)))), 0.5);
    // total pressure drop over liquid part of slug unit
    double dpdxSlug = slugFraction * (0.5 * fricFactorSlug * densityAverage * (1-alphaSlug) * fabs(UlSlug) * UlSlug * pow(kTwoPhase, 2) / data->diameter());

    //Pressure drop due to acceleration
    double dpdxAcceleration = data->densityLiquid() * (1-alphaSlug) * (velocityBubbleFront - UlSlug) * (UlSlug - UlInBubble) / (slugLength + bubbleLength);

    //Pressure drop due to gravity forces

    //overall void fraction for slug unit
    double alpha = slugFraction * alphaSlug + (1-slugFraction) * alphaBubble;

    //overall holdup for slug unit
    double holdup = slugFraction * (1-alphaSlug) + (1-slugFraction) * (1-alphaBubble);

    //total pressure drop due to acceleration forces
    double dpdxGrav = GRAVITY * sin(data->angleRad()) * (alpha * data->densityGas() + holdup * data->densityLiquid());

    //total pressure drop over a characteristic slug unit
    double dpdxTotal=-(dpdxSlug + dpdxBubble + dpdxAcceleration + dpdxGrav);
    qDebug() << "returning=" << dpdxTotal;
    return dpdxTotal;
}

/** The function calculates the characteristic variables neccessary to describe a slug unit.
*   @param data Flow data
*   @param frictionFunctions Pointer to class with pointer to functions
*   @param tolerance Numerical tolerance
*   @param x Default value (should be middle value)
*   @param alphaSlug Alpha value for slug
*   @param alphaBubble Alpha value for bubble
*   @param UlSlug Velocity of liquid
*   @param UgSlug Velocity of gas
*   @param velocityBubbleNeck Velocity of back of bubble
*   @param velocityBubbleFront Velocity of front of bubble
*   @param UlInBubble Velocity of liquid in bubble
*   @param UgInBubble Velocity of gas in bubble
*   @param slugFraction Slug fraction
*   @param slugLength Slug length
*   @param bubbleLength Bubble length
*/
void DefaultCalculateRegimeSlug::slugUnit(Data *data, FrictionFunctions *frictionFunctions, double tolerance, double x,
                         double &alphaSlug, double &alphaBubble, double &UlSlug,
                         double &velocityBubbleFront,
                         double &UlInBubble, double &UgInBubble,
                         double &slugFraction, double &slugLength, double &bubbleLength)
{
    qDebug() << "DefaultCalculateRegimeSlug::slugUnit()";
    double velocityBubbleNeck;
    //void fraction of slug (gas fraction)
    alphaSlug = alphaCalc(data->Usl(), data->Usg());
    //gas- and liquid velocities of slug
    UlSlug = frictionFunctions->pluginSlipRelation()->calculateLiquidVelocity(*data, data->Usl(), data->Usg(), alphaSlug);
    double UgSlug = frictionFunctions->pluginSlipRelation()->calculateGasVelocity(*data, UlSlug, alphaSlug);
    //bubble velocities of slug flow
    double velocityFlow, velocityProduction;
    bubbleGasVelocity(data, velocityFlow, velocityProduction, velocityBubbleFront, velocityBubbleNeck);
    int nArgs = 4;
    double *args = new double[nArgs];
    args[0] = alphaSlug;
    args[1] = velocityBubbleNeck;
    args[2] = UlSlug;
    args[3] = UgSlug;
    alphaBubble = CalculateRegimeShared::bisectvoid(data, frictionFunctions, voidBubble, args, nArgs, tolerance, x);
    velocitiesInBubble(alphaSlug, alphaBubble, UlSlug, UgSlug, velocityBubbleNeck, UlInBubble, UgInBubble);
    slugGeometry(data->diameter(), alphaSlug, alphaBubble, data->Usl(), UlInBubble, velocityBubbleNeck, slugFraction, slugLength, bubbleLength);
    delete [] args;
}

/** The function calculates the gas velocity in the bubble, and the liquid velocity in the film.
*   @param data Flow data
*   @param velocityFlow Velocity of  liquid in the film.
*   @param velocityProduction Don't know, not used
*   @param velocityBubbleFront Velocity of front of bubble
*   @param velocityBubbleNeck Velocity of back of bubble
*/
void DefaultCalculateRegimeSlug::bubbleGasVelocity(Data *data, double &velocityFlow, double &velocityProduction, double &velocityBubbleFront, double &velocityBubbleNeck)
{
    qDebug() << "DefaultCalculateRegimeSlug::bubbleGasVelocity()";
    //Don't know what these are for
    double Uov = 0.35 * pow((GRAVITY * data->diameter()), 0.5); // * U_osf;
    double Uoh = 0.54 * pow((GRAVITY * data->diameter()), 0.5); // * U_osf;

    double Um = data->Usg() + data->Usl();
    //production velocity
    double velocityProduction1 = 1.05 + 0.15 * pow(sin(data->angleRad()), 2);
    double velocityProduction2 = 1.2;
    //Flow velocity
    double velocityFlow1 = Uov * sin(data->angleRad()) + Uoh * cos(data->angleRad());
    double velocityFlow2 = Uov * sin(data->angleRad());

    double velocityBubbleFront1 = velocityProduction1 * Um + velocityFlow1;
    double velocityBubbleFront2 = velocityProduction2 * Um + velocityFlow2;

    //velocity of bubble front.
    if (velocityBubbleFront1 > velocityBubbleFront2) {
        velocityBubbleFront = velocityBubbleFront1;
        velocityProduction = velocityProduction1;
        velocityFlow = velocityFlow1;
    } else {
        velocityBubbleFront = velocityBubbleFront2;
        velocityProduction = velocityProduction2;
        velocityFlow = velocityFlow2;
    }
    //velocity of bubble neck.
    velocityBubbleNeck = velocityBubbleFront;
}

/** Calculates the gas and liquid velocity in the bubble.
*   @param alphaSlug Alpha value for slug
*   @param alphaBubble Alpha value for bubble
*   @param UlSlug Velocity of liquid
*   @param UgSlug Velocity of gas
*   @param velocityBubbleNeck Velocity of back of bubble
*   @param UlInBubble Velocity of liquid in bubble
*   @param UgInBubble Velocity of gas in bubble
*/
void DefaultCalculateRegimeSlug::velocitiesInBubble(double alphaSlug, double alphaBubble, double UlSlug, double UgSlug, double velocityBubbleNeck,
                                   double &UlInBubble, double &UgInBubble)
{
    qDebug() << "DefaultCalculateRegimeSlug::velocitiesInBubble()";
    UlInBubble = velocityBubbleNeck - ((1-alphaSlug) / (1-alphaBubble)) * (velocityBubbleNeck - UlSlug);
    UgInBubble = velocityBubbleNeck - (alphaSlug/alphaBubble) * (velocityBubbleNeck - UgSlug);
}

/** Calculates the gas-fraction (often denoted as void-fraction) of the slug
*   Theory: Model of alphaCalcSlug is developed by Pauchon/Gregory
*   @param Usl Superficial liquid velocity
*   @param Usg Superficial gas velocity
*/
double DefaultCalculateRegimeSlug::alphaCalc(double Usl, double Usg)
{
    qDebug() << "DefaultCalculateRegimeSlug::alphaCalc()";
    double re = 1 - pow((1 + fabs(Usg + Usl)/8.66), -1.39);
    if (re > 0.7) {
        re = 0.7;
    }
    qDebug() << "returning =" << re;
    return re;
}

/** Calculates the geometric variables neccessary to describe a slug unit
*   @param diameter Diameter of pipe
*   @param alphaSlug Alpha value for slug
*   @param alphaBubble Alpha value for bubble
*   @param Usl Superficial liquid velocity
*   @param UlInBubble Liquid velocity in bubble
*   @param velocityBubbleNeck Velocity of back of bubble
*   @param slugFraction Slug fraction
*   @param slugLength Length of slug
*   @param bubbleLength Length of bubble
*/
void DefaultCalculateRegimeSlug::slugGeometry(double diameter, double alphaSlug, double alphaBubble, double Usl, double UlInBubble, double velocityBubbleNeck,
                             double &slugFraction, double &slugLength, double &bubbleLength)
{
    qDebug() << "DefaultCalculateRegimeSlug::slugGeometry()";
    //relative slug fraction over control unit.
    slugFraction = (Usl - UlInBubble * (1-alphaBubble)) / (velocityBubbleNeck * (alphaBubble-alphaSlug));
    //slug length
    slugLength = 15*diameter;
    //bubble length
    bubbleLength = slugLength * (1-slugFraction)/slugFraction;
}

/** Calculates the void fraction of bubble in slug flow
*   @param data Flow data
*   @param frictionFunctions Pointer to class with pointer to functions
*   @param alphaBubble Void fraction guess
*   @param args pointer to double array of arguments to pass to evalFunction
*   @param nArg number of arguments in args array
*/
double DefaultCalculateRegimeSlug::voidBubble(Data *data, FrictionFunctions *frictionFunctions, double alphaBubble, double *args, int nArg)
{
    qDebug() << "DefaultCalculateRegimeSlug::voidBubble()";
    if (nArg != 4) {
        qDebug() << "incorrect number of arguments passed, bailing out!";
        return -1.0;
    }
    data->setAlpha(alphaBubble);
    double alphaSlug = args[0];
    double velocityBubbleNeck = args[1];
    double UlSlug = args[2];
    double UgSlug = args[3];
    //trigonometric approximation of angel in stratified flow.
    double angleBubble = interfaceAngle(alphaBubble);
    //trigonometric calculations og geometrical properties of stratified flow.
    double surfaceLiquid;
    double surfaceGas;
    double surfaceInterface;
    geometry(data, angleBubble, surfaceLiquid, surfaceGas, surfaceInterface);
    //flowing velocity of the liquid and then gas in bubbleunit of slug flow
    double UlInBubble, UgInBubble;
    velocitiesInBubble(alphaSlug, alphaBubble, UlSlug, UgSlug, velocityBubbleNeck, UlInBubble, UgInBubble);
    //Hydraulic diameter
    double hdiameterLiquid = CalculateRegimeShared::hydraulicDiameter(1-alphaBubble, data->area(), surfaceLiquid);
    double hdiameterGas = CalculateRegimeShared::hydraulicDiameter(alphaBubble, data->area(), surfaceGas + surfaceInterface);
    //reynolds number
    double reynoldsLiquid = CalculateRegimeShared::reynolds(data->densityLiquid(), hdiameterLiquid, UlInBubble, data->viscosityLiquid());
    double reynoldsGas = CalculateRegimeShared::reynolds(data->densityGas(), hdiameterGas, UgInBubble, data->viscosityGas());
    //Friction Factor
    double frictionFactorLiquid = frictionFunctions->pluginWallLiquid()->calculateLiquidFrictionFactor(*data, reynoldsLiquid, hdiameterLiquid);
    double frictionFactorGas = frictionFunctions->pluginWallGas()->calculateGasFrictionFactor(*data, reynoldsGas, hdiameterGas);
    double frictionFactorInterface = frictionFunctions->pluginInterface()->calculateFrictionFactor(*data, data->alpha(), reynoldsGas, hdiameterGas, 0, frictionFactorGas);

    //friction force film
    double FrictionLiquid = (surfaceLiquid / (8*data->area())) * data->densityLiquid() * frictionFactorLiquid * fabs(UlInBubble) * UlInBubble;
    //friction force bubble
    double FrictionGas = (surfaceGas / (8*data->area())) * data->densityGas() * frictionFactorGas * fabs(UgInBubble) * UgInBubble;
    //friction force bubble
    double FrictionInterface = (surfaceInterface / (8*data->area())) * data->densityGas() * frictionFactorInterface * fabs(UgInBubble - UlInBubble) * (UgInBubble - UlInBubble);
    //gravity force gas
    double FrictionGravGas = alphaBubble * data->densityGas() * GRAVITY * sin(data->angleRad());
    //gravity force liquid
    double FrictionGravLiquid = (1-alphaBubble) * data->densityLiquid() * GRAVITY * sin(data->angleRad());
    //holdup of gas bubbles in slug flow.
    double re = (1-alphaBubble) * FrictionGravGas - alphaBubble * FrictionGravLiquid - FrictionLiquid * alphaBubble + (1-alphaBubble) * FrictionGas + FrictionInterface;
    qDebug() << "returning=" << re;
    return re;
}

/** The function evaluates the characteristic geometrical parameters describing interfaces in two-phase flow.
*   @param data Flow data
*   @param angleInterface Interface angle
*   @param surfaceLiquid Length of interface between liquid phase and wall
*   @param surfaceGas Length of interface between gas phase and wall
*   @param surfaceInterface Length of interface between liquid phase and gas phase
*/
void DefaultCalculateRegimeSlug::geometry(Data *data, double angleInterface, double &surfaceLiquid, double &surfaceGas, double &surfaceInterface)
{
    qDebug() << "DefaultCalculateRegimeSlug::geometryStratified()";
    qDebug() << "angleInterface=" << angleInterface;
    surfaceLiquid = angleInterface*PI/180 * data->diameter();
    surfaceGas = PI * data->diameter() - surfaceLiquid;
    surfaceInterface = data->diameter() * sin(angleInterface*PI/180);
}

/** The function calculates the angle between the radii of the pipeline and the interface.
*   @param alpha Alpha value to calculate angle based on
*/
double DefaultCalculateRegimeSlug::interfaceAngle(double alpha)
{
    qDebug() << "DefaultCalculateRegimeSlug::interfaceAngle()2";
    qDebug() << "alpha = " << alpha;
    double re = PI * (1 - alpha) + pow((3*PI/2), 0.3333333333) * (1 - 2 * (1 - alpha) + pow(1-alpha, 0.3333333333) - pow(alpha, 0.3333333333));
    re = re*180/PI;
    qDebug() << "returning: " << re;
    return re;
}

Q_EXPORT_PLUGIN2(../../../application/MultiPhase/debug/lib/calculate/defaultcalculateregimeslug, DefaultCalculateRegimeSlug);
