/*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License as
* published by the Free Software Foundation; either version 2 of
* the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* General Public License for more details at
* http://www.gnu.org/copyleft/gpl.html
*
* Author: Marius Stene
* Email: marius.stene at gmail dot com
*
*/

#include "defaultcalculateregimestratified.h"


DefaultCalculateRegimeStratified::DefaultCalculateRegimeStratified()
{
    qDebug() << "DefaultCalculateRegimeStratified::DefaultCalculateRegimeStratified()";
}

DefaultCalculateRegimeStratified::~DefaultCalculateRegimeStratified()
{
    qDebug() << "DefaultCalculateRegimeStratified::~DefaultCalculateRegimeStratified()";
}

double DefaultCalculateRegimeStratified::calculateAlpha(Data *data, FrictionFunctions *frictionFunctions, double tolerance)
{
    qDebug() << "DefaultCalculateRegimeStratified::calculateAlpha()";
    double alpha = CalculateRegimeShared::bisectvoid(data, frictionFunctions, alphaStratified, NULL, 0, tolerance, 0.5);
    qDebug() << "DefaultCalculateRegimeStratified::calculateAlpha()::end";
    qDebug() << "returning =: " << alpha;
    return alpha;
}

QList<QPair<QString, double> > DefaultCalculateRegimeStratified::calculate(Data *data, FrictionFunctions *frictionFunctions, double tolerance)
{
    double alpha = calculateAlpha(data, frictionFunctions, tolerance);
    QList<QPair<QString, double> > list;
    list.append(qMakePair(QString("Alpha"), alpha));
    return list;
}

/** Calculate pressure loss in stratified flow
*   @param data Flow data
*   @param frictionFunctions Pointer to class with pointer to functions
*/
double DefaultCalculateRegimeStratified::calculatePressureDrop(Data *data, FrictionFunctions *frictionFunctions, double tolerance)
{
    qDebug() << "DefaultCalculateRegimeStratified::calculatePressureDrop()";
    double frictionLiquid;
    double frictionGas;
    double frictionInterface;
    double frictionGravityLiquid;
    double frictionGravityGas;
    frictionStratified(data, frictionFunctions, data->alpha(), frictionLiquid, frictionGas, frictionInterface, frictionGravityLiquid, frictionGravityGas);
    return -(frictionLiquid + frictionGas + frictionGravityLiquid + frictionGravityGas);
}

QString DefaultCalculateRegimeStratified::pluginName() const
{
    qDebug() << "DefaultCalculateRegimeStratified::pluginName()";
    return QString("Default Stratified");
}

bool DefaultCalculateRegimeStratified::isAnnular() const
{
    qDebug() << "DefaultCalculateRegimeStratified::isAnnular()";
    return false;
}

bool DefaultCalculateRegimeStratified::isStratified() const
{
    qDebug() << "DefaultCalculateRegimeStratified::isStratified()";
    return true;
}

bool DefaultCalculateRegimeStratified::isDispersed() const
{
    qDebug() << "DefaultCalculateRegimeStratified::isDispersed()";
    return false;
}

bool DefaultCalculateRegimeStratified::isSlug() const
{
    qDebug() << "DefaultCalculateRegimeStratified::isSlug()";
    return false;
}

/** Function meant to be passed as argument evalFunction to bisectvoid, finds the correct alpha value for stratified flow
*   @param data Flow data
*   @param frictionFunctions Pointer to class with pointer to functions
*   @param alpha Alpha value to evalute
*   @param args pointer to double array of arguments to pass to evalFunction
*   @param nArg number of arguments in args array
*/
double DefaultCalculateRegimeStratified::alphaStratified(Data *data, FrictionFunctions *frictionFunctions, double alpha, double *args, int nArg)
{
    qDebug() << "DefaultCalculateRegimeStratified::staticAlphaStratified()";
    if (nArg != 0) {
        qDebug() << "incorrect number of arguments passed, bailing out!";
        return -1.0;
    }
    double frictionLiquid;
    double frictionGas;
    double frictionInterface;
    double frictionGravityLiquid;
    double frictionGravityGas;
    frictionStratified(data, frictionFunctions, alpha, frictionLiquid, frictionGas, frictionInterface, frictionGravityLiquid, frictionGravityGas);
    qDebug() << "FRICTIONFU" << "frictionLiquid" << frictionLiquid << "frictionGas" << frictionGas << "frictionInterface" << frictionInterface;
    qDebug() << "frictionGravityLiquid" << frictionGravityLiquid << "frictionGravityGas" << frictionGravityGas;
    return (frictionInterface + (1-alpha) * frictionGas - alpha * frictionLiquid + (1-alpha) * frictionGravityGas - alpha * frictionGravityLiquid);
}

/** Calculate friction in stratified flow
*   @param data Flow data
*   @param frictionFunctions Pointer to class with pointer to functions
*   @param frictionLiquid Friction caused by liquid phase
*   @param frictionGas Friction caused by gas phase
*   @param frictionInterface Friction caused by interface
*   @param frictionGravityLiquid Friction caused by gravity on liquid phase
*   @param frictionGravityGas Friction caused by gravity on gas phase
*/
void DefaultCalculateRegimeStratified::frictionStratified(Data *data, FrictionFunctions *frictionFunctions, double alpha, double &frictionLiquid,
                                  double &frictionGas, double &frictionInterface, double &frictionGravityLiquid, double &frictionGravityGas)
{
    qDebug() << "DefaultCalculateRegimeStratified::frictionStratified()";
    double angleInterface = interfaceAngle(alpha);
    double surfaceLiquid;
    double surfaceGas;
    double surfaceInterface;
    double Ug = data->Usg()/alpha;
    double Ul = data->Usl()/(1-alpha);

    geometry(data, angleInterface, surfaceLiquid, surfaceGas, surfaceInterface);
    //Hydraulic diameter
    double hdiameterLiquid = CalculateRegimeShared::hydraulicDiameter(1-alpha, data->area(), surfaceLiquid);
    double hdiameterGas = CalculateRegimeShared::hydraulicDiameter(alpha, data->area(), surfaceInterface+surfaceGas);
    //Reynolds Number
    double reynoldsLiquid = CalculateRegimeShared::reynolds(data->densityLiquid(), hdiameterLiquid, Ul, data->viscosityLiquid());
    double reynoldsGas = CalculateRegimeShared::reynolds(data->densityGas(), hdiameterGas, Ug, data->viscosityGas());
    //Friction Factor
    double frictionFactorLiquid = frictionFunctions->pluginWallLiquid()->calculateLiquidFrictionFactor(*data, reynoldsLiquid, hdiameterLiquid);
    double frictionFactorGas = frictionFunctions->pluginWallGas()->calculateGasFrictionFactor(*data, reynoldsGas, hdiameterGas);
    double frictionFactorInterface = frictionFunctions->pluginInterface()->calculateFrictionFactor(*data, alpha, reynoldsGas, hdiameterGas, 0, frictionFactorGas);
    //Friction
    frictionLiquid = data->densityLiquid() * frictionFactorLiquid * fabs(Ul) * Ul * surfaceLiquid / (8*data->area());
    frictionGas = data->densityGas() * frictionFactorGas * fabs(Ug) * Ug * surfaceGas / (8*data->area());
    frictionInterface = data->densityGas() * frictionFactorInterface * fabs(Ug - Ul) * (Ug - Ul) * surfaceInterface / (8*data->area());
    //Friction due to gravity
    frictionGravityLiquid = (1-alpha) * data->densityLiquid() * GRAVITY * sin(data->angleRad());
    frictionGravityGas = alpha * data->densityGas() * GRAVITY * sin(data->angleRad());
}

/** The function evaluates the characteristic geometrical parameters describing interfaces in two-phase flow.
*   @param data Flow data
*   @param angleInterface Interface angle
*   @param surfaceLiquid Length of interface between liquid phase and wall
*   @param surfaceGas Length of interface between gas phase and wall
*   @param surfaceInterface Length of interface between liquid phase and gas phase
*/
void DefaultCalculateRegimeStratified::geometry(Data *data, double angleInterface, double &surfaceLiquid, double &surfaceGas, double &surfaceInterface)
{
    qDebug() << "DefaultCalculateRegimeStratified::geometryStratified()";
    qDebug() << "angleInterface=" << angleInterface;
    surfaceLiquid = angleInterface*PI/180 * data->diameter();
    surfaceGas = PI * data->diameter() - surfaceLiquid;
    surfaceInterface = data->diameter() * sin(angleInterface*PI/180);
}

/** The function calculates the angle between the radii of the pipeline and the interface.
*   @param alpha Alpha value to calculate angle based on
*/
double DefaultCalculateRegimeStratified::interfaceAngle(double alpha)
{
    qDebug() << "DefaultCalculateRegimeStratified::interfaceAngle()2";
    qDebug() << "alpha = " << alpha;
    double re = PI * (1 - alpha) + pow((3*PI/2), 0.3333333333) * (1 - 2 * (1 - alpha) + pow(1-alpha, 0.3333333333) - pow(alpha, 0.3333333333));
    re = re*180/PI;
    qDebug() << "returning: " << re;
    return re;
}


Q_EXPORT_PLUGIN2(../../../application/MultiPhase/debug/lib/calculate/defaultcalculateregimestratified, DefaultCalculateRegimeStratified);
