/*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License as
* published by the Free Software Foundation; either version 2 of
* the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* General Public License for more details at
* http://www.gnu.org/copyleft/gpl.html
*
* Author: Marius Stene
* Email: marius.stene at gmail dot com
*
*/

#include "kelvinhelmholz.h"

Kelvinhelmholz::Kelvinhelmholz()
{
qDebug() << "Kelvinhelmholz::Kelvinhelmholz()";
}

Kelvinhelmholz::~Kelvinhelmholz()
{
qDebug() << "Kelvinhelmholz::~Kelvinhelmholz()";
}

double Kelvinhelmholz::calculateRegimeTransition(Data *data, FrictionFunctions *frictionFunctions, CalculateFunctions *calculateFunctions, double tolerance, double Usg)
{
    qDebug() << "Kelvinhelmholz::calculateRegimeTransition()";
    double lowerbound = 0.0;
    double higherbound = 100*Usg;
    double alpha;
    double UgKH, Ug, UslTemp;
    while ((higherbound - lowerbound) > tolerance) {
        UslTemp = 0.5*(lowerbound+higherbound);
        data->setUsl(UslTemp);
        data->setVariable(Usg);
        alpha = calculateFunctions->pluginStratified()->calculateAlpha(data, frictionFunctions, tolerance);
        data->setAlpha(alpha);
        limitGasVelocity(data, Usg, alpha, data->angle(), UgKH, Ug);
        qDebug() << "AlphaUgKHUg" << alpha << UgKH <<  " " << Ug;
        if (Ug < UgKH) {
            lowerbound = UslTemp;
        } else {
            higherbound = UslTemp;
        }
    }
    qDebug() << "asdf2k3a alpha=" << alpha << " UgKH=" << UgKH << " Ug" << Ug << " Usg=" << Usg << " Usl=" << UslTemp;
    //sim_fig('update',n,'Kelvin-Helmholz',xrange);
    UslTemp= 0.5*(lowerbound+higherbound);
    return UslTemp;
}

regimetype_t Kelvinhelmholz::calculateRegime(Data *data, FrictionFunctions *frictionFunctions, CalculateFunctions *calculateFunctions, double tolerance, double Usg, double Usl)
{
    double transitionUsl = calculateRegimeTransition(data, frictionFunctions, calculateFunctions, tolerance, Usg);
    if (transitionUsl > Usl) {
        return REGIME_STRATIFIED;
    } else {
        return REGIME_SLUG;
    }
}


/** Calculates the limit of local gas
*   velocity, U_g, allowing stable stratified flow. The limit
*   values for gas velocity distinguishes between stable slug-
*   and stratified two-phase flow.
*/
void Kelvinhelmholz::limitGasVelocity(Data *data, double Usg, double alpha, double angle, double &UgKH, double &Ug)
{
    qDebug() << "Kelvinhelmholz::limitGasVelocity()";
    //local gas velocity
    Ug = Usg/alpha;
    //area of gas-phase
    double Ag = alpha * data->area();
    //characteristic angle of stratified flow.
    //double theta = calculateFunctions->pluginStratified()->nonStaticInterfaceAngle(alpha);
    double theta = interfaceAngle(alpha);
    qDebug() << "theta" << theta;
    //height of liquid phase
    double hl = data->diameter() * (1-cos(theta*PI/180))/2;
    //KH stability constant
    double C2 = 1 - (hl / data->diameter());
    //Differential change in area of liquid phase vs. height of liquid phase.
    double deltaAreaLiquidvsHeightLiquid = data->diameter() * sin(theta*PI/180);
    //KH limiting local gas-velocity.
    UgKH = C2 * pow((9.81 * (data->densityLiquid() - data->densityGas()) * Ag * cos(angle*PI/180) / (data->densityGas() * deltaAreaLiquidvsHeightLiquid)), 0.5);
}

/** The function calculates the angle between the radii of the pipeline and the interface.
*   @param alpha Alpha value to calculate angle based on
*/
double Kelvinhelmholz::interfaceAngle(double alpha)
{
    qDebug() << "Kelvinhelmholz::interfaceAngle()2";
    qDebug() << "alpha = " << alpha;
    double re = PI * (1 - alpha) + pow((3*PI/2), 0.3333333333) * (1 - 2 * (1 - alpha) + pow(1-alpha, 0.3333333333) - pow(alpha, 0.3333333333));
    re = re*180/PI;
    qDebug() << "returning: " << re;
    return re;
}

QString Kelvinhelmholz::pluginName() const
{
    qDebug() << "Kelvinhelmholz::pluginName()";
    return QString("&Kelvin Helmholz");
}

QString Kelvinhelmholz::transitionDescription() const
{
    qDebug() << "Kelvinhelmholz::transitionDescription()";
    return QString("Stratified - Slug");
}

QString Kelvinhelmholz::tooltip() const
{
    return QString("");
}

QString Kelvinhelmholz::statustip() const
{
    return QString("");
}

bool Kelvinhelmholz::transitionAnnular() const
{
    return false;
}

bool Kelvinhelmholz::transitionStratified() const
{
    return true;
}

bool Kelvinhelmholz::transitionDispersed() const
{
    return false;
}

bool Kelvinhelmholz::transitionSlug() const
{
    return true;
}

Q_EXPORT_PLUGIN2(../../../application/MultiPhase/debug/lib/regimetransition/kelvinhelmholz, Kelvinhelmholz);
